//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA rtlinux-scheduler executive declarations
//
// Project:	Yasa 2
// Author :	Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/rtlinux/kernel/include/yasa_rtlinux_scheduler.h,v 1.2 2003/01/22 17:02:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_RTLINUX_SCHEDULER_INCLUDE
#define	YASA_RTLINUX_SCHEDULER_INCLUDE

#include <yasa_rtlinux_sched.h>			// include default rtl scheduler

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize yasa scheduler extensions. This function will be called during InitModule of "rtl_sched.o"
// * set function calls
void inline yasa_initialize(YASA_SCHEDULER *sched, int cpu)
{
	YASA_ATTRIBUTES		attr;

	// initialize synchronization structure (contains behaviour of dynamic priorities)
	yasa_initsyncstruct();

	//printk("yasa_initialize entered\n");
	sched->cpu=cpu;
	sched->schedulerstart=sched->clock->gethrtime(sched->clock);
	sched->schedulerend=YASA_TIME_INFINITY;
	sched->nextschedulingtime=sched->schedulerstart;

	// get default scheduling parameters
	// priority of linux thread will be set later in calling function to -1
	yasa_pthread_sched_init( &(sched->rtl_linux_task), &(sched->rtl_linux_task.sched_param) );

	// load or select chosen scheduler
	// set function calls
	//sched->yasa_schedule=0;

	// ELLF extensions -->
	pthread_attr_init(&attr);										// construction of attr- and attr.sched object
	pthread_attr_setname_np(&attr, YASA_LINUX_THREAD_NAME);
	pthread_attr_setid_np( &attr, YASA_LINUX_THREAD_ID);
	yasa_pthread_create( &(sched->rtl_linux_task), &attr);			// set only the YASA attr to thread structure

	//sched->calcpreemptiontime			= 1;
	sched->yasa_startofthread 			= sched->clock->gethrtime(sched->clock);
	sched->yasa_startofthreadtick		= sched->yasa_startofthread;
	//sched->yasa_schedulerstart			= sched->yasa_startofthread;
	//sched->yasa_excludedthread   	  	= 0;						// no excluding thread

	// set default functions/definitions
	sched->calcpreemptiontime	= ortl_schedulingattr.calcpreemptiontime;
	sched->tick					= ortl_schedulingattr.tick;
	sched->schedule				= ortl_schedule;
	sched->yasa_schedulerattr	= &ortl_prioattr;
	YASA_SCHEDULERDATA(sched)=0;

	// do not insert linux task in rt task list !!!
	// It will be chosen automatically by Yasa if no thread was returned by the
	// scheduler
	//s -> rtl_tasks = &s->rtl_linux_task;		// this is done in calling function InitModule()
	sched->rtl_tasks = 0;


	// initialize logging functions
	yasa_logfileconstructor( sched, &sched->logfile );
	//yasa_initlogging( sched, YP_LOGBUFFERSIZE );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function will be called during every scheduling cycle if an non-linux thread was chosen
// Sence: Find the next activation timer value for next scheduling
/*
inline static YASA_THREAD * yasa_find_preemptor(YASA_SCHEDULER *s, YASA_THREAD *chosen)
{
	YASA_THREAD *t;
	YASA_THREAD *preemptor=0;
	for (t = s->rtl_tasks; t; t = t->next) {
		if (test_bit(RTL_THREAD_TIMERARMED, &t->threadflags)) {
			if (t->sched_param.sched_priority > chosen->sched_param.sched_priority)
			{
				if(!preemptor ||(t->resume_time < preemptor->resume_time))
				{
					preemptor = t;
				}
			}
		}
	}
	return preemptor;
}
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void inline yasa_cleanup(YASA_SCHEDULER *sched)
{
	yasa_stoplogging( &sched->logfile );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int inline yasa_rtl_schedule (void)
{
	schedule_t *sched;
	//struct rtl_thread_struct *t;
	struct rtl_thread_struct *new_task;
//	struct rtl_thread_struct *preemptor = 0;
	unsigned long interrupt_state;
	int cpu_id = rtl_getcpuid();
	hrtime_t now;
	hrtime_t preemptiontime;
	rtl_sigset_t mask;

	rtl_trace2 (RTL_TRACE_SCHED_IN, (long) pthread_self());

idle:	rtl_no_interrupts(interrupt_state);

	new_task = 0;
	sched = &rtl_sched[cpu_id];
	YASAL_LOG_COUNT_SCHEDULER_CALLS(sched);

	now = sched->clock->gethrtime(sched->clock);

	// if the thread is going into sleep mode during wait_np, there should no computed time
	// be added to prevent log entry "THREAD_CONTINUES"
	if ( RTL_CURRENT )
	{
		if ( YASA_SCHEDULABLE(RTL_CURRENT) )		// thread is finished, pthread_wait_np() called
			RTL_CURRENT->yasa_computedtime += now - sched->yasa_startofthreadtick;
		//else
		//	RTL_CURRENT->yasa_computedtime=0;
	}

	if (sched->clock->mode == RTL_CLOCK_MODE_ONESHOT) {
		sched->clock->value = now;
	}

	if ( now < sched->nextschedulingtime )			// scheduler is called explicitly
		sched->nextschedulingtime=now;
	YASAL_LOG_SCHEDULER_CALLED(sched, &now, &sched->nextschedulingtime );
	new_task=sched->schedule(sched, &now, &preemptiontime);
	if ( preemptiontime != HRTIME_INFINITY )
		preemptiontime-=now;						// create time offset from now

	/*
	for (t = sched->rtl_tasks; t; t = t->next) {
		// expire timers

		if (test_bit(RTL_THREAD_TIMERARMED, &t->threadflags)) {
			if (now >= t->resume_time) {
				clear_bit(RTL_THREAD_TIMERARMED, &t->threadflags);
				rtl_sigaddset (&t->pending, RTL_SIGNAL_TIMER);
				if (t->period != 0) { // periodic
					t->resume_time += t->period;
					// timer overrun
#ifndef CONFIG_RTL_OLD_TIMER_BEHAVIOUR
					while (now >= t->resume_time) {
						t->resume_time += t->period;
// 						rtl_printf("overrun");
					}
#endif
				} else {
					t->resume_time = HRTIME_INFINITY;
				}
			}
		}

		// and find highest priority runnable task
		if ((t->pending & ~t->blocked) && (!new_task ||
			(t->sched_param.sched_priority > new_task->sched_param.sched_priority))) {
				new_task = t;
		}
	}
	*/

/*#ifdef YP_ENABLE_LOGGING
	if ( new_task && ( new_task!= sched->rtl_current ) )
	{
		if ( ! new_task->yasa_computedtime )
		{
			YASAL_LOG_THREAD_START(new_task, &now, &YASA_CURRENTACTIVATION_TIME(new_task) );
		}
		else if ( new_task != sched->rtl_current )
		{
			YASAL_LOG_THREAD_CONT(new_task, &now, &YASA_CURRENTACTIVATION_TIME(new_task) );
		}
	}
	// no task chosen -> if there are exist a linux task we will choose it
	else
#else*/
	if ( ! new_task ) //|| new_task==&sched->rtl_linux_task )
//#endif
	{
/*		#ifdef YASA_SUPPORT_LOG_IDLE_THREAD
		if ( sched->rtl_current != &sched->rtl_linux_task )
			YASAL_LOG_IDLE_THREAD(sched, &now, 0 );
		#endif
*/		if ( sched->rtl_linux_task.pending & ~sched->rtl_linux_task.blocked )
		{
			new_task=&sched->rtl_linux_task;
			if ( preemptiontime >= (HRTICKS_PER_SEC / HZ) / 2 )
				preemptiontime=(HRTICKS_PER_SEC / HZ) / 2;				// OS tick
		}
		else
		{
			//sched->yasa_startofthreadtick = sched->clock->gethrtime(sched->clock);
			sched->yasa_startofthreadtick = now;
			rtl_allow_interrupts();
	/* 			rtl_cprintf ("idle"); */
			goto idle;
		}
	}

	if (sched->clock->mode == RTL_CLOCK_MODE_ONESHOT && !test_bit (RTL_SCHED_TIMER_OK, &sched->sched_flags))
	{
		(sched->clock)->settimer(sched->clock, preemptiontime);

/*		if ( (preemptor = find_preemptor(sched,new_task))) {
			(sched->clock)->settimer(sched->clock, preemptor->resume_time - now);
		} else {								// HRTICKS_PER_SEC=1000000000, HZ=100 -> time=5000000
			(sched->clock)->settimer(sched->clock, (HRTICKS_PER_SEC / HZ) / 2);
		}
*/		set_bit (RTL_SCHED_TIMER_OK, &sched->sched_flags);
	}

	// YASA-extensions -->
	sched->yasa_startofthreadtick = sched->clock->gethrtime(sched->clock);
#ifdef YASA_SUPPORT_LOG_SCHEDULER_FINISHED
	if ( preemptiontime != HRTIME_INFINITY )
		sched->nextschedulingtime=preemptiontime+sched->yasa_startofthreadtick;
	else
		sched->nextschedulingtime=HRTIME_INFINITY;
	YASAL_LOG_SCHEDULER_FINISHED( sched, &sched->yasa_startofthreadtick, &sched->nextschedulingtime);
#endif

#ifdef YP_ENABLE_LOGGING
	if (new_task != sched->rtl_current)
		YASAL_LOG_SCHEDULER_SWITCH( sched, &sched->yasa_startofthreadtick, 0 );
	if ( new_task != &(sched->rtl_linux_task) )
	{
		if ( ! new_task->yasa_computedtime )
		{
			YASAL_LOG_THREAD_START(new_task, &sched->yasa_startofthreadtick, &YASA_CURRENTACTIVATION_TIME(new_task) );
		}
		else // if ( new_task != sched->rtl_current )
		{
			YASAL_LOG_THREAD_CONT(new_task, &sched->yasa_startofthreadtick, &YASA_CURRENTACTIVATION_TIME(new_task) );
		}
	}
	// no task chosen -> if there are exist a linux task we will choose it
	else
	{
		YASAL_LOG_IDLE_THREAD(sched, &sched->yasa_startofthreadtick, 0 );
	}
#endif


	// <-- YASA-extensions

	if (new_task != sched->rtl_current) { /* switch out old, switch in new */

		// YASA-extensions -->
		//if ( ! sched->rtl_startofthread )
			sched->yasa_startofthread=sched->yasa_startofthreadtick;
		// <-- YASA-extensions


		if (new_task ==  &sched->rtl_linux_task) {
			rtl_make_rt_system_idle();

		} else {
			rtl_make_rt_system_active();
		}

		rtl_trace2 (RTL_TRACE_SCHED_CTX_SWITCH, (long) new_task);
		rtl_switch_to(&sched->rtl_current, new_task);
		/* delay switching the FPU context until it is really needed */
#ifdef CONFIG_RTL_FP_SUPPORT
		if (sched->rtl_current-> uses_fp &&\
			       	sched->rtl_task_fpu_owner != sched->rtl_current)
		{
			if (sched->rtl_task_fpu_owner)
			{
				rtl_fpu_save (sched,sched->rtl_task_fpu_owner);
			}
			rtl_fpu_restore (sched,sched->rtl_current);
			sched->rtl_task_fpu_owner = sched->rtl_current;
		}
#endif /* CONFIG_RTL_FP_SUPPORT */
	}
/*	if (RTL_CURRENT==rtl_get_linux_thread(rtl_getcpuid())) {
		unsigned long flags;
		rtl_allow_interrupts();
		__save_flags(flags);
		__restore_flags(flags);
	}*/
	mask = pthread_self()->pending;
	rtl_restore_interrupts(interrupt_state);

	if (pthread_self()->pending & ~(1 << RTL_SIGNAL_READY)) do_signal(pthread_self());

	if (!rtl_sigismember(&pthread_self()->pending, RTL_SIGNAL_READY)) {
		goto idle;
	}

	rtl_trace2 (RTL_TRACE_SCHED_OUT, (long) pthread_self());
	return mask;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASA_RTLINUX_THREAD_INCLUDE