#!/bin/sh
# author:  Eagleeye/DFT
# project: yasa

#######################################################################################
# function: make RTL-Kernel                                                           #
# Attention this function will exit with error codes!                                 #
#######################################################################################
make_rtl_kernel() {
	echo "Entering directory $RTLWORKDIR"
	cd $RTLWORKDIR
	
	touch $RTL/.config
	rm -v -f $RTL/.hdepend
	if [ -z $OPTIONFAST ]; then
		make dep
	fi
	make all
	
	# last command (make all) okay ?
	if [ $? -eq 0 ]; then
		if [ $RTL = $RTLWORKDIR ]; then
			echo "###########################"
			echo "# Compiling RTL-Modules ! #"
			echo "###########################"
			make devices
		else
			cd $RTL
			cp -v modules/*.o /usr/rtlinux/modules
		fi
		if [ $? -eq 0 ]; then
			echo "###########################"
			echo "# Installing RTL-Kernel ! #"
			echo "###########################"
			make install
			if [ $? -eq 0 ]; then
				echo
				echo "Yasa rtlinux extension compiled successfully."
				echo
				exit 0
			fi
		fi
	fi
	echo "Error occured! Yasa rtlinux extension could not be compiled successfully."
	exit 1
}


#######################################################################################
# Input: parameter 1 = source path of file
#        parameter 2 = destination path
make_backup()
{
	if [ ! -e $2$BACKUPEXTENSION ]; then
		#echo "Making backup of \"$2\"";
		cp -v $2 $2$BACKUPEXTENSION;
	else
		echo "Backup of \"$2$BACKUPEXTENSION\" already exists!";
	fi
	cp $1 $2
}

#######################################################################################
# Input: parameter 1 = source path of file
#        parameter 2 = destination path
restore_backup()
{
	if [ -e $2$BACKUPEXTENSION ]; then
		#echo "Restoring original \"$1\""
		mv -v $2$BACKUPEXTENSION $2
	else
		echo "Fatal error. Could not restore backup!"
		echo "backup file \"$2$BACKUPEXTENSION\" not found!"
		echo "Reinstall linux/rtlinux!"
		exit 1
	fi
}


#######################################################################################
# main callup                                                                         #
# arg1 = option <install|remove|all|clean>                                            #
## arg2 = source of rtlinux                                                            #
#######################################################################################
# if you run this script manually please set the following environment variables
#export	YASA=/usr/local/bin/yasa

#export YASA=/home/eagleeye/sources/yasa
#export YASA_EXECUTIV=rtlinux
#export YASA_EXECUTIVEDIR=$YASA/executives/rtlinux
#export YASA_INCLUDE="-I/home/eagleeye/sources/yasa/include -I/home/eagleeye/sources/yasa/executives/rtlinux -I/home/eagleeye/sources/yasa/projects/3tasks/rtlinux -I/home/eagleeye/sources/yasa/schedulers/prio"
#export RTL=/usr/src/rtlinux
#export YASA_PROJECTDIR=$YASA/projects/3tasks
#export YASA_PROJECTFILENAME=3tasks
#export YASA_SCHEDULERS=prio

##export                 RTL=$2
export         RTLSCHEDDIR=$RTL/schedulers
export       RTLINCLUDEDIR=$RTL/include
export	   BACKUPEXTENSION=.yasabackup
export     YASARTLPATCHDIR=$YASA_EXECUTIVEDIR/kernel/install/rtlinux_v`rtl-config --rtlVersion`
export         RTLMAKEFILE=$RTL/Makefile
export          RTLWORKDIR=$RTL
export           OPTIONONE="$1"
export          OPTIONFAST=

############################################
# Checking arguments
if [ $# -ne 1 ]; then
	echo "Error parsing arguments! Please specify:";
	echo "make_yasa_rtlinux install|remove";
	exit 1;
fi


################################################
# if one of the "fast"-options is specified
# then all actions will be done only in the
# rtlinux/scheduler directory. That will make
# the compiling process much faster
# Note: Only for testing purposes !!!
################################################
echo "Changing directory to :$YASA_EXECUTIVEDIR"
cd $YASA_EXECUTIVEDIR
case $OPTIONONE in
fastinstall)
					export OPTIONONE=install
					export RTLWORKDIR=$RTL/schedulers
					export OPTIONFAST=1
					;;
fastremove)
					export OPTIONONE=remove
					export RTLWORKDIR=$RTL/schedulers
					export OPTIONFAST=1
					;;
fastclean)
					export OPTIONONE=clean
					export RTLWORKDIR=$RTL/schedulers
					export OPTIONFAST=1
					;;
fastrtlkernel)
					export OPTIONONE=rtlkernel
					export RTLWORKDIR=$RTL/schedulers
					export OPTIONFAST=1
					;;
esac


############################################
# Do something
case $OPTIONONE in
install|all)
	if [ ! -e $RTL ]; then
		echo "Warning: Could not find RTLinux source directory at $RTL"
		echo "Compiling yasa extensions for RTLinux aborted!"
		echo "That does not affect other executives nor the"
		echo "YASA program itself."
		exit 0
	fi

	echo "######################################"
	echo "# Installing rtlinux yasa-extensions #"
	echo "######################################"
	echo "Current directory: `pwd`"
	echo "Patches used from $YASARTLPATCHDIR"

	###################################################
	######## PATCH ORIGINAL RTLinux-Makefile ##########
	make_backup $YASARTLPATCHDIR/Makefile 						$RTLMAKEFILE
	make_backup $YASARTLPATCHDIR/schedulers/Makefile			$RTLSCHEDDIR/Makefile
	make_backup $YASARTLPATCHDIR/schedulers/rtl_sched.c 		$RTLSCHEDDIR/rtl_sched.c
	make_backup $YASARTLPATCHDIR/schedulers/rtl_mutex.c 		$RTLSCHEDDIR/rtl_mutex.c
	make_backup $YASARTLPATCHDIR/include/rtl_sched.h 			$RTLINCLUDEDIR/rtl_sched.h
	make_backup $YASARTLPATCHDIR/include/rtl_mutex.h 			$RTLINCLUDEDIR/rtl_mutex.h
	make_backup $YASARTLPATCHDIR/linux/include/linux/sched.h	$RTL/linux/include/linux/sched.h


	echo "##########################"
	echo "# Compiling RTL-Kernel ! #"
	echo "##########################"
	make_rtl_kernel
	;;

remove)
	if [ ! -e $RTL ]; then
		echo "Warning: Could not find RTLinux source directory at $RTL"
		echo "Compiling original RTLinux aborted!"
		echo "That does not affect other executives nor the"
		echo "YASA program itself."
		exit 0
	fi

	echo "Checking for installed yasa versions"
	if [ ! -e $RTLMAKEFILE$BACKUPEXTENSION ]; then
		echo "Yasa RTLinux extensions are not installed or already uninstalled!"
		echo "If you want to compile the kernel again type \"make rtlkernel\""
		exit 0
	fi
	
	restore_backup $YASARTLPATCHDIR/Makefile 					$RTLMAKEFILE
	restore_backup $YASARTLPATCHDIR/schedulers/Makefile			$RTLSCHEDDIR/Makefile
	restore_backup $YASARTLPATCHDIR/schedulers/rtl_sched.c 		$RTLSCHEDDIR/rtl_sched.c
	restore_backup $YASARTLPATCHDIR/schedulers/rtl_mutex.c 		$RTLSCHEDDIR/rtl_mutex.c
	restore_backup $YASARTLPATCHDIR/include/rtl_sched.h 		$RTLINCLUDEDIR/rtl_sched.h
	restore_backup $YASARTLPATCHDIR/include/rtl_mutex.h 		$RTLINCLUDEDIR/rtl_mutex.h
	restore_backup $YASARTLPATCHDIR/linux/include/linux/sched.h	$RTL/linux/include/linux/sched.h

	echo "################################"
	echo "# Compiling original RTLinux ! #"
	echo "################################"
	make clean
	make dep
	make_rtl_kernel
	;;

clean)
	if [ ! -e $RTL ]; then
		echo "Warning: Could not find RTLinux source directory at $RTL";
		echo "Cleaning original RTLinux aborted!";
		echo "That does not affect other executives nor the";
		echo "YASA program itself.";
		exit 0
	fi

	echo "Checking for installed yasa versions"
	if [ ! -e $RTLMAKEFILE$BACKUPEXTENSION ]; then
		echo "Yasa RTLinux extensions are not installed or already uninstalled!";
		exit 0;
	fi
	cd $RTLWORKDIR

	make -C $RTLWORKDIR clean
	if [ $? -eq 0 ]; then
		exit 0
	fi
	echo "Error occured! Yasa rtlinux could not clean up"
	exit 1
	;;

rtlkernel)
	echo "###############################"
	echo "# Compiling current RTLinux ! #"
	echo "###############################"
	make_rtl_kernel
	;;

make_backup)
	make_backup hallo test
	;;
	
*)
	echo "Unknown option: $OPTIONONE"

esac
exit 0
