//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA simulator declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/simulator/sources/simulator.c,v 1.3 2003/01/23 09:18:30 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>

#include <simulator_functions.h>

#define SIMULATOR_STARTTIME	0

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// internal functions
YASA_THREAD* 	ys_findthread( YASA_SCHEDULER *schedulers, const char *name );
int				ys_init_scheduling(YASA_SCHEDULER *schedulers, YASA_ENVIRONMENT_ATTR *environmentattr);
void 			ys_end_scheduling(YASA_SCHEDULER *schedulers, YASA_ENVIRONMENT_ATTR *environmentattr, FILE *handle);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_SCHEDULER		schedulers[YP_MAX_CPU];
int					cpu;
YASA_MUTEX			*firstmutex;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//int yasa_simulator_scheduling(const struct yasa_scheduling_attr *schedulingattr, struct yasa_thread_attr *threadattr, struct yasa_mutex_attr *mutexattr, FILE *handle )
int yasa_simulator_scheduling(struct yasa_environment_attr *environmentattr, FILE *handle )
{
	int				retvalue=YASAE_NO_ERROR;

	retvalue=ys_init_scheduling(schedulers, environmentattr );
	if ( ! retvalue )
	{
		retvalue=ys_scheduling(schedulers);
	}
	else
		printf("Error %d occured!\n", retvalue);

	ys_end_scheduling(schedulers, environmentattr, handle);
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// support functions
int ys_init_scheduling(YASA_SCHEDULER *schedulers, YASA_ENVIRONMENT_ATTR *environmentattr)
{
	const YASA_SCHEDULING_ATTR	*schedulingattr=environmentattr->firstschedulingattr;
	YASA_THREAD_ATTR			*threadattr=environmentattr->firstthreadattr;
	//YASA_MUTEX_ATTR				*newmutex=environmentattr->firstmutexattr;
	int							retvalue=YASAE_NO_ERROR;
	int							cpu;
	YASA_THREAD					*threadtmp;
	YASA_THREAD					*thread;
	YASA_THREAD_ATTR			*firstthreadattr;
	YASA_LOG_ENTRY_THREADPARAM	threadlogparam;
	YASA_TIME					synchronizedstart=environmentattr->synchronizationoffset+SIMULATOR_STARTTIME;
	// clear synchronization struct
	yasa_initsyncstruct();

	// prepare simulation & select scheduler
	firstthreadattr=threadattr;
	for (cpu=0; cpu<YP_MAX_CPU; cpu++)
	{
		// scheduler initialization
		yasa_logfileconstructor( &(schedulers[cpu]), &(schedulers[cpu].logfile));
		retvalue=yasa_initlogging(&(schedulers[cpu].logfile), YP_LOGGINGSIZE);
		if ( retvalue )
			return retvalue;
		schedulers[cpu].nextschedulingtime	= SIMULATOR_STARTTIME;
		schedulers[cpu].cpu					= cpu;
		schedulers[cpu].currenttime			= SIMULATOR_STARTTIME;
		schedulers[cpu].currentthread		= 0;
		schedulers[cpu].firsttask			= 0;
		schedulers[cpu].schedulerstart		= SIMULATOR_STARTTIME;
		schedulers[cpu].schedulerend		= schedulingattr->schedulingend+SIMULATOR_STARTTIME;
		schedulers[cpu].calcpreemptiontime	= schedulingattr->calcpreemptiontime;
		schedulers[cpu].tick				= schedulingattr->tick ? schedulingattr->tick : YASA_DEF_TICK;
		if ( schedulingattr->schedulerattr )
		{
			schedulers[cpu].schedule=schedulingattr->schedulerattr->yasa_schedule;
			schedulers[cpu].yasa_schedulerattr=schedulingattr->schedulerattr;
			if ( schedulingattr->schedulerattr->yasa_init )
			{
				retvalue=schedulingattr->schedulerattr->yasa_init(&schedulers[cpu]);
				if ( retvalue )
					return retvalue;
			}
		}
		else
			return EINVAL;					// schedulerattr missing
		YASAL_LOG_SCHEDULER_SYNCHRONIZED_START( &schedulers[cpu], YASA_SCHEDULER_ID, &synchronizedstart, 0 );
		YASAL_LOG_SCHEDULER_SYNCHRONIZED_START( &schedulers[cpu], YASA_IDLE_ID, &synchronizedstart, 0 );
		schedulingattr++;
	}

	// insert threads
	while ( threadattr )
	{
		// thread initialization
		if ( threadattr->cpu < YP_MAX_CPU )
		{
			thread=malloc( sizeof(YASA_THREAD) );
			if ( thread )
			{
				threadtmp=schedulers[threadattr->cpu].firsttask;
				if ( !threadtmp )
				{
					schedulers[threadattr->cpu].firsttask=thread;
				}
				else
				{
					while ( threadtmp->next )
						threadtmp=threadtmp->next;
					threadtmp->next=thread;
				}
				// static members
				thread->next					= 0;
				yasa_id_copy( &thread->id, &threadattr->id );
				thread->threadtype				= threadattr->threadtype;
				YASA_PRIORITY(thread)			= threadattr->priority;
				thread->period					= threadattr->period;
				YASA_REQUIRED_TIME(thread)		= threadattr->requiredtime;
				YASA_DEADLINE(thread)			= threadattr->deadline;
				YASA_DEADLINE_TOLERANCE(thread)	= threadattr->deadlinetolerance;
				YASA_DL_BEHAVIOUR(thread)		= threadattr->deadlinebehaviour;
				YASA_MAXLAXITY(thread)			= YASA_DEADLINE(thread) - YASA_REQUIRED_TIME(thread);
				thread->activationvectors		= threadattr->activationvectors;
				thread->nextactivationvectorpos	= threadattr->activationvectors;
				thread->currentactivationvectorpos= threadattr->activationvectors;
				thread->resourcevectors			= threadattr->resourcevectors;
				thread->resourcevectorpos		= threadattr->resourcevectors;
				thread->cpu						= threadattr->cpu;
				thread->scheduler				= &schedulers[threadattr->cpu];
				thread->schedulingdata			= 0;
				thread->emergencythread			= 0;
				thread->emergencycallingthread	= 0;

				// dynamic members
				thread->resume_time				= threadattr->offset+environmentattr->synchronizationoffset+SIMULATOR_STARTTIME;
				thread->nextactivation_time		= threadattr->offset+environmentattr->synchronizationoffset+SIMULATOR_STARTTIME;
				thread->currentactivation_time	= threadattr->offset+environmentattr->synchronizationoffset+SIMULATOR_STARTTIME;
				thread->excluded				= 0;
				thread->computedtime			= 0;
				thread->blocked					= 0;
				thread->deadlinemissed			= 0;
				thread->deadlinetolerancemissed	= 0;
				thread->counter					= 10;

				if ( thread->resume_time )
					thread->pending=0;							// will be started at thread->resumetime
				else
					thread->pending=1<<YS_SIGNAL_READY;
				thread->threadflags=1<<YS_TIMER_ARMED;
			}
			else
			{
				retvalue=YASAE_OUT_OF_MEMORY;
				break;
			}
		}
		else
		{
			retvalue=YASAE_WRONG_CPU;
			break;
		}
		yasa_id_copy ( &(threadlogparam.id), &(thread->id) );
		YASAL_LOG_THREAD_CREATED( & schedulers[0], 0, &threadlogparam );
		YASAL_LOG_SCHEDULER_SYNCHRONIZED_START( thread->scheduler, thread->id.id, &synchronizedstart, 0 );		// t=0;

		threadattr=threadattr->next;
	}


	// link emergency threads
	threadattr=firstthreadattr;
	while ( threadattr )
	{
		if ( threadattr->emergencythread )
		{
			thread=ys_findthread( schedulers, threadattr->id.name);
			if ( thread )
			{
				thread->emergencythread=ys_findthread( schedulers, threadattr->emergencythread->id.name );
				if ( thread->emergencythread )
				{
					if ( ! thread->emergencythread->emergencycallingthread )
					{
						thread->emergencythread->emergencycallingthread=thread;
						// exclude from scheduling
						thread->emergencythread->pending=0;							// will be started at thread->resumetime
						thread->emergencythread->threadflags&= ~(1<<YS_TIMER_ARMED);// this thread is not armed
						thread->emergencythread->resume_time=YASA_TIME_INFINITY;
					}
					else
						return YASAE_EMERGENCYCALL_DEFINE_TWICE;
				}
			}
		}
		threadattr=threadattr->next;
	};

	// create mutexes
	yasa_calculateceilings(environmentattr->firstmutexattr, firstthreadattr);
	ys_alloc_mutex( &firstmutex, environmentattr->firstmutexattr );


	return retvalue;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// finish simulation
//void ys_end_scheduling(YASA_SCHEDULER *schedulers, const YASA_SCHEDULING_ATTR *schedulingattr, FILE *handle)
void ys_end_scheduling(YASA_SCHEDULER *schedulers, YASA_ENVIRONMENT_ATTR *environmentattr, FILE *handle)
{
	int						cpu;
	YASA_THREAD				*thread;
	void*					retval=(void*)1;
	const YASA_SCHEDULING_ATTR	*schedulingattr=environmentattr->firstschedulingattr;

	for (cpu=0; cpu<YP_MAX_CPU; cpu++)
	{
		if ( schedulingattr->schedulerattr )
		{
			if ( schedulingattr->schedulerattr->yasa_cleanup )
			{
				schedulingattr->schedulerattr->yasa_cleanup(&schedulers[cpu]);
			}
		}

		// free thread memory
		while ( schedulers[cpu].firsttask )
		{
			thread=schedulers[cpu].firsttask;
			YASAL_LOG_THREAD_EXIT( thread, &schedulers[cpu].currenttime, &retval );

			schedulers[cpu].firsttask=thread->next;
			free(thread);
		}

		yasa_printlogfile(&schedulers[cpu].logfile, &schedulers[cpu].schedulerstart, cpu, handle);
		yasa_stoplogging(&schedulers[cpu].logfile);
		schedulingattr++;
	}

	ys_free_mutex( &firstmutex );
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_THREAD* ys_findthread( YASA_SCHEDULER *schedulers, const char *name )
{
	int					cpu;
	YASA_THREAD			*thread;

	for (cpu=0; cpu<YP_MAX_CPU; cpu++)
	{
		thread=schedulers[cpu].firsttask;
		while ( thread )
		{
			if ( strcmp( name, yasa_id_get_name(&thread->id) ) == 0 )
				return thread;
			thread=thread->next;
		}
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef __CYGWIN__
YASA_SCHEDULER* GetCurrentScheduler()
{
	return &schedulers[cpu];
}
#endif