//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA simulator declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/simulator/sources/simulator.h,v 1.4 2003/01/23 09:18:30 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_SIMULATOR_INCLUDE
#define	YASA_SIMULATOR_INCLUDE

#include <malloc.h>
#include <stdio.h>
#include <string.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef INT64
 #ifdef WIN32
  #define INT64						__int64
 #else
  #define INT64						long long
 #endif
#endif
#ifndef YASA_TIME
  #define YASA_TIME					INT64
#endif
#ifndef YASA_TIME_INFINITY
 #ifdef WIN32
  #define YASA_TIME_INFINITY			0x7fffFfffFfffFfff
 #else
  #define YASA_TIME_INFINITY			0x7fffFfffFfffFfffLL
 #endif
#endif

#define PRINTK						printf






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define YASA_EXECUTIVE_NAME					"simulator"
#define YASA_EXECUTIVE_VERSION				"1.2"
#define YASA_DEFAULT_DEADLINE				YASA_TIME_INFINITY
#define YASA_DEFAULT_RESUMETIME				YASA_TIME_INFINITY
#define YASA_DEFAULT_REQUIREDTIME			YASA_TIME_INFINITY

#define YS_TIMER_ARMED						0						// is the value in resume time 0=valid?
//#define YS_DEADLINE_MISSED				1						// deadline already missed 1=yes
																	// =1 means, that the value must be recalculated

#define YS_SIGNAL_WAKEUP					1
#define YS_SIGNAL_SUSPEND					2
#define YS_SIGNAL_READY						3						// thread is ready to be executed
#define YS_SIGNAL_TIMER						4						// new timer value is set

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// is the resume time/time value set for this thread?
//
#define YASA_TEST_TIMER_ARMED(thread)		thread->threadflags & (1<<YS_TIMER_ARMED)
#define YASA_UNSET_TIMER_ARMED(thread)		thread->threadflags=thread->threadflags&~(1<<YS_TIMER_ARMED)
#define YASA_SIGNAL_TIMER(thread)			thread->pending|=(1<<YS_SIGNAL_TIMER)
#define YASA_SCHEDULABLE(thread)			( thread->pending )
#define YASA_CHECK_NEXT_ACTIVATION(thread)	(*now >= YASA_NEXTACTIVATION_TIME(thread) )
#define YASA_MALLOC(size)					malloc(size)
#define YASA_FREE(ptr)						free(ptr)
#define YASA_SCHEDULERSTART(sched)			(sched)->schedulerstart
#define YASA_THREAD_CANCEL(thread)			ys_thread_wait(thread)
#define YASA_SET_TIMER_ARMED(thread)		(thread)->threadflags|=(1<<YS_TIMER_ARMED)
#define YASA_CALCPREEMPTIONTIME(sched)		(sched->calcpreemptiontime)
#define YASA_TEST_AND_SET_BIT(bit, ptr)		( *ptr & (1<<bit ) ? 1 : ! ( *ptr|=(1<<bit) ) )
#define YASA_CLEAR_BIT(bit, ptr)			( *ptr&=~(1<<bit) )
#define YASA_RESCHEDULE						ys_reschedule		// rtl_schedule()


#ifdef WIN32
 #define snprintf _snprintf
#endif

#define YASA_LOGWRITE(handle, buffer)		fprintf(handle,buffer)
#define YASA_LOGWRITE1(string, arg)			sprintf(buffer, string, arg); \
											YASA_LOGWRITE( handle,  buffer )
#define YASA_LOGWRITE2(string, arg1, arg2)	sprintf(buffer, string, arg1, arg2); \
											YASA_LOGWRITE( handle, buffer )
#define YASA_LOGWRITE3(string, arg1, arg2,arg3)	sprintf(buffer, string, arg1, arg2, arg3); \
											YASA_LOGWRITE( handle, buffer )
#define YASA_LOGWRITE4(string, arg1, arg2,arg3,arg4)	sprintf(buffer, string, arg1, arg2, arg3,arg4); \
											YASA_LOGWRITE( handle, buffer )
#define YASA_LOGEND							'\n'


#define YASA_TEST_DEADLINE_MISSED(t,now)	( ! t->deadlinetolerancemissed && YASA_CURRENT_DEADLINE(t) <= *now )
#define YASA_DEADLINE_ALREADY_MISSED(thread) ( thread->deadlinemissed )
#define YASA_SET_DEADLINE_MISSED(thread)	thread->deadlinemissed=1;
#define YASA_DEADLINE_TOLERANCE_ALREADY_MISSED(thread) ( thread->deadlinetolerancemissed )
#define YASA_SET_DEADLINE_TOLERANCE_MISSED(thread)	thread->deadlinetolerancemissed=1;
#define YASA_SETTICKSCHEDULINGTIME(ptime, now, scheduler)	{	if (scheduler->tick && scheduler->tick!=YASA_MAX_TICK )	\
																	*ptime = *now + scheduler->tick;					\
																else													\
																	*ptime = YASA_MAX_TICK;								}
#define YASA_INITSCHEDULINGLOOP(scheduler, newthread)	if ( scheduler->currentthread && YASA_SCHEDULABLE(scheduler->currentthread) ) newthread=YASA_CURRENT_THREAD;	\
													else newthread=0;

// Min/Max definitions
#define YASA_MIN_CPU						1
#define YASA_MAX_CPU						NR_CPUS
#define YASA_MIN_PRIORITY					0
#define YASA_MAX_PRIORITY					0x7ffffffe
#define YASA_MAX_LOGGINGSIZE				-1
#define YASA_MIN_TICK						1
//#define YASA_DEF_TICK						1000
#define YASA_MAX_TICK						YASA_TIME_INFINITY			// YASA_TIME_INFINITY-1
#define YASA_DEF_TICK						YASA_MAX_TICK
#define YASA_MUTEX_DEFAULT_PRIORITY			YASA_MIN_PRIORITY-1
#define YASA_MUTEX_DEFAULT_CEILING			YASA_MAX_PRIORITY+1
#define YASA_MUTEX_MAXSTACK					127		// rtlinux RTL_SEM_NSEMS_MAX


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// scheduler mappings
#ifdef __CYGWIN__	// necessary to prevent cygwin linker bug
 #define YASA_CURRENT_SCHEDULER				( GetCurrentScheduler() )
#else				// save a call
 #define YASA_CURRENT_SCHEDULER				( &schedulers[cpu])
#endif
#define YASA_TASKSET(scheduler) 			(scheduler->firsttask)
#define YASA_SCHEDULER_ATTR(sched)			( sched->yasa_schedulerattr )
#define YASA_GETCURRENTTIME(sched)			(sched->currenttime)
#define YASA_GETSCHEDULER(thread)			(thread->scheduler)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread mappings
#define YASA_NEXT_THREAD(thread) 			thread->next
#define YASA_RESUME_TIME(thread) 			thread->resume_time
#define YASA_NEXTACTIVATION_TIME(thread) 	thread->nextactivation_time
#define YASA_CURRENTACTIVATION_TIME(thread) thread->currentactivation_time
#define YASA_DEADLINE(thread)				thread->sched_param.deadline
#define YASA_DEADLINE_TOLERANCE(thread)		thread->sched_param.deadlinetolerance
#define YASA_CURRENT_DEADLINE(thread)		thread->currentdeadline
#define YASA_PERIOD(thread) 				thread->period
#define YASA_COMPUTED_TIME(thread)			thread->computedtime
#define YASA_REQUIRED_TIME(thread)			thread->sched_param.requiredtime
#define YASA_PRIORITY(thread)				thread->sched_param.priority
#define YASA_MAXLAXITY(thread)				thread->maxlaxity
#define YASA_EXCLUDED(thread)				thread->excluded
#define YASA_SCHEDULINGDATA(thread)			thread->schedulingdata
#define YASA_THREADTYPE(thread)				thread->threadtype
//#define YASA_THREAD_EXCLUDED(thread)		thread->excluded
//#define YASA_THREAD_SCHEDULER_DATA(thread)	thread->schedulerdata
#define YASA_DL_BEHAVIOUR(thread)			thread->sched_param.deadlinebehaviour
#define YASA_EMERGENCYTHREAD(thread)		thread->emergencythread
#define YASA_CURRENTACTIVATIONVECTORPOS(thread)	thread->currentactivationvectorpos
#define YASA_NEXTACTIVATIONVECTORPOS(thread)	thread->nextactivationvectorpos
#define YASA_RESETRESOURCEVECTOR(thread)	thread->resourcevectorpos=thread->resourcevectors
#define YASA_CURRENT_THREAD					(YASA_CURRENT_SCHEDULER->currentthread)
#define YASA_SUSPEND_THREAD(thread)			self->pending = (1<<YS_SIGNAL_SUSPEND)
											//RTL_MARK_SUSPENDED (self);
#define YASA_WAKEUP_THREAD(thread)			thread->pending = (1<<YS_SIGNAL_WAKEUP);	/* signal that the thread should be waked up, no reschedule */
#define YASA_EARLIER_THREAD(thread, newthread)		if ( !newthread ) newthread=thread;	// used in fcfs scheduler, sequence of threads differs from executive to executive

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// mutex mappings
#define YASA_MUTEX_PRIORITY_CEILING(mutex)	(mutex->priorityceiling)
#define YASA_MUTEX_OLD_PRIORITY(mutex)		(mutex->oldpriority)
#define YASA_MUTEX_LOCKED(mutex)			(mutex->locked)
#define YASA_MUTEX_WAKEUP(mutex)			ys_wait_wakeup( &((mutex)->waitqueue) )


struct yasa_thread;
struct yasa_scheduler;
struct yasa_scheduling_attr;
struct yasa_thread_attr;
struct yasa_mutex_attr;
struct yasa_environment_attr;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_wait_entry
{
	//int 						magic;
	struct yasa_thread		 	*waiter;
	struct yasa_wait_entry	 	*next;
};
typedef struct yasa_wait_entry YASA_WAIT_ENTRY;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_wait_queue
{
	struct yasa_wait_entry		*queue;
	//spinlock_t *p_lock;
};
typedef struct yasa_wait_queue YASA_WAIT_QUEUE;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_sched_param
{
	int						priority;				// priority of task
	YASA_TIME				deadline;				// deadline in period
	YASA_TIME				deadlinetolerance;		// tolerance after deadline
	YASA_TIME				requiredtime;			// time to compute
	int						deadlinebehaviour;		// behaviour if deadline is missed
};
typedef struct yasa_sched_param YASA_SCHED_PARAM;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
union yasa_dyn_priority
{
	YASA_TIME				deadline;
	YASA_TIME				laxity;
	YASA_TIME				requiredtime;
	YASA_TIME				shortesttime;
	int						priority;
};
typedef union yasa_dyn_priority YASA_DYN_PRIORITY;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_mutex
{
	struct yasa_mutex		*next;					// only for internal use
	YASA_ID					id;						// id of mutex
	int						locked;					// 0=not locked
	int						protocol;				// one of the protocol PCP, PIP, DPCP, SRP etc.
	struct yasa_thread		*thread;
	int						oldpriority;			// PIP, PCP
	int						priorityceiling;		// PIP, PCP
	YASA_DYN_PRIORITY		dynamicceiling;			// DPCP
    YASA_DYN_PRIORITY		preemptceiling;			// SRP
    YASA_WAIT_QUEUE			waitqueue;
};
typedef struct yasa_mutex YASA_MUTEX;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_thread
{
	// static members
	struct yasa_thread			*next;
	YASA_ID						id;
	int							threadtype;			// type of thread (periodic,nonperiodic...)
	int							counter;			// dyn. priority (Unix systems)
	YASA_TIME					period;				// period of task
	YASA_TIME					maxlaxity;			// laxity in period at time t=0
	YASA_TIME					*activationvectors;	// start time of asynchronous threads
	struct yasa_resourcevector	*resourcevectors;	// start time of resource actions
	unsigned int				cpu;
	struct yasa_scheduler		*scheduler;
	struct yasa_thread			*emergencythread;		// ptr to emergencycall
	struct yasa_thread			*emergencycallingthread;// ptr to thread which is calling the emergencycall
	YASA_SCHED_PARAM			sched_param;

	// dynamic members
	YASA_TIME					resume_time;		// when should this thread resume ?
	YASA_TIME					currentactivation_time;	// start of THIS(!!!) period
	YASA_TIME					nextactivation_time;// start of next or next next period (if deadline was missed). This counts up
	YASA_TIME					currentdeadline;	// deadline of current period since bootup (YASA_TIME_INFINITY if not set)
	YASA_TIME					*nextactivationvectorpos;
	YASA_TIME					*currentactivationvectorpos;
	struct yasa_resourcevector	*resourcevectorpos;
	int							excluded;			// is this process (excluded 0=no, otherwise yes)
	YASA_TIME					schedulingdata;		// can be used by the scheduler, e.g. ELLF, default=0
	YASA_TIME					computedtime;		// computed time since thread start
	int							pending;			// ala pending in rtlinux
	int							blocked;
	int							threadflags;
	int							deadlinemissed;
	int							deadlinetolerancemissed;
};

typedef struct yasa_thread YASA_THREAD;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_scheduler
{
	YASA_THREAD*				firsttask;			// ptr to array
	YASA_THREAD*				currentthread;		// ptr to current thread
	YASA_TIME					currenttime;		// current time
	YASA_TIME					nextschedulingtime;	// to of next scheduling procedure
	YASA_TIME					schedulerstart;		// start of scheduling
	YASA_TIME					schedulerend;		// end of scheduling
	YASA_TIME					tick;				// This is the tick value
	int							cpu;				// number of cpu
	int							calcpreemptiontime;	// this scheduler should calculate a preemption time
	YASA_LOGFILE				logfile;
	const struct yasa_scheduler_attr	*yasa_schedulerattr;	// ptr to structure of scheduler
	YASA_THREAD*				(*schedule)(struct yasa_scheduler *scheduler, const YASA_TIME *now, YASA_TIME *ptime);	// scheduling function
};
typedef struct yasa_scheduler YASA_SCHEDULER;






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Logfile support functions
#define YASA_LOCKLOGFILE(a)		return 1;
#define YASA_LOCKLOGFILEWAIT(a)	return 1;
#define YASA_UNLOCKLOGFILE(a)	return;





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// declare support functions provided by simulator executive

int				yasa_simulator_scheduling(struct yasa_environment_attr *environmentattr, FILE *handle );
void			ys_thread_wait(YASA_THREAD *thread);
void			ys_reschedule(void);
void			ys_do_signal(YASA_THREAD *thread);
int 			ys_wait_wakeup (YASA_WAIT_QUEUE *waitqueue);
#ifdef __CYGWIN__	// necessary to prevent cygwin linker bug
 YASA_SCHEDULER* GetCurrentScheduler(void);
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//extern	YASA_SCHEDULER	**schedulers[YP_MAX_CPU];
extern YASA_SCHEDULER	schedulers[];
extern int				cpu;
extern YASA_MUTEX		*firstmutex;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif														// ifdef YASA_SIMULATOR_INCLUDE
