//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA global declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/include/yasa/yasa.h,v 1.2 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_INCLUDE
#define	YASA_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa_errors.h>
#include <yasa/yasa_types.h>



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef FALSE
#define FALSE (1==0)
#endif

#ifndef TRUE
#define TRUE (1==1)
#endif

#ifdef WIN32
 /*#ifdef _DEBUG
  #define INLINE
 #else
  #define INLINE	__inline
 #endif
 */
  #define INLINE	__inline
#else
 #define INLINE	inline
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct
{
	int				id;										// id of thread/mutex/attr if set otherwise 0
	char			name[YASA_MAXNAME+2];					// name of thread if set otherwise ""
} YASA_ID;

int 		yasa_id_init(YASA_ID *id);
int 		yasa_id_copy(YASA_ID *dest, const YASA_ID *src);
int			yasa_id_set_id(YASA_ID *id, int newid);
int			yasa_id_get_id(const YASA_ID *id);
const int*	yasa_id_get_idptr(const YASA_ID *id);
const char *yasa_id_get_name(const YASA_ID *id);
int			yasa_id_set_name(YASA_ID *id, const char *name);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_logfile
{
		char*			logbuffer;
		unsigned int	logbuffersize;
		unsigned int	logbufferpos;
		int				locked;			// 1=yes, 0=no
		int				schedulercounts;
		void			*scheduler;
};
typedef struct yasa_logfile YASA_LOGFILE;




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// project settings:
// * project declarations
// * scheduler declarations
// * executive declarations
#include <yasa_projectconfig.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yasa_resourcevector
{
	void				*data;
	int					action;
	YASA_TIME			time;
};
typedef struct yasa_resourcevector YASA_RESOURCEVECTOR;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This structure describes the behaviour of each thread to be created by YASA
struct yasa_thread_attr
{
	struct yasa_thread_attr		*next;						// ptr to next init structure

	// static members
	YASA_ID						id;							// id of thread
	int							threadtype;					// type of thread (periodic,nonperiodic...)
	int							priority;					// priority of task
	YASA_TIME					period;						// period of task
	YASA_TIME					requiredtime;				// time to compute
	YASA_TIME					deadline;					// deadline in period
	YASA_TIME					deadlinetolerance;			// tolerance of deadline
	YASA_TIME					offset;						// start of first execution, since start of simulation
	YASA_TIME					*activationvectors;			// start time of asynchronous threads
	struct yasa_resourcevector	*resourcevectors;			// start time of resource actions
	unsigned int				cpu;						// number of used cpu (0...n)
	int							deadlinebehaviour;			// behaviour if deadline is missed
	struct yasa_thread_attr		*emergencythread;			// emergency thread
	void*						(*threadinit)(void* param, void **userdata);// initialization of thread
	void*						(*threadexecute)(void **userdata);			// execution code of thread
	void						(*threadcleanup)(void **userdata);			// cleanup thread
};
typedef struct yasa_thread_attr YASA_THREAD_ATTR;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This structure describes the behaviour of each mutex to be created by YASA
struct yasa_mutex_attr
{
	struct yasa_mutex_attr	*next;
	YASA_ID					id;
	int						protocol;
	int						priorityceiling;		// \ will be calculated before creating mutexes if values are
	YASA_DYN_PRIORITY		preemptceiling;			// / not set !
};
typedef struct yasa_mutex_attr YASA_MUTEX_ATTR;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa_logging.h>
#include <yasa/yasa_scheduling.h>
#include <yasa/yasa_resources.h>
#include <yasa/yasa_resourcesdynamic.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif														// ifndef YASA_INCLUDE
