//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA resource handling (mutexs etc.) declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/15
// $Header: /sources/yasa/include/yasa/yasa_resources.h,v 1.2 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_RESOURCES_INCLUDE
#define	YASA_RESOURCES_INCLUDE

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>





// ascii names of supported synchronization protocols
#define YASASPN_NOTHING			"No protocol"	// No synchronization protocol
#define YASASPN_PIP				"PIP"			// Priority Inheritance Protocol = Basic Priority Inheritance Protocol (used in RTLinux _RTL_POSIX_THREAD_PRIO_PROTECT)
#define YASASPN_CSP				"CSP"			// Ceiling Semaphore Protocol sometimes called IPCP=Instant Priority Ceiling Protocol
#define YASASPN_PCP				"PCP"			// Priority Ceiling Protocol
#define YASASPN_DPCP			"DPCP"			// Dynamic Priority Ceiling Protocol
#define YASASPN_KP				"KP"			// Kernelized monitor
#define YASASPN_SRP				"SRP"			// source code partly from  "Ismael Ripoll & Patricia Balbastre"
#define YASASPN_UNKNOWN			"Unknown protocol"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definitions to support Priority Ceiling Protocol (PCP)
typedef struct
{
	YASA_MUTEX		*mutex;
} YASA_PCP_STACK;

typedef struct
{
	int				ceiling;
	int				stackpos;		// position in YASA_PCP_STACK, until threads must be waked up after a mutex unlock
	YASA_THREAD		*thread;
} YASA_PCP_SYSTEM_CEILING_STACK;
#define YASA_PCP_MAXSTACK			YASA_MUTEX_MAXSTACK







//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// additional function declarations to support mutexes in executive
//int	 yasa_mutex_init( YASA_MUTEX *mutex, YASA_MUTEX_ATTR *attr);
int yasa_mutex_try_lock(YASA_MUTEX *mutex);
void yasa_mutex_unlock(YASA_MUTEX *mutex);
void yasa_calculateceilings(YASA_MUTEX_ATTR *mutexattr, YASA_THREAD_ATTR *threadattr);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif															// ifdef YASA_RESOURCES_INCLUDE
