//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA global declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/include/yasa/yasa_types.h,v 1.3 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_TYPES_INCLUDE
#define	YASA_TYPES_INCLUDE

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Thread IDs
#define YASA_IDLE_ID				-1
#define YASA_SCHEDULER_ID			-2
#define YASA_YASASUBSYSTEM			-3					// the yasa sub system has done something
#define YASA_THREAD_UNKNOWN			-4


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// global definitions
#define YASA_VERSION				1
#define YASA_MAXNAME				20					// max length of task names
#define YASA_DEFAULT_NAME		 	"NoName"
#define YASA_DEFAULT_ID				0
#define YASA_LOGFILE_NAME			"logfile"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// predefined scheduler types
//#define YASA_DEFAULT_SCHEDULER		0

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// scheduler types
#define YASA_SCHEDULER_MINTYPE		0
#define YASA_SCHEDULER_STATIC		0
#define YASA_SCHEDULER_DYNAMIC		1
#define YASA_SCHEDULER_MAXTYPE		1


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread types
#define YASA_THREAD_MINTYPE			0
#define YASA_THREAD_NONPERIODIC		0
#define YASA_THREAD_PERIODIC		1
#define YASA_THREAD_ASYNCHRONOUS	2					// sporadic (with deadline) and asynchronous (without deadline)
#define YASA_THREAD_MAXTYPE			2

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// behaviour if deadlines missed
#define YASA_DL_MIN					0
#define YASA_DL_IGNORE				0
#define YASA_DL_CANCEL_THREAD		1
#define YASA_DL_SKIPOVER			2					// Skip next thread
#define YASA_DL_EMERGENCYTHREAD		3
#define YASA_DL_MAX					3
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// behaviour if semaphore locked
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Supported synchronization protocols
#define YASASP_MIN					0
#define YASASP_NOTHING				0	// No synchronization protocol
#define YASASP_PIP					1	// Priority Inheritance Protocol = Basic Priority Inheritance Protocol (used in RTLinux _RTL_POSIX_THREAD_PRIO_PROTECT)
#define YASASP_CSP					2	// Ceiling Semaphore Protocol sometimes called IPCP=Instant Priority Ceiling Protocol
#define YASASP_PCP					3	// Priority Ceiling Protocol
#define YASASP_DPCP					4	// Dynamic Priority Ceiling Protocol
#define YASASP_KP					5	// Kernelized monitor
#define YASASP_SRP					6	// source code partly from  "Ismael Ripoll & Patricia Balbastre"
 										// Stack Resource Policy implementation (2001-07-20)
#define YASASP_MAX					6

//#define YASA_SL_WAIT				0					// no specific protocol
//#define YASA_SL_MUTEX_BPIP		1					// Basic priority inheritance protocol


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Logging types
// Attention: THREAD logging types should be defined in one solig numbered block (mutex logging types too)
#define YASAL_UNKNOWN					0
#define YASAL_THREAD_CREATED			1		// when should the task start (make a flash)?, param=required time
#define YASAL_THREAD_ACTIVATE			2		// when should the task start (make a flash)?, param=required time
#define YASAL_THREAD_EXIT				3		// thread is exiting
#define YASAL_THREAD_START				4		// start of computing (make opened bracket)
#define YASAL_THREAD_CONT				5		// task is continuing, param: activation time
#define YASAL_THREAD_SUSPEND			6		// task finished (make closed bracket), param: activation time
#define YASAL_THREAD_TOLERANCE_START	7		// start of computing after deadline, param: activation time
#define YASAL_THREAD_TOLERANCE_CONT		8		// task is continuing, param: activation time
#define YASAL_THREAD_TOLERANCE_SUSPEND	9		// thread suspends after deadline, param: activation time
#define YASAL_THREAD_SKIPPED			10		// time: activation time, param: deadline
#define YASAL_THREAD_TYPE				11		// task type changed, param=type
#define YASAL_THREAD_EXCLUDED			12		// task is excluded (param=1=yes)
#define YASAL_DEADLINE_MATCH			13		// upward arrow (black), time: now, param: current deadline // param=activation time
#define YASAL_DEADLINE_MISS				14		// make upward arrow (red), time: now, param: current deadline, param=activation time
#define YASAL_DEADLINE_MISS_DETECTED	15		// deadline was detected at time param=detection time
#define YASAL_DEADLINE_TOLERANCE_MISS	16		// a deadline tolerance was missed, param=activation time
#define YASAL_TIME_TO_COMPUTE			17		// time to compute at the moment the scheduler detected a deadline miss
#define YASAL_MUTEX_CREATED				18		// mutex was created
#define YASAL_MUTEX_DESTROYED			19		// mutex was destroyed
#define YASAL_MUTEX_LOCKED				20		// mutex was locked successfully
#define YASAL_MUTEX_TRY_N_BLOCKED		21		// thread tried to lock a mutex and was blocked
#define YASAL_MUTEX_UNLOCKED			22		// semaphore was unlocked successfully
#define YASAL_MUTEX_RELOCKED			23		// mutex was relocked
#define YASAL_MUTEX_NOT_LOCKED			24		// mutex was not locked
#define YASAL_MUTEX_PRIORITY_CHANGED	25		// during mutex lock the priority of the thread is changed
#define YASAL_MUTEX_SYSTEM_CEILING		26		// system ceiling changed
#define YASAL_MESSAGE					27		// make message flag
#define YASAL_EMERGENCY_THREAD			28		// emergency thread activated, param=id of cancelled thread
#define YASAL_SCHEDULER_CHANGED			29		// scheduler was changed
#define YASAL_SCHEDULER_CALLED			30		// the scheduler was called
#define YASAL_SCHEDULER_FINISHED		31		// param=when the scheduler should be called again
#define YASAL_SCHEDULER_SWITCH			32		// param=old thread id
#define YASAL_SCHEDULER_SYNCHRONIZED_START 33	// param=
#define YASAL_IDLE_THREAD				34		// the scheduler did not select a thread -> idle !
#define YASAL_JITTER					35		// param=end of jitter
#define YASAL_MAX						35
#define YASAL_BUFFER_FULL				255

#define YASAL_THREADNAME_POS			100		// position(column) of thread name in logfile
#define YASAL_MUTEXNAME_POS				100		// position(column) of mutex name in logfile
#define YASAL_THREADID_POS				82		// position(column) of thread id in logfile
#define YASAL_MUTEXID_POS				82		// position(column) of mutex id in logfile
#define YASAL_THREAD_EXCLUDED_POS		82		// position(column) of description of thread state in logfile
#define YASAL_YASAPARAM_POS				82		// position(column) of a YASATIME parameter

// strings, which will be written to logfiles after simulation
#define YASALS_YASA_VERSION				"Yasa version     : "
#define YASALS_EXECUTIVE_NAME			"Executive name   : "
#define YASALS_EXECUTIVE_VERSION		"Executive version: "
#define YASALS_CPU_NUMBER				"CPU number       : "
#define YASALS_SCHEDULER_CALLED			"Scheduler called : "
#define YASALS_LOGFILE_SIZE				"Max. logfile size: "
#define YASALS_LOGFILE_POSITION			"Logfile position : "
#define YASALS_CURRENT_TIME				"Current end time : "
#define YASALS_SCHEDULER_START			"Scheduler start  : "
#define YASALS_END_TIME					"Scheduled endtime: "
#define YASALS_DURATION					"Duration         : "
#define YASALS_SCHEDULER_TICK			"Scheduler tick   : "
#define YASALS_SCHEDULING_TIME			"Scheduling time  : "
#define YASALS_CALC_PREEMPTION_TIME		"Calculate preemption time: "
#define YASALS_NAME_OF_SCHEDULER		"Name of scheduler        : "
#define YASALS_VERSION_OF_SCHEDULER		"Version of scheduler     : "
#define YASALS_NEW_LINE					"\n"
#define YASALS_IDENTIFIER				"YL: "
/*
#define YASALS_IDENTIFIER_START			"YSTART: "
#define YASALS_IDENTIFIER_GLOBAL		"YG: "
#define YASALS_IDENTIFIER_LOG			"YL: "
#define YASALS_IDENTIFIER_END			"YEND: "
*/
#define YASALS_THREAD					"thread"
#define YASALS_EXCLUDED					"            excluded"
#define YASALS_NOT_EXCLUDED				"        not excluded"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// resource actions
#define YASAA_ACTION_MIN				0
#define YASAA_NO_MORE_ACTION			0
#define YASAA_MUTEX_LOCK				1
#define YASAA_MUTEX_UNLOCK				2
#define YASAA_ACTION_MAX				2



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif														// ifndef YASA_INCLUDE
