//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA logging implementation
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/yasa/yasa_logging.c,v 1.2 2003/01/23 09:18:33 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef YP_ENABLE_LOGGING

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread context: linux kernel
void yasa_logfileconstructor(YASA_SCHEDULER *sched, YASA_LOGFILE *logfile)
{
	logfile->logbuffer=0;
	logfile->logbuffersize=0;
	logfile->logbufferpos=0;
	logfile->locked=0;
	logfile->schedulercounts=0;
	logfile->scheduler=sched;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread context: linux kernel
int yasa_initlogging(YASA_LOGFILE *logfile, size_t size)
{
	if ( size<sizeof(YASA_LOG_ENTRY) )
		return YASAE_LOGBUFFER_TOO_SHORT;

	if ( ! logfile )
		return YASAE_UNKNOWN_SCHEDULER;

	// remove old logfile
	yasa_stoplogging(logfile);
	yasa_locklogfilewait( logfile );

	logfile->logbuffer = YASA_MALLOC(sizeof(char)*size);
	if ( logfile->logbuffer )
	{
		logfile->logbuffersize=size;
		yasa_resetlogging(logfile);
		yasa_unlocklogfile(logfile);
		return YASAE_NO_ERROR;
	}
	else
	{
		yasa_unlocklogfile(logfile);
		return YASAE_OUT_OF_MEMORY;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Thread context: Linux
void yasa_resetlogging(YASA_LOGFILE *logfile)
{
	logfile->logbufferpos=0;
	logfile->schedulercounts=0;
	((YASA_LOG_ENTRY*)logfile->logbuffer)->type=YASAL_UNKNOWN;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread context: linux kernel
void yasa_stoplogging(YASA_LOGFILE *logfile)
{
	// wait until exclusive access to logfile
	yasa_locklogfilewait( logfile );

	if ( logfile->logbuffer )
	{
		YASA_FREE(logfile->logbuffer);
	}
	logfile->logbuffer=0;
	logfile->logbuffersize=0;
	logfile->logbufferpos=0;
	logfile->schedulercounts=0;

	yasa_unlocklogfile(logfile);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
int yasa_snprintf(char *buffer, int size, const char *string, ...)
{

}
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread context: linux kernel
void yasa_printlonglong(void *handle, unsigned INT64 time)
{
	const unsigned INT64 table[]=
	{
	  //18446744073709551616-1=2^64
		10000000000000000000,
		1000000000000000000,
		100000000000000000,
		10000000000000000,
		1000000000000000,
		100000000000000,
		10000000000000,
		1000000000000,
		100000000000,
		10000000000,
		1000000000,
		100000000,
		10000000,
		1000000,
		100000,
		10000,
		1000,
		100,
		10,
		1,
		0
	};

	const unsigned INT64	*ptr=table;
	int			j;
	char		a;
	char		buffer[2];
	int			hidezero=1;


	for ( j=0; j<20; j++)
	{
		a=0;
		while ( time >= *ptr )
		{
			a++;
			time=time - (*ptr);
		};
		if ( a || !hidezero || j==19 )
		{
			buffer[0]=a+'0';
			buffer[1]=0;
			YASA_LOGWRITE(handle, buffer);
			//PRINTK("%c", a+'0' );
			hidezero=0;
		}
		else
			YASA_LOGWRITE(handle, " ");
		ptr++;
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yasa_writeloggingend(void *handle)
{
	char	buffer[2];

	buffer[0]=YASA_LOGEND;
	buffer[1]=0;
	YASA_LOGWRITE(handle, buffer);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread context: linux kernel
int yasa_printlogfile(YASA_LOGFILE *logfile, const YASA_TIME *schedulerstart, int cpu, void* handle)
{
	YASA_LOG_ENTRY	*log;
	unsigned int	pos,i;
	char			*param;
	char			buffer[LOGBUFFERSIZE];
	YASA_SCHEDULER	*sched=(YASA_SCHEDULER*)(logfile->scheduler);
	YASA_TIME		now;

	// arguments are valid ?
	if ( !logfile )
	{
		PRINTK("LOG: No logbuffer allocated\n");
		//YASA_LOGWRITE(handle,"LOG: No logbuffer allocated\n");
		yasa_writeloggingend(handle);
		return EINVAL;
	}
	if ( !schedulerstart || !handle || !logfile->logbuffer )
	{
		PRINTK("LOG: Invalid arguments\n");
		//YASA_LOGWRITE(handle,"LOG: Invalid arguments\n");
		yasa_writeloggingend(handle);
		return EINVAL;
	}

	// wait until exclusive access to logfile
	yasa_locklogfilewait( logfile );

	// print yasa version
	// YASALS_YASA_VERSION
	YASA_LOGWRITE(handle, YASALS_YASA_VERSION );
	YASA_LOGWRITE(handle, "1.0" );
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);

	// print executive name
	YASA_LOGWRITE(handle, YASALS_EXECUTIVE_NAME);					// "Name of executive    : "
	YASA_LOGWRITE(handle, YASA_EXECUTIVE_NAME );
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);

	// print executive version
	YASA_LOGWRITE(handle, YASALS_EXECUTIVE_VERSION);				// "Version of executive : "
	YASA_LOGWRITE(handle, YASA_EXECUTIVE_VERSION );
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);

	// print cpu number
	YASA_LOGWRITE(handle, YASALS_CPU_NUMBER);							// "CPU number      : "
	yasa_printlonglong( handle, (YASA_TIME)sched->cpu );

	// print scheduling counts
#ifdef YASA_SUPPORT_LOG_COUNT_SCHEDULER_CALLS
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_SCHEDULER_CALLED);						// "\nScheduler called: "
	yasa_printlonglong( handle, (YASA_TIME)logfile->schedulercounts );
#endif

	// print logfile size
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_LOGFILE_SIZE);							// "Logfile size    : "
	yasa_printlonglong( handle, (YASA_TIME)logfile->logbuffersize );

	// print logfile pos
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_LOGFILE_POSITION);						// "Logfile position: "
	yasa_printlonglong( handle, (YASA_TIME)logfile->logbufferpos );

	// print scheduler start
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_SCHEDULER_START);						// "Scheduler start : "
	yasa_printlonglong( handle, sched->schedulerstart );

	// print end time
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_END_TIME );							// "End time        : "
	yasa_printlonglong( handle, sched->schedulerend );

	// print now
	now=YASA_GETCURRENTTIME(sched);
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_CURRENT_TIME);							// "Current time    : "
	yasa_printlonglong( handle, now );

	// print duration
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_DURATION);								// "Duration        : "
	//yasa_printlonglong( handle, sched->schedulerend-sched->schedulerstart );
	yasa_printlonglong( handle, now-sched->schedulerstart );

	// print scheduler tick
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_SCHEDULER_TICK );						// "Scheduler tick  : "
	yasa_printlonglong( handle, sched->tick );

	// print scheduling time
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_SCHEDULING_TIME );						// "Scheduling time : "
#ifdef YP_SCHEDULINGTIME
	yasa_printlonglong( handle, YP_SCHEDULINGTIME );
#else
	yasa_printlonglong( handle, 0 );
#endif

	// Calculate preemption time
	YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	YASA_LOGWRITE(handle, YASALS_CALC_PREEMPTION_TIME );				// "Calculate preemption time: "
	if ( sched->calcpreemptiontime )
		YASA_LOGWRITE(handle, "yes" );
	else
		YASA_LOGWRITE(handle, "no" );

	// print scheduler name
	if ( YASA_SCHEDULER_ATTR(sched) )
	{
		YASA_LOGWRITE(handle, YASALS_NEW_LINE);
		YASA_LOGWRITE(handle, YASALS_NAME_OF_SCHEDULER);				// "Name of scheduler        : "
		YASA_LOGWRITE(handle, yasa_id_get_name( & YASA_SCHEDULER_ATTR(sched)->id ) );
		YASA_LOGWRITE(handle, YASALS_NEW_LINE);
		YASA_LOGWRITE(handle, YASALS_VERSION_OF_SCHEDULER);				// "Version of scheduler     : "
		YASA_LOGWRITE(handle, YASA_SCHEDULER_ATTR(sched)->scheduler_version );
		YASA_LOGWRITE(handle, YASALS_NEW_LINE);
	}


	// print log entries to pipe/stdout/kernelout
	pos=0;
	while ( pos<logfile->logbufferpos )
	{
		log=(YASA_LOG_ENTRY*) &(logfile->logbuffer[pos]);
		param = (log->length == sizeof(YASA_LOG_ENTRY) ) ? "" : ((char*)log)+sizeof(YASA_LOG_ENTRY);
		YASA_LOGWRITE(handle, YASALS_IDENTIFIER);						// "YL: "
		yasa_printlonglong( handle, log->time - *schedulerstart );

		//YASA_LOGWRITE2(" cpu: %2d, thread: %3d,", cpu, log->threadid);
		YASA_LOGWRITE2(" %s: %3d,", YASALS_THREAD, log->threadid);		// // " thread: "

		switch ( log->type )
		{
			default:
			case YASAL_UNKNOWN:				YASA_LOGWRITE3(" %2d: %s, param          : %s\n",
															YASAL_UNKNOWN, YASALT_UNKNOWN	, param	);
											break;
			case YASAL_THREAD_CREATED:		YASA_LOGWRITE4(" %2d: %s, thread id      : %10d, name: \"%s\"", YASAL_THREAD_CREATED,
												YASALT_THREAD_CREATED		, ((YASA_LOG_ENTRY_THREADPARAM*)param)->id.id, (const char*)&(((YASA_LOG_ENTRY_THREADPARAM*)param)->id.name) );

											//for (i=strlen( (const char*)&(((YASA_LOG_ENTRY_THREADPARAM*)param)->id.name ) ); i<YASA_MAXNAME; i++)
											//	YASA_LOGWRITE(handle, " ");
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_ACTIVATE:		YASA_LOGWRITE2(" %2d: %s, sch. activation: "	 ,
															YASAL_THREAD_ACTIVATE	,YASALT_THREAD_ACTIVATE	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_EXIT:			YASA_LOGWRITE3(" %2d: %s, retvalue       : %20p\n"	 ,
															YASAL_THREAD_EXIT		,YASALT_THREAD_EXIT		, *(void**)param );
											break;
			//case YASAL_THREAD_CANCEL:		YASA_LOGWRITE2(" %2d: %s, no param\n"	 ,
			//												YASAL_THREAD_CANCEL		,YASALT_THREAD_EXIT		);
			//								break;
			case YASAL_THREAD_START:		YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_START		,YASALT_THREAD_START	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_CONT:			YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_CONT		,YASALT_THREAD_CONT		);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_SUSPEND:		YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_SUSPEND	,YASALT_THREAD_SUSPEND	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_TOLERANCE_START:YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_TOLERANCE_START,YASALT_THREAD_TOLERANCE_START	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_TOLERANCE_CONT:YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_TOLERANCE_CONT		,YASALT_THREAD_TOLERANCE_CONT);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_TOLERANCE_SUSPEND:YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_TOLERANCE_SUSPEND	,YASALT_THREAD_TOLERANCE_SUSPEND	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_SKIPPED:		YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_THREAD_SKIPPED	,YASALT_THREAD_SKIPPED	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_THREAD_TYPE:			YASA_LOGWRITE2(" %2d: %s, thread type    : ", YASAL_THREAD_TYPE				, YASALT_THREAD_TYPE);
											switch ( *param )
											{
												case YASALTT_NON_PERIODIC:		YASA_LOGWRITE(handle, "non periodic\n");
																				break;
												case YASALTT_PERIODIC:			YASA_LOGWRITE(handle, "periodic\n");
																				break;
												case YASALTT_PENDING_PERIODIC:	YASA_LOGWRITE(handle, "periodic pending\n");
																				break;
												default:						YASA_LOGWRITE(handle, "unknown thread type\n");
																				break;
											}
											break;
			case YASAL_THREAD_EXCLUDED:		YASA_LOGWRITE2(" %2d: %s, thread mode    : ", YASAL_THREAD_EXCLUDED			, YASALT_THREAD_EXCLUDED);
											switch ( *param )
											{
												case 1:		YASA_LOGWRITE(handle, YASALS_EXCLUDED);
															break;
												default:	YASA_LOGWRITE(handle, YASALS_NOT_EXCLUDED);
															break;
											}
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_DEADLINE_MATCH:		YASA_LOGWRITE2(" %2d: %s, deadline       : "    ,
															YASAL_DEADLINE_MATCH	,YASALT_DEADLINE_MATCH	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_DEADLINE_MISS:		YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_DEADLINE_MISS		,YASALT_DEADLINE_MISS	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_DEADLINE_MISS_DETECTED:YASA_LOGWRITE2(" %2d: %s, deadline       : "    ,
															YASAL_DEADLINE_MISS_DETECTED, YASALT_DEADLINE_MISS_DETECTED	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_DEADLINE_TOLERANCE_MISS:YASA_LOGWRITE2(" %2d: %s, activation time: "    ,
															YASAL_DEADLINE_TOLERANCE_MISS		,YASALT_DEADLINE_TOLERANCE_MISS	);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_TIME_TO_COMPUTE:		YASA_LOGWRITE2(" %2d: %s, time to compute: "    ,
															YASAL_TIME_TO_COMPUTE	,YASALT_TIME_TO_COMPUTE	);
											yasa_printlonglong( handle, *(YASA_TIME*)param);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_MUTEX_CREATED:		YASA_LOGWRITE4(" %2d: %s, mutex id       : %10d, name: \"%s\"", YASAL_MUTEX_CREATED, YASALT_MUTEX_CREATED
												, ((YASA_LOG_ENTRY_MUTEXPARAM*)param)->id.id, (const char*)&((YASA_LOG_ENTRY_MUTEXPARAM*)param)->id.name );
											for (i=strlen( (const char*)&(((YASA_LOG_ENTRY_MUTEXPARAM*)param)->id.name ) ); i<YASA_MAXNAME; i++)
												YASA_LOGWRITE(handle, " ");
											YASA_LOGWRITE(handle, ", protocol: ");
											switch ( ((YASA_LOG_ENTRY_MUTEXPARAM*)param)->protocol )
											{
											case YASASP_NOTHING:	YASA_LOGWRITE(handle,YASASPN_NOTHING);
																	break;
											case YASASP_PIP:		YASA_LOGWRITE(handle,YASASPN_PIP);
																	break;
											case YASASP_CSP:		YASA_LOGWRITE(handle,YASASPN_CSP);
																	break;
											case YASASP_PCP:		YASA_LOGWRITE(handle,YASASPN_PCP);
																	break;
											case YASASP_DPCP:		YASA_LOGWRITE(handle,YASASPN_DPCP);
																	break;
											case YASASP_KP:			YASA_LOGWRITE(handle,YASASPN_KP);
																	break;
											case YASASP_SRP:		YASA_LOGWRITE(handle,YASASPN_SRP);
																	break;
											default:				YASA_LOGWRITE(handle,YASASPN_UNKNOWN);
																	break;
											}
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_MUTEX_DESTROYED:		YASA_LOGWRITE3(" %2d: %s, mutex id       : %20d\n", YASAL_MUTEX_DESTROYED	, YASALT_MUTEX_DESTROYED	, *(int*)param	);
											break;
			case YASAL_MUTEX_LOCKED:		YASA_LOGWRITE3(" %2d: %s, mutex id       : %20d\n", YASAL_MUTEX_LOCKED		, YASALT_MUTEX_LOCKED		, *(int*)param	);
											break;
			case YASAL_MUTEX_TRY_N_BLOCKED:	YASA_LOGWRITE3(" %2d: %s, mutex id       : %20d\n", YASAL_MUTEX_TRY_N_BLOCKED	, YASALT_MUTEX_TRY_N_BLOCKED, *(int*)param	);
											break;
			case YASAL_MUTEX_UNLOCKED:		YASA_LOGWRITE3(" %2d: %s, mutex id       : %20d\n", YASAL_MUTEX_UNLOCKED		, YASALT_MUTEX_UNLOCKED		, *(int*)param	);
											break;
			case YASAL_MUTEX_RELOCKED:		YASA_LOGWRITE3(" %2d: %s, mutex id       : %20d\n", YASAL_MUTEX_RELOCKED		, YASALT_MUTEX_RELOCKED		, *(int*)param	);
											break;
			case YASAL_MUTEX_NOT_LOCKED:	YASA_LOGWRITE3(" %2d: %s, mutex id       : %20d\n", YASAL_MUTEX_NOT_LOCKED	, YASALT_MUTEX_NOT_LOCKED	, *(int*)param	);
											break;
			case YASAL_MUTEX_PRIORITY_CHANGED:YASA_LOGWRITE3(" %2d: %s, priority       : %20d\n", YASAL_MUTEX_PRIORITY_CHANGED, YASALT_MUTEX_PRIORITY_CHANGED, *(int*)param	);
											break;
			case YASAL_MUTEX_SYSTEM_CEILING:YASA_LOGWRITE3(" %2d: %s, ceiling        : %20d\n", YASAL_MUTEX_SYSTEM_CEILING, YASALT_MUTEX_SYSTEM_CEILING, *(int*)param	);
											break;
			case YASAL_MESSAGE:				YASA_LOGWRITE3(" %2d: %s, message        : %s\n", YASAL_MESSAGE				, YASALT_MESSAGE			, param	);
											break;
			case YASAL_EMERGENCY_THREAD:	YASA_LOGWRITE3(" %2d: %s, param          : %20d\n", YASAL_EMERGENCY_THREAD	, YASALT_EMERGENCY_THREAD	, *param);
											break;
			case YASAL_SCHEDULER_CHANGED:	YASA_LOGWRITE3(" %2d: %s, to             : %s\n", YASAL_SCHEDULER_CHANGED	, YASALT_SCHEDULER_CHANGED	, param	);
											break;
			case YASAL_SCHEDULER_CALLED:	YASA_LOGWRITE2(" %2d: %s, scheduled start: ", YASAL_SCHEDULER_CALLED	, YASALT_SCHEDULER_CALLED				);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_SCHEDULER_FINISHED:	YASA_LOGWRITE2(" %2d: %s, next scheduling: "    , YASAL_SCHEDULER_FINISHED	, YASALT_SCHEDULER_FINISHED			);
//											yasa_printlonglong( handle, *(YASA_TIME*)param);
											if ( *(YASA_TIME*)param == YASA_TIME_INFINITY )
												*(YASA_TIME*)param=*schedulerstart;
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
			case YASAL_SCHEDULER_SWITCH:	YASA_LOGWRITE2(" %2d: %s\n"    , YASAL_SCHEDULER_SWITCH	, YASALT_SCHEDULER_SWITCH	);
											break;
			case YASAL_SCHEDULER_SYNCHRONIZED_START:
											YASA_LOGWRITE2(" %2d: %s\n"    , YASAL_SCHEDULER_SYNCHRONIZED_START, YASALT_SCHEDULER_SYNCHRONIZED_START	);
											break;
			case YASAL_IDLE_THREAD:			YASA_LOGWRITE3(" %2d: %s, param          : %s\n", YASAL_IDLE_THREAD			, YASALT_IDLE_THREAD			, param	);
											break;
			case YASAL_JITTER:				YASA_LOGWRITE2(" %2d: %s, jitter end     : ", YASAL_JITTER				, YASALT_JITTER					);
											yasa_printlonglong( handle, *(YASA_TIME*)param-*schedulerstart);
											YASA_LOGWRITE(handle, "\n");
											break;
#if YASAL_MAX!=35
#error There are inconsistences handling logging types!
#endif
		}
		pos+= log->length;
	};
	yasa_writeloggingend(handle);			// write eof
	yasa_resetlogging( logfile );
	yasa_unlocklogfile(logfile);

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is logging an entry.
//! thread context: scheduler, rt-kernel
//! Do not call from different cpu!
//! id=thread id
void yasa_logscheduler(YASA_SCHEDULER *sched, int id, char type, const YASA_TIME *time, const void *param)
{
	size_t			size;
	YASA_LOG_ENTRY	*log;

	// get exclusive access to logfile. Do not wait
	if ( yasa_locklogfile(&sched->logfile) )			// logging is disabled during "logfile reading"
	{
		// is buffer allocated ?
		if ( sched->logfile.logbuffer )
		{
			// is buffer already full ?
			log=(YASA_LOG_ENTRY*) &(sched->logfile.logbuffer[sched->logfile.logbufferpos]);
			if ( sched->logfile.logbufferpos < sched->logfile.logbuffersize - sizeof(YASA_LOG_ENTRY) )
			{
				log->time= ( time ) ? *time : YASA_GETCURRENTTIME(sched);

				log->threadid=id;
				size=0;
				switch (type)
				{
					case YASAL_THREAD_EXIT:		if (param)
													size=sizeof(void*);
												break;

					// param: optional message
					default:					param="Illegal type";
					case YASAL_SCHEDULER_CHANGED:
					case YASAL_MESSAGE:			if ( param)
													size=strlen(param)+1;
												break;
					case YASAL_THREAD_EXCLUDED:
					case YASAL_THREAD_TYPE:		if ( param)
													size=sizeof(int);
												break;

					case YASAL_THREAD_CREATED:	if (param)
													size=sizeof(YASA_LOG_ENTRY_THREADPARAM);
												break;

					case YASAL_MUTEX_CREATED:	if ( param)
													size=sizeof(YASA_LOG_ENTRY_MUTEXPARAM);
												break;
					// param: a time value
					case YASAL_SCHEDULER_CALLED:
					case YASAL_JITTER:
					case YASAL_THREAD_ACTIVATE:
					case YASAL_THREAD_START:
					case YASAL_THREAD_CONT:
					case YASAL_THREAD_SUSPEND:
					case YASAL_THREAD_TOLERANCE_START:
					case YASAL_THREAD_TOLERANCE_CONT:
					case YASAL_THREAD_TOLERANCE_SUSPEND:
					case YASAL_THREAD_SKIPPED:
					case YASAL_SCHEDULER_FINISHED:
					case YASAL_TIME_TO_COMPUTE:
					case YASAL_DEADLINE_MATCH:
					case YASAL_DEADLINE_TOLERANCE_MISS:
					case YASAL_DEADLINE_MISS_DETECTED:
					case YASAL_DEADLINE_MISS:	size=sizeof(YASA_TIME);
												break;

					case YASAL_MUTEX_TRY_N_BLOCKED:
					case YASAL_MUTEX_DESTROYED:
					case YASAL_MUTEX_LOCKED:
					case YASAL_MUTEX_UNLOCKED:
					case YASAL_MUTEX_RELOCKED:
					case YASAL_MUTEX_NOT_LOCKED:
					case YASAL_MUTEX_PRIORITY_CHANGED:
					case YASAL_MUTEX_SYSTEM_CEILING:
					case YASAL_EMERGENCY_THREAD:
												size=sizeof(int);
												break;
					case YASAL_SCHEDULER_SYNCHRONIZED_START:
					case YASAL_SCHEDULER_SWITCH:
					case YASAL_IDLE_THREAD:
					//case YASAL_THREAD_CANCEL:
												break;
				}
#if YASAL_MAX!=35
#error There are inconsistences handling logging types!
#endif

				// copy param
				if ( size && size < sched->logfile.logbuffersize-sched->logfile.logbufferpos-sizeof(YASA_LOG_ENTRY) )
				{
					if ( param )
					{
						memcpy( ((char*)log)+sizeof(YASA_LOG_ENTRY), param, size);
					}
					else
					{
						PRINTK("LOG: Param Error!\n");
						size=0;
					}
				}

				size=(size+sizeof(YASA_LOG_ENTRY)+1) & ~(1);		// align
				//printf("nr: %d\n", sched->yasa_logbufferpos);
				if ( sched->logfile.logbufferpos+size+sizeof(YASA_LOG_ENTRY) <= sched->logfile.logbuffersize )
				{
					//log->threadid=thread->yasa_id;
					log->type=type;
					log->length=size;
					sched->logfile.logbufferpos+=size;
				}
/*				else
				{
					l2++;
					if ( l2<10 )
						PRINTK("size=%d\n", size);
					log->type=YASAL_BUFFER_FULL;
					log->length=sizeof(YASA_LOG_ENTRY);
				}
			}
			else
				PRINTK("Buffer 123 full\n");
*/			}
		}
		yasa_unlocklogfile( &sched->logfile );
	}
/*	else
	{
		l1++;
		if ( l1<10 )
			PRINTK("Log error\n");
	}
*/
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yasa_logentry(YASA_THREAD *thread, char type, const YASA_TIME *time, const void *param)
{
	if ( !thread )
	{
		PRINTK("YLLOG: Fatal Error!\n");
		return;
	}
	// is buffer already full ?
	yasa_logscheduler( YASA_GETSCHEDULER(thread), yasa_id_get_id(&thread->id), type, time, param);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// return value:
// 1=locked by this function, 0=could not be locked (already locked)
// thread context: linux kernel and scheduler!!!
char yasa_locklogfile(YASA_LOGFILE *logfile)
{
	YASA_LOCKLOGFILE(logfile)
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// return value:
// 1=locked by this function, otherwise 0
// thread context: linux kernel (not scheduler!!!)
char yasa_locklogfilewait(YASA_LOGFILE *logfile)
{
	YASA_LOCKLOGFILEWAIT(logfile)
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread context: linux kernel and scheduler!!!
void yasa_unlocklogfile(YASA_LOGFILE *logfile)
{
	YASA_UNLOCKLOGFILE(logfile)
}


#endif														// #ifdef YASA_ENABLE_LOGGING
