//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Action handler implementation
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/actionhandler.cpp,v 1.3 2003/01/23 09:18:34 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "actionhandler.h"
#include "actions.h"
#include "application.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YActionHandler::InitInstance(QWidget *widget)
{
	int count;
	int	retvalue;
	YActions	*ptr;
	parentwidget=widget;
	for ( count=0; count<MAX_ACTIONS ; count++ )
		actions[count]=0;

//	actions[WORKSPACE_ACTIONS]			= new YWorkSpaceActions(*this);
	actions[PROJECTSET_ACTIONS]			= new YProjectSetActions(*this);
	actions[PROJECT_ACTIONS]			= new YProjectActions(*this);
	actions[TASKSET_ACTIONS]			= new YTaskSetActions(*this);
	actions[TASK_ACTIONS]				= new YTaskActions(*this);
	actions[MUTEXSET_ACTIONS]			= new YMutexSetActions(*this);
	actions[MUTEX_ACTIONS]				= new YMutexActions(*this);
	actions[ENVIRONMENTSET_ACTIONS]		= new YEnvironmentSetActions(*this);
	actions[ENVIRONMENT_ACTIONS]		= new YEnvironmentActions(*this);
	actions[SCHEDULERSET_ACTIONS]		= new YSchedulerSetActions(*this);
	actions[SCHEDULER_ACTIONS]			= new YSchedulerActions(*this);
	actions[SOURCESET_ACTIONS]			= new YSourceSetActions(*this);
	actions[FILEPATH_ACTIONS]			= new YFilePathActions(*this);
	actions[HEADERSET_ACTIONS]			= new YHeaderSetActions(*this);
	actions[RESULTS_ACTIONS]			= new YResultsActions(*this);
	actions[TIMING_RESULT_ACTIONS]		= new YTimingResultActions(*this);
	actions[STATISTIC_RESULT_ACTIONS]	= new YStatisticResultActions(*this);
	for ( count=0; count<MAX_ACTIONS ; count++ )
	{	if ( actions[count]==0 )
		{
			qWarning("YActionHandler::InitInstance() action[] is zero. That should not be possible!");
			qWarning("Did you forget to include a \"new\" action command?");
			return YASAE_OUT_OF_MEMORY;
		}
		ptr=actions[count];
		retvalue=actions[count]->InitInstance();
		if ( retvalue )
			return retvalue;
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! members of QArray will be destroyed automatically and before entering the destructor
YActionHandler::~YActionHandler()
{
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YActionHandler::UpdateActions()
{
	for ( int count=0; count<MAX_ACTIONS ; count++ )
	{
		actions[count]->UpdateActions();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YActionHandler::SetCurrentObject(YObject* object)
{
	YActionHandler	*ah=((YApplication*)qApp)->GetActionHandler();
	if ( !ah )
		return;

	if ( ah->object )
	{
		disconnect( ah->object, SIGNAL( destroyed() ), ah, SLOT( ObjectInvalid() ) );
		ah->object=0;
	}

	if ( object )
	{
		if ( object->GetActions() )
		{
			connect( object, SIGNAL( destroyed() ), ah, SLOT( ObjectInvalid() ) );
			ah->object=object;
			return;
		}
		else
			qWarning("YActionHandler::SetCurrentObject(): object->actions not defined !\n");
	}
	else
		qWarning("YActionHandler::SetCurrentObject(): object is zero !\n");
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YActions* YActionHandler::GetActions(unsigned int a)
{
	return ((YApplication*)qApp)->GetActionHandler()->_GetActions(a);
}
