//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Action code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/actions.cpp,v 1.3 2003/01/23 09:18:34 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "actionhandler.h"
#include "actions.h"
#include "qinputdialog.h"
#include "qmessagebox.h"
#include "mainwindow.h"
#include "projectwindow.h"
#include "taskwindow.h"
#include "mutexwindow.h"
#include "environmentwindow.h"
#include "sourcesview.h"
#include "helpwindow.h"
#include "workspacewindow.h"
#include "application.h"
#include "environment.h"
#include "timingwindows.h"
#include "timinghandler.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YActions::YActions(YActionHandler &ah) :
setactions(MAX_ACTIONS),
actionhandler(ah)
{}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YActions::InitInstance()
{
	setactions[ACTIVATE] = new QAction( MSGP_PROPERTIES			, MSGP_PROPERTIES				, 0	, this );
	for ( int count=0; count<MAX_ACTIONS ; count++ )
	{	if ( setactions[count]==0 )
			return YASAE_OUT_OF_MEMORY;
	}
	connect( setactions[ACTIVATE], SIGNAL( activated() ), this, SLOT( _activate() ) );
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YActions::UpdateActions()
{
	if ( actionhandler.GetCurrentObject() )
	{
		setactions[ACTIVATE]->setEnabled(true);
	}
	else
	{
		setactions[ACTIVATE]->setEnabled(false);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YActions::_CreateMenu(YObject *object, QPopupMenu *menu) const
{
	if ( setactions[ACTIVATE]->addTo( menu ) )
		return 0;
	return YASAE_OUT_OF_MEMORY;
	Q_UNUSED(object);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YActions::CreateMenu(YObject* object, QPopupMenu *parent)
{
	if ( object && object->GetActions() )
	{
		if ( parent )
			return object->GetActions()->_CreateMenu(object, parent);
		else
		{
			qWarning("YActions::CreateMenu(): parentmenu not defined!");
			return 0;
		}
	}
	qWarning("YActions::CreateMenu(): object or object->actions for object \"%s\" not defined!", object->GetName().latin1() );
	return YASAE_INTERNAL_ERROR;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YActions::CreateTreeItems(YObject* object, QListViewItem *parent, QListViewItem *after)
{
	if ( object && object->GetActions() )
	{
		if ( parent )
			return object->GetActions()->_CreateTreeItems(object, parent, after);
		else
		{
			return 0;
		}
	}
	qWarning("YActions::CreateTreeItems(): object or object->actions for object \"%s\" not defined!", object->GetName().latin1() );
	return YASAE_INTERNAL_ERROR;
}

// can be called from the whole application to set a new "currentobject" and to run an "activate" process
int YActions::_Activate(YObject *object)
{
//	qWarning("YActions::Activate() called. That should not be happened! Object: \"%s\"", object->GetName().latin1() );
	//return 0;
	return YASAE_NOT_IMPLEMENTED;
	Q_UNUSED(object);
}

// can be called from the whole application to set a new "currentobject" and to run an "activate" process
int YActions::Activate(YObject *object)
{
	if ( !object )
		qWarning("YActions::Activate() called with invalid ptr to object=0");
	actionhandler.SetCurrentObject(object);
	return _Activate(object);
}
int YActions::_activate()
{
	return Activate( actionhandler.GetCurrentObject() );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSetActions::InitInstance()
{
	setactions[ADD]			= new QAction( MSGP_SETACTION_ADD		, MSGP_SETACTION_ADD_MENU		, 0	, this );
	setactions[REMOVE_ALL]	= new QAction( MSGP_SETACTION_REMOVE_ALL, MSGP_SETACTION_REMOVE_ALL_MENU, 0	, this );
	for ( int count=0; count<MAX_ACTIONS ; count++ )
	{	if ( setactions[count]==0 )
			return YASAE_OUT_OF_MEMORY;
	}
	connect( setactions[ADD]		, SIGNAL( activated() ), this, SLOT( AddEntry() ) );
	connect( setactions[REMOVE_ALL]	, SIGNAL( activated() ), this, SLOT( RemoveAllEntries() ) );
	return YActions::InitInstance();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Insert menu entries into the given menu
//! Object must be a set!
int YSetActions::_CreateMenu(YObject *object, QPopupMenu *menu) const
{
	bool	success=true;
	success&=setactions[ADD]->addTo( menu );
	success&=setactions[REMOVE_ALL]->addTo( menu );
	(menu)->insertSeparator();
	if ( success )
		return YActions::_CreateMenu(object, menu);
	return YASAE_OUT_OF_MEMORY;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSetActions::UpdateActions()
{
	if ( actionhandler.GetCurrentObject() )
		setactions[REMOVE_ALL]->setEnabled(true);
	else
		setactions[REMOVE_ALL]->setEnabled(false);
	YActions::UpdateActions();
}
int YSetActions::RemoveAllEntries()
{	return RemoveAll( actionhandler.GetCurrentObject() );
}
YObject* YSetActions::AddEntry()
{	return Add ( actionhandler.GetCurrentObject() );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YSetActions::Add(YObject *object)
{
	int	retvalue=YASAE_ERROR_IN_ARGUMENTS;
	YObject	*newobject=0;
	if ( object )
	{
		bool	ok;
		QString	name;
		QString	text=MSGP_ACTION_NAME_QUESTION;
		((YSet*)object)->GetUniqueEntryName(name);
		text=text.arg( name );
		name=QInputDialog::getText( MSGP_SETACTION_ADD, text, QLineEdit::Normal, name, &ok, 0);
		retvalue=YASAE_OPERATION_ABORTED;
		if ( ok )
		{
			newobject=((YSet*)object)->CreateNewItem(name);
			if ( newobject && newobject->GetActions() )
			{
				newobject->GetActions()->Activate(newobject);
			}
		}
	}
	else
		qWarning("YSetAction::Add(): set is zero!\n");
	return newobject;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSetActions::RemoveAll(YObject *object) const
{
	if ( object )
	{
		((YSet*)object)->ClearSet();
		return 0;
	}
	else
		qWarning("YAction::RemoveAll(): set is zero!\n");
	return YASAE_ERROR_IN_ARGUMENTS;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSetActions::NewObjectCreated(YObject *object)
{
	YWorkSpace *ws=actionhandler.GetWorkSpaceManager().GetWorkSpace();
	if (ws)
		emit ws->NewObjectCreated(object);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSetActions::ObjectRemoved(YObject *object)
{
	YWorkSpace *ws=actionhandler.GetWorkSpaceManager().GetWorkSpace();
	if (ws)
		emit ws->ObjectRemoved(object);
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEntryActions::InitInstance()
{
	entryactions[RENAME]			= new QAction( MSGP_ENTRYACTION_RENAME		, MSGP_ENTRYACTION_RENAME_MENU		, 0		, this );
	entryactions[REMOVE]			= new QAction( MSGP_ENTRYACTION_REMOVE		, MSGP_ENTRYACTION_REMOVE_MENU		, 0		, this );
	for ( int count=0; count<MAX_ACTIONS ; count++ )
	{	if ( entryactions[count]==0 )
			return YASAE_OUT_OF_MEMORY;
	}
	connect( entryactions[RENAME]		, SIGNAL( activated() ), this, SLOT( RenameEntry() ) );
	connect( entryactions[REMOVE]		, SIGNAL( activated() ), this, SLOT( RemoveEntry() ) );
	return YActions::InitInstance();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEntryActions::_CreateMenu(YObject *object, QPopupMenu *menu) const
{
	bool	success=true;
	success&=entryactions[RENAME]->addTo( menu );
	success&=entryactions[REMOVE]->addTo( menu );
	(menu)->insertSeparator();
	if ( success )
		return YActions::_CreateMenu(object, menu);
	return YASAE_OUT_OF_MEMORY;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEntryActions::UpdateActions()
{
	if ( actionhandler.GetCurrentObject() )
	{
		entryactions[RENAME]->setEnabled(true);
		entryactions[REMOVE]->setEnabled(true);
	}
	else
	{
		entryactions[RENAME]->setEnabled(false);
		entryactions[REMOVE]->setEnabled(false);
	}
	YActions::UpdateActions();
}
int YEntryActions::RenameEntry()
{	return Rename( actionhandler.GetCurrentObject() );
}
int YEntryActions::RemoveEntry()
{	return Remove( actionhandler.GetCurrentObject() );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEntryActions::Remove(YObject *object) const
{
	if ( object )
	{
		if ( object->GetSet() )
		{
			object->GetSet()->RemoveItem(object);
			return 0;
		}
	}
	qWarning("YAction::Remove(): Object is zero or GetSet() failed!");
	return YASAE_ERROR_IN_ARGUMENTS;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEntryActions::Rename(YObject *object) const
{
	int	retvalue=YASAE_ERROR_IN_ARGUMENTS;
	if ( object )
	{
		bool	ok;
		QString	text=MSGP_ACTION_NAME_QUESTION;
		text=text.arg( object->GetName() );
		text=QInputDialog::getText( MSGP_ENTRYACTION_RENAME , text, QLineEdit::Normal, object->GetName(), &ok, 0);
		retvalue=YASAE_OPERATION_ABORTED;
		if ( ok )
		{
			retvalue=object->SetName(text);
		}
	}
	else
		qWarning("YAction::Rename(): Entry is zero!");
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectSetActions::InitInstance()
{
	int retvalue=YSetActions::InitInstance();
	if ( ! retvalue )
	{
		projectsetactions[CLEANUP]			= new QAction( MSGP_CLEANUP_PROJECTS		, MSGP_CLEANUP_PROJECTS_MENU		, 0		, this );
		for ( int count=0; count<MAX_ACTIONS ; count++ )
		{	if ( projectsetactions[count]==0 )
				return YASAE_OUT_OF_MEMORY;
		}
		connect( projectsetactions[CLEANUP]		, SIGNAL( activated() ), this, SLOT( CleanupProjects() ) );
	}
	return retvalue;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectSetActions::_CreateMenu(YObject *object, QPopupMenu *menu) const
{
	bool	success=true;
	success&=projectsetactions[CLEANUP]->addTo( menu );
	if ( success )
		return YSetActions::_CreateMenu(object, menu);
	return YASAE_OUT_OF_MEMORY;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectSetActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YFrameObjectArgs(YWorkSpaceFrame::CreateWorkSpaceFrame, object) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YProjectSetActions::CleanupProjects()
{
	((YProjectSet*)actionhandler.GetCurrentObject() )->CleanupProjects();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectActions::Rename(YObject *object) const
{
	int	retvalue;
	retvalue=YEntryActions::Rename(object);
	if ( ! retvalue )
		((YProject*)object)->SetProjectDir( object->GetName() );
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectActions::InitInstance()
{
	projectactions[CLEANUP]			= new QAction( MSGP_CLEANUP_PROJECT		, MSGP_CLEANUP_PROJECT_MENU		, 0		, this );
	for ( int count=0; count<MAX_ACTIONS ; count++ )
	{	if ( projectactions[count]==0 )
			return YASAE_OUT_OF_MEMORY;
	}
	connect( projectactions[CLEANUP]		, SIGNAL( activated() ), this, SLOT( CleanupProject() ) );
	return YEntryActions::InitInstance();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectActions::_CreateMenu(YObject *object, QPopupMenu *menu) const
{
	bool	success=true;
	success&=projectactions[CLEANUP]->addTo( menu );
	if ( success )
		return YEntryActions::_CreateMenu(object, menu);
	return YASAE_OUT_OF_MEMORY;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YProjectActions::CleanupProject()
{
	((YProject*)actionhandler.GetCurrentObject() )->CleanupProject();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YFrameObjectArgs(YProjectFrame::CreateProjectFrame, object) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProjectActions::Remove(YObject *entry) const
{
	if ( entry )
		((YProject*) entry)->RemoveProjectDirectory();
	return YEntryActions::Remove(entry);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTimingResultActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YTimingFrameObjectArgs(YResultFrame::CreateTimingFrame, (YTimingResults*)object ) );
}

int	YTimingResultActions::DoWindowCommand(YObject *object, YActions::COMMAND_TYPE command, YMDIWindow *window)
{
	YResultFrame	*frame;
	if ( window && object)
	{
		frame=(YResultFrame *)window->GetMainFrame();
		if ( frame )
		{
			switch (command)
			{
			case ZOOM_IN:		((YTimingHandler*)frame->GetResultHandler())->ZoomIn();
								return 0;
			case ZOOM_OUT:		((YTimingHandler*)frame->GetResultHandler())->ZoomOut();
								return 0;
			default:			break;
			}
		}
	}
	return YActions::DoWindowCommand(object, command, window);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YStatisticResultActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YStatisticFrameObjectArgs(YResultFrame::CreateStatisticFrame, (YStatisticResults*)object ) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTaskActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YFrameObjectArgs(YTaskFrame::CreateTaskFrame, object) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMutexActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YFrameObjectArgs(YMutexFrame::CreateMutexFrame, object) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::InitInstance()
{	eactions[EXECUTE]			= new QAction( MSGP_ENVIRONMENT_EXECUTE		, MSGP_ENVIRONMENT_EXECUTE_MENU		, 0		, this );
	eactions[TIMINGS]			= new QAction( MSGP_ENVIRONMENT_TIMINGS		, MSGP_ENVIRONMENT_TIMINGS_MENU		, 0		, this );
	eactions[STATISTICS]		= new QAction( MSGP_ENVIRONMENT_STATISTICS	, MSGP_ENVIRONMENT_STATISTICS_MENU	, 0		, this );
	eactions[LOGFILE]			= new QAction( MSGP_ENVIRONMENT_LOGFILE		, MSGP_ENVIRONMENT_LOGFILE_MENU		, 0		, this );
	eactions[CLEANUP]			= new QAction( MSGP_ENVIRONMENT_CLEANUP		, MSGP_ENVIRONMENT_CLEANUP_MENU		, 0		, this );
	for ( int count=0; count<MAX_ACTIONS ; count++ )
	{	if ( eactions[count]==0 )
			return YASAE_OUT_OF_MEMORY;
	}
	connect( eactions[EXECUTE]		, SIGNAL( activated() ), this, SLOT( ExecuteEnvironment() ) );
	connect( eactions[TIMINGS]		, SIGNAL( activated() ), this, SLOT( ShowTimingResults() ) );
	connect( eactions[STATISTICS]	, SIGNAL( activated() ), this, SLOT( ShowStatisticResults() ) );
	connect( eactions[LOGFILE]		, SIGNAL( activated() ), this, SLOT( ShowLogFile() ) );
	connect( eactions[CLEANUP]		, SIGNAL( activated() ), this, SLOT( CleanupEnvironment() ) );
	return YEntryActions::InitInstance();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::_CreateMenu(YObject *object, QPopupMenu *menu) const
{	bool	success=true;
	success&=eactions[EXECUTE]->addTo( menu );
	success&=eactions[TIMINGS]->addTo( menu );
	success&=eactions[STATISTICS]->addTo( menu );
	success&=eactions[LOGFILE]->addTo( menu );
	success&=eactions[CLEANUP]->addTo( menu );
	(menu)->insertSeparator();
	if ( success )
		return YEntryActions::_CreateMenu(object, menu);
	return YASAE_OUT_OF_MEMORY;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentActions::UpdateActions()
{	if ( actionhandler.GetCurrentObject() )
	{
		eactions[EXECUTE]->setEnabled(true);
		eactions[TIMINGS]->setEnabled(true);
		eactions[STATISTICS]->setEnabled(true);
		eactions[LOGFILE]->setEnabled(true);
		eactions[CLEANUP]->setEnabled(true);
	}
	else
	{
		eactions[EXECUTE]->setEnabled(false);
		eactions[TIMINGS]->setEnabled(false);
		eactions[STATISTICS]->setEnabled(false);
		eactions[LOGFILE]->setEnabled(false);
		eactions[CLEANUP]->setEnabled(false);
	}
	YEntryActions::UpdateActions();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int	 YEnvironmentActions::DoTreeCommand(YObject *treeobject, YActions::COMMAND_TYPE command)
{
	int	retvalue;
	if ( command == YActions::EXECUTE )
	{
		if ( treeobject && treeobject->GetActions() == this )
		{
			retvalue=((YEnvironment*)treeobject)->ExecuteEnvironment();
			if ( retvalue )
				return retvalue;
			return ShowTimingResults( ((YEnvironment*)treeobject) );
		}
	}
	return YActions::DoTreeCommand(treeobject, command );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentActions::ExecuteEnvironment()
{
	((YEnvironment*)actionhandler.GetCurrentObject() )->ExecuteEnvironment();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentActions::CleanupEnvironment()
{
	((YEnvironment*)actionhandler.GetCurrentObject() )->CleanupEnvironment();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::ShowLogFile(YEnvironment *e)
{
	//YFrameObjectArgs	*args=new YFrameObjectArgs(YLogFileViewFrame::CreateLogFileViewFrame, &e->GetLogFilePath() );
	YFrameObjectArgs	*args=new YFrameObjectArgs(YLogFileViewFrame::CreateLogFileViewFrame, e );
	if ( args )
	{
		args->SetTitle( args->GetTitle() + MSGP_ENVIRONMENT_LOGFILE_TITLE );
		return YMDIWindow::OpenNewWindow( args );
	}
	return YASAE_OUT_OF_MEMORY;

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentActions::ShowLogFile()
{
	ShowLogFile( (YEnvironment*) actionhandler.GetCurrentObject() );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::ShowTimingResults(YEnvironment *e)
{
	return e->GetResults().GetTimingResults().GetActions()->Activate(&e->GetResults().GetTimingResults() );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentActions::ShowTimingResults()
{
	YEnvironment	*e=((YEnvironment*)actionhandler.GetCurrentObject() );
	ShowTimingResults( e );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::ShowStatisticResults(YEnvironment *e)
{
	return e->GetResults().GetStatisticResults().GetActions()->Activate(&e->GetResults().GetStatisticResults() );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentActions::ShowStatisticResults()
{
	YEnvironment	*e=((YEnvironment*)actionhandler.GetCurrentObject() );
	ShowStatisticResults( e );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YFrameObjectArgs(YEnvironmentFrame::CreateEnvironmentFrame, object) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::Remove(YObject *entry) const
{
	if ( entry )
		((YEnvironment*) entry)->CleanupEnvironment();
	return YEntryActions::Remove(entry);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentActions::Rename(YObject *object) const
{
	QString	oldpath=((YEnvironment*)object)->GetEnvironmentDirectory(true);
	int	retvalue;
	retvalue=YEntryActions::Rename(object);
	if ( ! retvalue )
		((YEnvironment*)object)->CleanupEnvironment(oldpath);
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFilePathActions::_Activate(YObject *object)
{
	return YMDIWindow::OpenNewWindow( new YFrameObjectArgs(YSourceViewFrame::CreateSourceViewFrame, object) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YFilePathSetActions::Add(YObject *object)
{
	YFilePath			*newobject=0;
	int	retvalue=YASAE_ERROR_IN_ARGUMENTS;
	if ( object )
	{
		YFilePathSet	*set=((YFilePathSet*)object);
		QString			projectdir;

		projectdir=((YProject*)set->GetParent())->CopyProjectDir(true );
		retvalue=dialog.Request(projectdir, QFileDialog::ExistingFiles);
		if ( ! retvalue  )
		{
			QString		filepath;
			QFileInfo	finfo;
			QStringList	list=dialog.selectedFiles();
	        for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
			{
				filepath=*it;
				finfo.setFile(filepath);

				YFileOperations::MakeRelative(filepath, projectdir);
				newobject=(YFilePath*) ((YSet*)object)->CreateNewItem(finfo.fileName());
				if ( ! newobject )
				{
					QString	messagetext;
					messagetext=(MSGP_ENTRY_EXIST_QUESTION).arg(filepath);
					QMessageBox::information(0, MSGP_WARNING, messagetext, MSGP_ERROR_OK);
					break;
				}
				newobject->SetPath(filepath) ;
			}
		}
	}
	else
		qWarning("YSetAction::Add(): set is zero!\n");
	return newobject;
}
