//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA datatypes declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/datatypes.h,v 1.5 2003/01/24 15:47:40 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_DATATYPES_INCLUDE
#define	YASAGUI_DATATYPES_INCLUDE

#include "object.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Typedef_YDerivedObject(QString, YString)
class YString : public YDerivedType<QString>
{
public:				YString(const QString &c=QString::null) :
					YDerivedType<QString>(c)
					{}
	virtual QString	GetContentString() const					{	return GetContent();					}
	virtual QString	GetVisibleContentString() const				{	return GetContent();					}
	static QString	Justify(QString tmp, int width=YASATIME_JUSTIFYNUMBER, QChar fillchar=YASATIME_JUSTIFYFILL);
	virtual	int		ReadFromString(const QString &s)			{	SetContent(s);
																	return 0;								}
	const QString&	operator = (const QString &c)				{	return SetContent(c);					}
	// base class is a type of QString!
	//					operator QString() const					{	return GetContent();					}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
typedef INT64	YASA_TIME;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTIME : public YDerivedType<YASA_TIME>
{
public:				YTIME(YASA_TIME t=YASA_TIME_MIN): YDerivedType<YASA_TIME>(t) {}
					YTIME(const YTIME *t)			: YDerivedType<YASA_TIME>(t) {}
	enum			{ YASA_SHORT_INFINITY_TIME=-1, YASA_TIME_MIN=0, YASA_TIME_INT_MAX=0x7fffffff };
	virtual QString	GetContentString()	const;
	static QString	GetContentString(YASA_TIME time);
	static QString	GetContentStringMax(YASA_TIME time);
	static QString	GetSourceString(YASA_TIME time);
	static YASA_TIME ReadYasaTimeFromString(const QString &str, bool *ok);
	QString			GetVisibleContentString() const;
	static QString	GetVisibleContentString(YASA_TIME time);
	virtual int		ReadFromString(const QString &s);
	bool			IsInfinity() const							{	return GetContent()==YASA_TIME_INFINITY;}
	YASA_TIME		operator - (const YTIME &t) const			{	return GetContent() -  t.GetContent();	}
	bool			operator < (const YTIME &t) const			{	return GetContent() <  t.GetContent();	}
	bool			operator <=(const YTIME &t) const			{	return GetContent() <= t.GetContent();	}
	bool			operator > (const YTIME &t) const			{	return GetContent() >  t.GetContent();	}
	bool			operator >=(const YTIME &t) const			{	return GetContent() >= t.GetContent();	}
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
// should be removed in future
					operator YASA_TIME() const					{	return GetContent();					}

	//! Sets time to the specified value. To set an infinite time use YTIME::YASA_SHORT_INFINITY_TIME or
	//! YASA_TIME_INFINITY
	virtual bool	CheckLimits(YASA_TIME &dest, const YASA_TIME &source) const
																{
																	if ( source == -1 )
																	{
																		dest=YASA_TIME_INFINITY;
																		return true;
																	}
																	if ( source < YASA_TIME_MIN )
																	{
																		qWarning("YASA_TIME::SetTime() called with illegal arguments");
																		dest=YASA_TIME_MIN;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_PERIOD : public YTIME
{public:			YASA_PERIOD(YASA_TIME t=YASA_TIME_INFINITY)	: YTIME(t)								  { }
					YASA_PERIOD(const YTIME *t)					: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_DEADLINE : public YTIME
{public:			YASA_DEADLINE(YASA_TIME t=YASA_TIME_INFINITY): YTIME(t)								  { }
					YASA_DEADLINE(const YTIME *t)				: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
	virtual bool	CheckLimits(YASA_TIME &dest, const YASA_TIME &source) const
																{
																	if ( source <= 0 )
																	{
																		dest=YASA_TIME_INFINITY;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_DEADLINETOLERANCE : public YTIME
{public:			YASA_DEADLINETOLERANCE(YASA_TIME t=0)		: YTIME(t)								  { }
					YASA_DEADLINETOLERANCE(const YTIME *t)		: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_LAXITY : public YTIME
{public:			YASA_LAXITY(YASA_TIME t=YASA_TIME_INFINITY)	: YTIME(t)								  { }
					YASA_LAXITY(const YTIME *t)					: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_REQUIREDTIME : public YTIME
{public:			YASA_REQUIREDTIME(YASA_TIME t=0/*YASA_MIN_REQUIRED_TIME*/)	: YTIME(t)					  { }
					YASA_REQUIREDTIME(const YTIME *t)			: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_OFFSET : public YTIME
{public:			YASA_OFFSET(YASA_TIME t=0)					: YTIME(t)								  { }
					YASA_OFFSET(const YTIME *t)					: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_SCHEDULINGTIME : public YTIME
{public:			YASA_SCHEDULINGTIME(YASA_TIME t=0)			: YTIME(t)								  { }
					YASA_SCHEDULINGTIME(const YTIME *t)			: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_SYNCHRONIZATIONOFFSET : public YTIME
{public:			YASA_SYNCHRONIZATIONOFFSET(YASA_TIME t=0)	: YTIME(t)								  { }
					YASA_SYNCHRONIZATIONOFFSET(const YTIME *t)	: YTIME(t)								  { }
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);					}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! if YASA_TIME==0 it will be used as
class YASA_TICK : public YTIME
{	YASA_TIME		min_tick;
public:				YASA_TICK(YASA_TIME t=YASA_TIME_INFINITY)	: YTIME(t), min_tick(YASA_TIME_MIN)		  { }
					YASA_TICK(const YTIME *t)					: YTIME(t), min_tick(YASA_TIME_MIN)		  { }
	void			SetMinTimerTick(YASA_TIME t);
	virtual bool	CheckLimits(YASA_TIME &dest, const YASA_TIME &source) const;
	const YASA_TIME& operator = (const YASA_TIME &c)			{	return SetContent(c);								}
	virtual QString	GetVisibleContentString()	const			{	if ( (GetContent() == (YASA_TIME)YTIME::YASA_TIME_MIN)		||
																		 (GetContent() == (YASA_TIME)YTIME::YASA_TIME_INT_MAX)	||
																		 IsInfinity()											)
																		 return MSGP_YASA_TIME_INFINITY;
																	return QString::number( (long)GetContent() );		}


};






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_PRIORITY : public YDerivedType<int>
{public:			YASA_PRIORITY(int p=YG_MIN_PRIORITY) :	YDerivedType<int>(p) {}
					YASA_PRIORITY(const YASA_PRIORITY *p):	YDerivedType<int>(p) {}
	virtual QString	GetContentString()	const							{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString()	const					{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
	int				operator - (const YASA_PRIORITY &p) const	{	return GetContent() -  p.GetContent();		}
	bool			operator < (const YASA_PRIORITY &p) const	{	return GetContent() <  p.GetContent();		}
	bool			operator <=(const YASA_PRIORITY &p) const	{	return GetContent() <= p.GetContent();		}
	bool			operator > (const YASA_PRIORITY &p) const	{	return GetContent() >  p.GetContent();		}
	bool			operator >=(const YASA_PRIORITY &p) const	{	return GetContent() >= p.GetContent();		}
					operator int() const						{	return (int)GetContent();					}
	const int&		operator = (const int &c)						{	return SetContent(c);								}
	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YG_MIN_PRIORITY )
																	{	qWarning("YASA_PRIORITY::SetPriority() called with illegal arguments");
																		dest=YG_MIN_PRIORITY;
																		return true;
																	}
																	if ( source > YG_MAX_PRIORITY )
																	{	qWarning("YASA_PRIORITY::SetPriority() called with illegal arguments");
																		dest=YG_MAX_PRIORITY;
																		return true;
																	}
																	dest=source;
																	return true;
																}

};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Handling valid CPU number, if cpu=0 the cpu is not specified (Default). That means a task can run on
//! every cpu and can switch between cpus if the executive supports this feature.
//! Executives using MAXCPU=0 are supporting CPU's as much as needed
//! CPU numbe are valid between YG_MIN_CPU and YG_MAX_CPU
///////////! The attribute "bool allowzero" is used to enable/disable the number 0 as MAXCPU.
class YASA_CPU : public YDerivedType<int>
{public:			YASA_CPU(int c=YG_MIN_CPU) : YDerivedType<int>(c) {}
					YASA_CPU(const YASA_CPU *c, int c2=YG_MIN_CPU) : YDerivedType<int>(c2,c) {}
	virtual QString	GetContentString() const					{	return QString::number( GetContent() );			}
	virtual QString	GetVisibleContentString() const				{	return QString::number( GetContent() );			}
	virtual int		GetMinValue() const							{	return YG_MIN_CPU;								}
	virtual int		ReadFromString(const QString &s);
	bool			operator < (const YASA_CPU &c) const		{	return GetContent() <  c.GetContent();			}
	bool			operator <=(const YASA_CPU &c) const		{	return GetContent() <= c.GetContent();			}
	bool			operator > (const YASA_CPU &c) const		{	return GetContent() >  c.GetContent();			}
	bool			operator >=(const YASA_CPU &c) const		{	return GetContent() >= c.GetContent();			}
	const int&		operator = (const int &c)					{	return SetContent(c);							}
					operator int() const						{	return (int)GetContent();						}
	virtual bool	CheckLimits(int &dest, const int &source) const;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Handling valid number of CPUs. Valid values are 1-n YG_MAX_CPU
//! if CPU =-1 it will be corrected to maxmimum number of cpu used in the program
//! CPU=0 is illegal
class YASA_MAXCPU : public YASA_CPU
{public:			YASA_MAXCPU(int c=YG_DEFAULT_NUMBER_OF_CPUS) :
					YASA_CPU(c)
					{}
					YASA_MAXCPU(const YASA_CPU *c) :
					YASA_CPU(c)
					{}
	virtual int		GetMinValue() const							{	return 1;										}
	bool			operator < (const YASA_CPU &c) const		{	return GetContent() <  c.GetContent();			}
	bool			operator <=(const YASA_CPU &c) const		{	return GetContent() <= c.GetContent();			}
	bool			operator > (const YASA_CPU &c) const		{	return GetContent() >  c.GetContent();			}
	bool			operator >=(const YASA_CPU &c) const		{	return GetContent() >= c.GetContent();			}
	const int&		operator = (const int &c)					{	return SetContent(c);							}
					operator int() const						{	return (int)GetContent();						}
	virtual bool	CheckLimits(int &dest, const int &source) const;
	enum			{ YASA_FLEXIBLE_MAX_CPU=0 };
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
class YASA_REQUIREDVERSION : public YDerivedType<double>
{public:			YASA_REQUIREDVERSION(double v=YASA_VERSION)	 : YDerivedType<double>(v)							{}
					YASA_REQUIREDVERSION(const YASA_REQUIREDVERSION *t): YDerivedType<double>(t)					{}
	virtual QString	GetContentString() const					{	return QString::number( GetContent() );			}
	virtual QString	GetVisibleContentString() const				{	return QString::number( GetContent() );			}
	virtual int		ReadFromString(const QString &s);
					operator double() const						{	return (double)GetContent();					}
	const int&		operator = (const int &c)					{	return SetContent(c);							}
	virtual bool	CheckLimits(double &dest, const double &source) const
																{	if ( source > YASA_VERSION )
																	{	qWarning("YASA_REQUIREDVERSION::SetTaskType() called with illegal arguments");
																		dest=YASA_THREAD_MINTYPE;
																		return true;
																	}
																	if ( source > YASA_THREAD_MAXTYPE )
																	{	qWarning("YASA_TASKTYPE::SetTaskType() called with illegal arguments");
																		dest=YASA_THREAD_MAXTYPE;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};
*/

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_TASKTYPE : public YDerivedType<int>
{public:			YASA_TASKTYPE(int t=YG_MIN_CPU)		 : YDerivedType<int>(t)	{}
					YASA_TASKTYPE(const YASA_TASKTYPE *t): YDerivedType<int>(t)	{}
	virtual QString	GetContentString() const					{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const				{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator int() const						{	return (int)GetContent();					}
	const int&		operator = (const int &c)					{	return SetContent(c);								}
	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YASA_THREAD_MINTYPE )
																	{	qWarning("YASA_TASKTYPE::SetTaskType() called with illegal arguments");
																		dest=YASA_THREAD_MINTYPE;
																		return true;
																	}
																	if ( source > YASA_THREAD_MAXTYPE )
																	{	qWarning("YASA_TASKTYPE::SetTaskType() called with illegal arguments");
																		dest=YASA_THREAD_MAXTYPE;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_SCHEDULERTYPE : public YDerivedType<int>
{public:			YASA_SCHEDULERTYPE(int t=YASA_SCHEDULER_STATIC) : YDerivedType<int>(t)	{}
					YASA_SCHEDULERTYPE(const YASA_SCHEDULERTYPE *t) : YDerivedType<int>(t)	{}
	virtual QString	GetContentString() const							{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const					{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator int() const						{	return (int)GetContent();						}
	const int&		operator = (const int &c)						{	return SetContent(c);								}
	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YASA_SCHEDULER_MINTYPE )
																	{	qWarning("YASA_SCHEDULERTYPE::SetTaskType() called with illegal arguments");
																		dest=YASA_SCHEDULER_MINTYPE;
																		return true;
																	}
																	if ( source > YASA_SCHEDULER_MAXTYPE )
																	{	qWarning("YASA_SCHEDULERTYPE::SetTaskType() called with illegal arguments");
																		dest=YASA_SCHEDULER_MAXTYPE;
																		return true;
																	}
																	dest=source;
																	return true;
																}

};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_SERVERTYPE : public YDerivedType<int>
{public:			YASA_SERVERTYPE(int t=YG_MIN_CPU)		 :	YDerivedType<int>(t) {}
					YASA_SERVERTYPE(const YASA_SERVERTYPE *t):	YDerivedType<int>(t) {}
	virtual QString	GetContentString() const							{	return QString::number( GetContent() );			}
	virtual QString	GetVisibleContentString() const					{	return QString::number( GetContent() );			}
	virtual int		ReadFromString(const QString &s);
					operator int() const						{	return (int)GetContent();						}
	const int&		operator = (const int &c)						{	return SetContent(c);								}
	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YASA_THREAD_MINTYPE )
																	{	qWarning("YASA_SERVERTYPE::SetServerType() called with illegal arguments");
																		dest=YASA_THREAD_MINTYPE;
																		return true;
																	}
																	if ( source > YASA_THREAD_MAXTYPE )
																	{	qWarning("YASA_SERVERTYPE::SetServerType() called with illegal arguments");
																		dest=YASA_THREAD_MAXTYPE;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_DLBEHAVIOUR : public YDerivedType<int>
{public:			YASA_DLBEHAVIOUR(int b=YASA_DL_IGNORE)		: YDerivedType<int>(b)	{}
					YASA_DLBEHAVIOUR(const YASA_DLBEHAVIOUR *b) : YDerivedType<int>(b)	{}
	virtual QString	GetContentString() const							{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const					{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator int() const						{	return (int)GetContent();					}
	const int&		operator = (const int &c)						{	return SetContent(c);								}
	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YASA_DL_MIN )
																	{	qWarning("YASA_DLBEHAVIOUR::SetDLBehaviour() called with illegal arguments");
																		dest=YASA_DL_MIN;
																		return true;
																	}
																	if ( source > YASA_DL_MAX )
																	{	qWarning("YASA_DLBEHAVIOUR::SetDLBehaviour() called with illegal arguments");
																		dest=YASA_DL_MAX;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_MUTEXPROTOCOL : public YDerivedType<int>
{public:			YASA_MUTEXPROTOCOL(int p=YASASP_NOTHING)		: YDerivedType<int>(p) {}
					YASA_MUTEXPROTOCOL(const YASA_MUTEXPROTOCOL *p) : YDerivedType<int>(p) {}
	virtual QString	GetContentString() const							{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const					{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator int() const						{	return (int)GetContent();					}
	const int&		operator = (const int &c)						{	return SetContent(c);								}

	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YASASP_MIN )
																	{	qWarning("YASA_MUTEXPROTOCOL::SetMutexProtocol() called with illegal arguments");
																		dest=YASASP_MIN;
																		return true;
																	}
																	if ( source > YASASP_MAX )
																	{	qWarning("YASA_MUTEXPROTOCOL::SetMutexProtocol() called with illegal arguments");
																		dest=YASASP_MAX;
																		return true;
																	}
																	dest=source;
																	return true;
																}

};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_RESOURCEACTION : public YDerivedType<int>
{public:			YASA_RESOURCEACTION(int p=YASAA_NO_MORE_ACTION)  : YDerivedType<int>(p) {}
					YASA_RESOURCEACTION(const YASA_RESOURCEACTION *p): YDerivedType<int>(p) {}
	virtual QString	GetContentString() const					{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const				{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator int() const						{	return (int)GetContent();					}
	const int&		operator = (const int &c)					{	return SetContent(c);								}
	virtual bool	CheckLimits(int &dest, const int &source) const
																{	if ( source < YASAA_ACTION_MIN )
																	{	qWarning("YASA_RESOURCEACTION::SetResourceAction() called with illegal arguments");
																		dest=YASAA_ACTION_MIN;
																		return true;
																	}
																	if ( source > YASAA_ACTION_MAX )
																	{	qWarning("YASA_RESOURCEACTION::SetResourceAction() called with illegal arguments");
																		dest=YASAA_ACTION_MAX;
																		return true;
																	}
																	dest=source;
																	return true;
																}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_CALCPREEMPTIONTIME : public YDerivedType<bool>
{public:			YASA_CALCPREEMPTIONTIME(bool b=SUPPORT_CALC_PREEMPTIONTIME) : YDerivedType<bool>(b) {}
					YASA_CALCPREEMPTIONTIME(const YASA_CALCPREEMPTIONTIME *b): YDerivedType<bool>(b) {}
	virtual QString	GetContentString() const					{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const				{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator bool() const						{	return GetContent();						}
	virtual bool	CheckLimits(bool &dest, const bool &source) const;
	const bool&		operator = (const bool &c)					{	return SetContent(c);								}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YASA_TIMERTICK : public YDerivedType<bool>
{public:			YASA_TIMERTICK(bool b=SUPPORT_TIMERTICK) : YDerivedType<bool>(b) {}
					YASA_TIMERTICK(const YASA_TIMERTICK *b): YDerivedType<bool>(b) {}
	virtual QString	GetContentString() const					{	return QString::number( GetContent() );		}
	virtual QString	GetVisibleContentString() const				{	return QString::number( GetContent() );		}
	virtual int		ReadFromString(const QString &s);
					operator bool() const						{	return GetContent();						}
	virtual bool	CheckLimits(bool &dest, const bool &source) const;
	const bool&		operator = (const bool &c)					{	return SetContent(c);								}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_CONFIG_INCLUDE
