//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Environment window code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/03/22
// $Header: /sources/yasa/yasagui/environmentwindow.cpp,v 1.3 2003/01/23 09:18:34 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include "environmentwindow.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentFrame::InitInstance()
{
	YEnvironment		*environment	= (YEnvironment*)GetObject();
	QFrame				*mainwidget		= new QFrame( GetMainWidget() );
	QVBoxLayout			*vboxlayout		= new QVBoxLayout(mainwidget, MDI_SCROLLVIEW_BORDER );

	environment->GetSchedulerSet().MaxCPUChanged();

	QGroupBox	*group1	= new QGroupBox( 2, Horizontal, MSGP_WIDGET_PROPERTIES	, mainwidget );
	YSchedulerListViewGroup	*group2	= new YSchedulerListViewGroup(environment	, mainwidget);
	QGroupBox	*group3	= new QGroupBox( 1, Horizontal, MSGP_EXECUTIVE_FEATURES	, mainwidget );
	QGroupBox	*group4	= new QGroupBox( 1, Horizontal, MSGP_WIDGET_DESCRIPTION	, mainwidget );

	vboxlayout->addWidget( group1 );
	vboxlayout->addWidget( group2 );
	vboxlayout->addWidget( group3 );
	vboxlayout->addWidget( group4 );
	group1->setAlignment(AlignHCenter);
	group3->setAlignment(AlignHCenter);
	group4->setAlignment(AlignHCenter);

	QFrame *upperleft = new QVBox( group1 );
	QFrame *upperright=	new QVBox( group1 );

	new	QLabel(MSGP_WIDGET_NAME							, upperleft );
	new	QLabel(MSGP_ENVIRONMENT_EXECUTIVE				, upperleft );
	new	QLabel(MSGP_ENVIRONMENT_SCHEDULING_TIME			, upperleft );
	new	QLabel(MSGP_ENVIRONMENT_SYNCHRONIZATION_OFFSET	, upperleft );
	new	QLabel(MSGP_ENVIRONMENT_GCC_COMPILER_OPTIONS	, upperleft );

	new YNameWidget(environment							, upperright );
	new YExecutiveWidget(environment					, upperright );
	new YSchedulingTimeWidget(environment				, upperright );
	new YSynchronizationOffsetWidget(environment		, upperright );
	new YGCCCompilerOptionsWidget(environment			, upperright );
	group2->InitInstance();

	new YExecutiveFeatureListWidget(environment			, group3 );

	// lower group
	new YDescriptionWidget( environment, group4 );

	scrollview.addChild(mainwidget);
	scrollview.setResizePolicy(QScrollView::AutoOneFit);

	return YObjectFrame::InitInstance();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The executive was changed by clicking the combo box
void YExecutiveWidget::UpdateValue(const QString &e)
{
	((YEnvironment*)object)->SetExecutive( (YExecutivePlugIn*) executivelist.FindObject(e) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! intern selected executive changed -> Update executive combobox
void YExecutiveWidget::UpdateWidget()
{
	YEnvironment	*environment=(YEnvironment*)object;
	if ( environment->GetExecutive() )				// valid executive set ?
	{
		if ( environment->GetExecutive()->GetName() != currentText()  )
		{
			// find executive name in combo list
			for ( int i=0; i<count(); i++)
			{
				if ( environment->GetExecutive()->GetName() == text(i) )
				{
					setCurrentItem(i);
					return;
				}
			}
			setCurrentItem(0);									// "not defined"
		}
	}
	else
		setCurrentItem(0);										// "not defined"
}







//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSchedulerListViewGroup::InitInstance()
{
	int	retvalue=YASAE_OUT_OF_MEMORY;
	listview = new YSchedulerListView(&((YEnvironment*)YWidget::object)->GetSchedulerSet(), this, MSGP_ENVIRONMENT_CPU);
	if ( listview )
	{
		retvalue=listview->InitInstance();
		if ( ! retvalue )
		{
			hbox		= new QHBox( this );
			cpuinfo		= new QLineEdit(hbox);
			cpuinfo->setReadOnly(true);
			schedulercombo= new QComboBox(hbox);
			schedulerlist.AddItem(0, MSGP_ENVIRONMENT_SCHEDULER_UNDEFINED, false);
			schedulercombo->insertStringList(schedulerlist);

			calcpreemptiontimecombo= new QComboBox(hbox);
			calcpreemptiontimecombo->insertItem(MSGP_ENVIRONMENT_DERIVE_PROJECT);
			calcpreemptiontimecombo->insertItem(MSGP_ERROR_YES);
			calcpreemptiontimecombo->insertItem(MSGP_ERROR_NO);
			timertickwidget= new YTickEnvironmentWidget(object, hbox, listview);		//! Attention object is a type of environment !
			timertickwidget->setMinValue(YG_MIN_TIMER_TICK);
			connect( listview, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(UpdateWidgets(QListViewItem *) ) );
			connect( listview, SIGNAL(ListViewEmpty()), this, SLOT(ListViewEmpty() ) );
			connect( schedulercombo, SIGNAL(activated(int)), this, SLOT(SchedulerChanged(int) ) );
			connect( calcpreemptiontimecombo, SIGNAL(activated(int)), this, SLOT(CalcPreemptionTimeChanged(int) ) );
//			connect( timertickwidget, SIGNAL(valueChanged(int)), this, SLOT(TickValueChanged(int) ) );
//			connect( YWidget::object, SIGNAL(UpdateObject(YObject *)), this, SLOT(UpdateWidgets(YObject *) ) );
			connect( &schedulerlist, SIGNAL(StringListChanged()), this, SLOT(SchedulerListChanged() ) );
			//UpdateSchedulerList(0);
			UpdateWidgets(0);
			retvalue=0;
		}
	}
	return retvalue;
}





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Update the "edit line"
void YSchedulerListViewGroup::UpdateWidgets(QListViewItem *item)
{
	cpuinfo->setEnabled( item );
	schedulercombo->setEnabled( item );
	calcpreemptiontimecombo->setEnabled( item );
	timertickwidget->setEnabled( item );

	if ( item )
	{

		if ( scheduler != ((YScheduler*)((YSchedulerListViewItem*)item)->GetObject()) )
		{
			if ( scheduler )
			{
				disconnect( scheduler, SIGNAL( UpdateObject() ), this, SLOT( UpdateWidget() ) );
				disconnect( scheduler, SIGNAL( UpdateObject() ), timertickwidget, SLOT( UpdateWidget() ) );
//				disconnect( scheduler, SIGNAL( UpdateObject(YObject *) ), (YListViewItem*)item, SLOT( UpdateObject(YObject *) ) );
			}
			scheduler=((YScheduler*)((YSchedulerListViewItem*)item)->GetObject());
			if ( scheduler )
			{
				connect( scheduler, SIGNAL( UpdateObject() ), this, SLOT( UpdateWidget() ) );
				connect( scheduler, SIGNAL( UpdateObject() ), timertickwidget, SLOT( UpdateWidget() ) );
//				connect( scheduler, SIGNAL( UpdateObject(YObject *) ), (YListViewItem*)item, SLOT( UpdateObject(YObject *) ) );
			}
		}


		if ( scheduler )
		{
			//unsigned int cpu=((YEnvironment*)object)->GetSchedulerSet().FindItemIndex(scheduler);
			// cpu text
			cpuinfo->setText(scheduler->GetName() );

			// scheduler name
			schedulercombo->setCurrentItem( schedulerlist.FindIndex(scheduler->GetScheduler() ) );

			// calc preemption time && timer tick applicable ?
			if ( scheduler->GetEnvironment()->GetExecutive() && scheduler->GetScheduler() )
			{
				// calc preemption time
				if ( ! scheduler->GetCalcPreemptionTime().IsValid() )
					calcpreemptiontimecombo->setCurrentItem( 0 );
				else
				{
					if ( scheduler->GetCalcPreemptionTime() )
						calcpreemptiontimecombo->setCurrentItem( 1 );
					else
						calcpreemptiontimecombo->setCurrentItem( 2 );
				}
				if ( ! scheduler->GetSupportCalcPreemptionTime() )
					calcpreemptiontimecombo->setEnabled(0);

				// tick value
				if ( ! scheduler->GetSupportTimerTick().GetContent() )
					timertickwidget->setEnabled(0);
				else
					timertickwidget->SetYasaTime( scheduler->GetTimerTick() );
			}
			else
			{
				calcpreemptiontimecombo->setEnabled(0);
				timertickwidget->setEnabled(0);
			}

		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The scheduler was changed in the editline (scheduler combo box)
void YSchedulerListViewGroup::SchedulerChanged(int index)
{
	if ( scheduler )
	{
		scheduler->SetScheduler( schedulercombo->text(index) );
		//UpdateWidgets( listview->currentItem() );
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The scheduler was changed in the editline (scheduler combo box)
void YSchedulerListViewGroup::CalcPreemptionTimeChanged(int index)
{
	if ( scheduler )
	{
		switch ( index )
		{
		case 0:	scheduler->DeriveCalcPreemptionTime();
				break;
		case 1:	scheduler->SetCalcPreemptionTime(true);
				break;
		case 2:	scheduler->SetCalcPreemptionTime(false);
				break;
		}

//		UpdateWidgets( listview->currentItem() );
	}
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The scheduler listviewitem must be updated
void YSchedulerListViewItem::UpdateObject(YObject *o)
{
	YScheduler	*scheduler=(YScheduler*)o;
	if ( scheduler )
	{
		QString	text;

		// scheduler name
		if ( scheduler->GetScheduler() )
			setText(1, scheduler->GetScheduler()->GetName() );
		else
			setText(1, MSGP_ENVIRONMENT_SCHEDULER_UNDEFINED);

		// calc preemption time
		if ( scheduler->GetEnvironment()->GetExecutive() && scheduler->GetScheduler() )
		{
			if ( scheduler->GetCalcPreemptionTime() )
				text=MSGP_ERROR_YES;
			else
				text=MSGP_ERROR_NO;
			text+=" ";
			if ( scheduler->GetCalcPreemptionTime().IsDisabled() )
				text+=MSGP_NOT_SUPPORTED;
			else
			{
				if ( ! scheduler->GetCalcPreemptionTime().IsValid() )
					text+=MSGP_DERIVED;
			}
		}
		else
			text=MSGP_NOT_APPLICABLE;											// no executive||scheduler set
		setText(2, text);

		// tick value
		text=QString::null;
		if ( scheduler->GetEnvironment()->GetExecutive() && scheduler->GetScheduler() )
		{
			if ( scheduler->GetSupportTimerTick() )
			{
				text=scheduler->GetTimerTick().GetVisibleContentString();
				if ( ! scheduler->GetTimerTick().IsValid() )
				{
					text+=" ";
					text+=MSGP_DERIVED;
				}
			}
			else
				text=MSGP_NOT_SUPPORTED;
		}
		else
			text=MSGP_NOT_APPLICABLE;											// no executive||scheduler set
		setText(3, text);
	}

	YListViewItem::UpdateObject(o);
}
