//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA logfile parser
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/logfile.cpp,v 1.4 2003/01/23 09:18:35 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "logfile.h"
#include <stdio.h>
#include "filedialog.h"
#include "application.h"
#include "output.h"
#include "project.h"
#include "plugins.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YProjectLogFile::EvaluateLoggingActions(YLoggingParser &p)
{
	YEnvironmentSet			*eset=&project->GetEnvironmentSet();		// environment set (list of environments)

	for( unsigned int index=0; index<eset->GetCount();index++)
	{
		// logfile of one environment
		YEnvironmentLogFileSet	&elogfile = ((YEnvironment*)eset->At(index))->GetEnvironmentLogFileSet();
		p.SetEnvironmentLogFile( &elogfile, index );
		elogfile.EvaluateLoggingActions(p);
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_TIME YProjectLogFile::GetMaxDuration() const
{
	YEnvironmentSet *eset=&project->GetEnvironmentSet();
	YASA_TIME	time=0,tmp;
	for( unsigned int index=0; index<eset->GetCount();index++)
	{
		tmp=((YEnvironment*)eset->At(index))->GetEnvironmentLogFileSet().GetMaxDuration();
		if ( tmp>time )
			time=tmp;
	}
	return time;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QStringList YProjectLogFile::GetEnvironmentNames() const
{
	QStringList		list;
	YEnvironmentSet *eset=&project->GetEnvironmentSet();
	for( unsigned int index=0; index<eset->GetCount();index++)
	{
		list.append( eset->At(index)->GetName() );
	}
	return list;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned int YProjectLogFile::GetMaxEntries() const
{
	YEnvironmentSet *eset=&project->GetEnvironmentSet();
	return eset->GetCount();
}











//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YEnvironmentLogFileSet::YEnvironmentLogFileSet() :
YSet(0				, 0						, YObjectArgs( 0, 0) )
{
	ResetEnvironmentLogFile();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentLogFileSet::ResetEnvironmentLogFile()
{
	ClearSet();							// delete all cpu logfile
	taskset.ClearSet();
	mutexset.ClearSet();
	ResetStatistics();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentLogFileSet::ResetStatistics()
{
	taskset.ResetStatistics();
	mutexset.ResetStatistics();
	for( unsigned int index=0; index<GetCount();index++)
	{
		((YCPULogFile*)At(index))->GetStatistics().ResetStatistics();
	}
	GetStatistics().CalculateStatistics(*this);							// calculate defaults
}


//! Generate statistics. The CPU specific statistic mostly done during the parsing process
int YEnvironmentLogFileSet::GenerateStatistics()
{
	taskset.GenerateStatistics(*this);
	mutexset.GenerateStatistics(*this);
	for( unsigned int index=0; index<GetCount();index++)
	{
		if ( At(index) )
			((YCPULogFile*)At(index))->GetStatistics().CalculateStatistics( *(YCPULogFile*)At(index) );
	}
	GetStatistics().CalculateStatistics( *this );

	// Evaluate statistics
	//int		i,j,k;
	//i=GetStatistics().
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironmentLogFileSet::ParseLogFile(const QString &filename)
{
	int				retvalue=YASAE_COULD_NOT_OPEN_FILE;
	int				cpu=0;
	YLogFile		file(filename);
	QFileInfo		fileinfo(filename);
	YCPULogFile		*cpulogfile;


	// Logfile already read ?
	if ( GetCount() && fileinfo.exists() && readdate >= fileinfo.lastModified() )
	{
		return 0;
	}

	ResetEnvironmentLogFile();														// clear all cpu logfiles
	if ( file.open(IO_ReadOnly) )
	{
		//ResetEnvironmentLogFile();														// clear all cpu logfiles
		YApplication::GetOutput()->Message(MSGP_LF_PARSING_LOGFILE.arg(filename) );

		while (1)
		{
			// create CPU logfile (list of log file entries of every cpu)
			cpulogfile=(YCPULogFile*)CreateNewItem( MSGP_LF_CPU_NAME.arg(cpu) );
			if ( ! cpulogfile )
			{
				retvalue=YASAE_OPERATION_ABORTED;
				break;
			}

			retvalue=cpulogfile->ParseCPUSection(file);
			if ( retvalue )
			{
				YApplication::GetOutput()->Error( MSGP_LF_PARSE_ERROR.arg(filename).arg(file.GetLineNumber() ) );
				ResetEnvironmentLogFile();
				return retvalue;
			}
			if ( file.atEnd() )
			{
				readdate=QDateTime::currentDateTime();
				break;
			}
			cpu++;
		};
		// Attention: print sequence is changing
		//taskset.SortTaskSet();
		//mutexset.SortMutexSet();

		// Generate statistics. The CPU specific statistic mostly done during the parsing process
		YApplication::GetOutput()->Message(MSGP_LF_MAKE_STATISTIC );
		//retvalue=GenerateStatistics();

		//emit LogFileUpdated();
		if ( ! retvalue )
			YApplication::GetOutput()->Message(MSGP_LF_LOGFILE_READY);
		else
			YApplication::GetOutput()->Error(MSGP_LF_EVALUATING_ERROR.arg(filename) );

	}
	else
	{
		YApplication::GetOutput()->Error(MSGP_LF_LOGFILE_NOT_FOUND.arg(filename) );
	}
	GenerateStatistics();
	emit LogFileUpdated();

	return retvalue;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Reads one of the global arguments and checks if it is correct
//! If the function fails the argument time will set to zero !
int YLogFile::ReadGlobalArg(YASA_TIME &time, const QString &comparestr )
{
	int			retvalue=YASAE_ERROR_IN_ARGUMENTS;
	QString		str;
	bool		ok;

	if ( ReadLine(str) != -1 )
	{
		if ( str.length() >= comparestr.length() )
		{
			if ( str.left( comparestr.length() ) == comparestr )
			{
				time=YTIME::ReadYasaTimeFromString( str.right( str.length() - comparestr.length() ) , &ok);
				if ( ok && time>=0 )
					retvalue=0;
			}
		}
	}
	if (retvalue)
		time=0;
	return 0;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Reads one of the global arguments and checks if it is correct
//! If the function fails the argument time will set to zero !
int YLogFile::ReadGlobalDoubleArg(double &value, const QString &comparestr )
{
	int			retvalue=YASAE_ERROR_IN_ARGUMENTS;
	QString		str;
	bool		ok;

	if ( ReadLine(str) != -1 )
	{
		if ( str.length() >= comparestr.length() )
		{
			if ( str.left( comparestr.length() ) == comparestr )
			{
				value=str.right( str.length() - comparestr.length() ).toDouble(&ok);
				if ( ok )
					retvalue=0;
			}
		}
	}
	if (retvalue)
		value=0;
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Reads a YASA-TIME from the logging parameter
YASA_TIME YCPULogFile::ReadYasaTimeParam(const QString &str, bool &ok)
{
	return YTIME::ReadYasaTimeFromString( str.mid( YASAL_YASAPARAM_POS, str.length()-YASAL_YASAPARAM_POS ), &ok );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function extracts the name of a mutex|thread from the given string
//! e.g. name=""task1", parameter: 1" -> "task1"
QString YCPULogFile::ExtractName(const QString src)
{
	QChar			a,b=' ';
	unsigned int	start=0,index;

	if ( !src.length() )
		return QString::null;
	a=src[0];
	// quotation marks used ?
	if ( a=='"' )
	{
		b='"';
		start=1;
	}
	for (index=start; index<src.length(); index++ )
	{
		if ( src.constref(index)==b )
		{
			return src.mid(start,index-start);
		}
	}
	return QString::null;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function set the endtime of an earlier created range logging entry (Task, Mutex, Excluded)
void YCPULogFile::SetEndRange(YLogAction **range, YLogAction *action, YASA_TIME t)
{
	if ( *range )
	{
		(*range)->SetEndTime(t);
	}
	*range=action;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Create a jitter logging entry if needed.
YLogAction* YCPULogFile::CreateJitterLogAction(YASA_TIME time, YASA_TIME endtime)
{
	YLogAction	*logaction=0;
	if ( time!=endtime )
	{
		if ( endtime<time )
		{
			YASA_TIME	tmp=time;
			time=endtime;
			endtime=tmp;
		}
		logaction = logactions.InsertNewLogAction( new YLogAction );
		if ( ! logaction )
			return 0;
		logaction->SetAction( YASAL_JITTER );
		logaction->SetStartTime(time);
		logaction->SetEndTime(endtime);
	}
	return logaction;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YCPULogFile::ParseCPUSection(YLogFile &file)
{
	YEnvironmentLogFileSet	*environmentlogfile=(YEnvironmentLogFileSet*) GetSet();
	YLogFileTaskSet			&taskset=environmentlogfile->GetTaskSet();
	YLogFileMutexSet		&mutexset=environmentlogfile->GetMutexSet();
	YLogFileTask			*task;
	YLogFileTask			*currenttask=0;			// current work thread, no system or idle task
	YLogFileMutex			*mutex;
	YLogAction				*lastthreadrange=0;		// ptr to task which currently running
	QList<YLogAction>		lastmutexrangelist;		// list of locked mutexes
	QList<YLogAction>		excludedthreadlist;
	YLogAction				*mutexrangeaction;
	YLogAction				*jitterlogaction;
	YPlugInManager			*pm=YApplication::GetPlugInManager();

	double					version;
	int						identifiersize;
	int						threadid;
	int						mutexid;
	int						taskswitches=0;
	int						misseddeadlines=0;
	int						matcheddeadlines=0;
	int						detecteddeadlinemisses=0;
	int						deadlinetolerancemisses=0;
	int						changedpriority=-1;		// the priority was changed by a mutex
	YASA_TIME				lastidletime=-1,idletime=0;
	YASA_TIME				lasttimeofscheduling=-1, timeofscheduling=0;
	QList<YLogFileTask>		usedtasks;
	unsigned int			action,index,index2;
	QString					str,tmp,param;
	YASA_TIME				time, paramtime;
	bool					ok;
	bool					excludedstateentered=true;	// only valid if action=YASAL_THREAD_EXCLUDED
	YLogAction				*logaction;
	YLogFileTask			*lastthreadcomputedptr=0;
	YASA_TIME				lastthreadcomputedtime=0;
	YASA_TIME				nextscheduling=0;
	YASA_TIME				currentdeadline=YASA_TIME_INFINITY;
	YASA_TIME				scheduledactivationtime=YASA_TIME_INFINITY;
	YASA_TIME				utilization=0;

	// yasa version
	if ( ! file.ReadGlobalDoubleArg(version, YASALS_YASA_VERSION ) )
		(( YEnvironmentLogFileSet*)GetSet())->GetStatistics().SetYasaVersion( version );

	// executive name
	if ( file.ReadLine(str) == -1 )
		return YASAE_ERROR_IN_ARGUMENTS;
	tmp=YASALS_EXECUTIVE_NAME;
	if ( str.left( tmp.length() ) != tmp )
		return YASAE_ERROR_IN_ARGUMENTS;
	str=str.right( str.length() - tmp.length() );
	(( YEnvironmentLogFileSet*)GetSet())->GetStatistics().SetExecutiveName( str );

	// executive version
	if ( ! file.ReadGlobalDoubleArg(version, YASALS_EXECUTIVE_VERSION ) )
		(( YEnvironmentLogFileSet*)GetSet())->GetStatistics().SetExecutiveVersion(version);

	// number of cpu
	if ( ! file.ReadGlobalArg(time, YASALS_CPU_NUMBER ) )
		SetCPU( (int)time );

	// scheduler called
	if ( ! file.ReadGlobalArg(time, YASALS_SCHEDULER_CALLED ) )
		statistics.SetSchedulerCalls( (int)time );
		//schedulercalled=(int)time;

	// logfile size
	if ( ! file.ReadGlobalArg(time, YASALS_LOGFILE_SIZE ) )
		statistics.SetLogBufferSize( (int) time );
		//logfilesize=(long)time;

	// logfile position
	if ( ! file.ReadGlobalArg(time, YASALS_LOGFILE_POSITION ) )
		statistics.SetUsedLogBufferSize( (int) time );
		//logfileposition=(long)time;

	// scheduler start
	if ( ! file.ReadGlobalArg(time, YASALS_SCHEDULER_START ) )
		statistics.SetExecutionStart( time );
		//schedulerstart=time;

	// scheduler end time (scheduled end )
	if ( ! file.ReadGlobalArg(time, YASALS_END_TIME ) )
	{
		statistics.SetScheduledExecutionEnd( time );
		//schedulerend=time;
	}

	// current time of stopping the project (real end)
	if ( ! file.ReadGlobalArg(time, YASALS_CURRENT_TIME ) )
	{
		statistics.SetExecutionEnd( time );
		//statistics.SetCurrentTime( time );
		//currenttime=time;
	}

	// duration
	if ( ! file.ReadGlobalArg(time, YASALS_DURATION ) )
	{
		statistics.SetExecutionDuration( time );
	}
		//duration=time;

	// scheduler tick
	if ( ! file.ReadGlobalArg(time, YASALS_SCHEDULER_TICK ) )
		statistics.SetSchedulerTick( time );
		//schedulertick=time;

	// scheduling time (time of one scheduling cycle, only useful in simulators)
	if ( ! file.ReadGlobalArg(time, YASALS_SCHEDULING_TIME ) )
	{
		//statistics.SetSchedulingTime( time );
		//schedulingtime=time;
	}

	// calc preemption time
	if ( file.ReadLine(str) == -1 )
		return YASAE_ERROR_IN_ARGUMENTS;
	tmp=YASALS_CALC_PREEMPTION_TIME;
	if ( str.left(tmp.length() ) != tmp )
		return YASAE_ERROR_IN_ARGUMENTS;
	str=str.right( str.length() - tmp.length() );
	tmp="yes";
	if ( str.left( tmp.length() ) == tmp )
		statistics.SetCalcPreemptionTime(true);
		//calcpreemptiontime=true;
	else
		statistics.SetCalcPreemptionTime(false);
		//calcpreemptiontime=false;

	// scheduler name
	if ( file.ReadLine(str) == -1 )
		return YASAE_ERROR_IN_ARGUMENTS;
	tmp=YASALS_NAME_OF_SCHEDULER;
	if ( str.left( tmp.length() ) != tmp )
		return YASAE_ERROR_IN_ARGUMENTS;
	str=str.right( str.length() - tmp.length() -3 );				// cut "ys_" too

	//statistics.SetSchedulerName(str);
	statistics.SetSchedulerName( pm->GetSchedulerPlugIns().FindLongName(str) );

	// scheduler version
	if ( ! file.ReadGlobalDoubleArg(version, YASALS_VERSION_OF_SCHEDULER ) )
		GetStatistics().SetSchedulerVersion( version );


	// read log entries now
	tmp=YASALS_IDENTIFIER;
	identifiersize=tmp.length();
	while (1)
	{
		task=0;
		mutex=0;
		if ( file.ReadLine(str) == -1 )
		{
			if ( ! file.atEnd() )
				return YASAE_ERROR_IN_ARGUMENTS;
			str=QString::null;
		}
		if ( ! str.length() )										// end of this cpu
			break;
		// check and remove "YL:"
		if ( str.left( identifiersize ) != YASALS_IDENTIFIER )
			return YASAE_ERROR_IN_ARGUMENTS;
		//str.remove( 0, identifiersize);

		// check, read and remove "time"
		time=YTIME::ReadYasaTimeFromString(str.mid(3,21), &ok);
		if ( !ok )
			return YASAE_ERROR_IN_ARGUMENTS;
		//str.remove( 0, 20);

		// check, read and remove threadid
		threadid=YTIME::ReadYasaTimeFromString( str.mid(32, 4), &ok );
		if ( !ok )
			return YASAE_ERROR_IN_ARGUMENTS;

		// check, read and remove action
		action=YTIME::ReadYasaTimeFromString(str.mid(37, 3), &ok);
		if ( !ok )
			return YASAE_ERROR_IN_ARGUMENTS;


		// create list entry
		logaction = logactions.InsertNewLogAction( new YLogAction );
		if ( ! logaction )
			return YASAE_OUT_OF_MEMORY;
		logaction->SetAction( (ACTIONINT) action);
		logaction->SetStartTime(time);
		jitterlogaction=0;
		if ( action > ACTIONINTMAX )
			qWarning("YCPULogFile::ParseCPUSection() action overflow! Change datatype \"ACTIONINT\"");

		switch (action)
		{
			case YASAL_UNKNOWN:					break;
			case YASAL_THREAD_CREATED:			break;
			case YASAL_THREAD_ACTIVATE:			scheduledactivationtime=ReadYasaTimeParam(str, ok);
												if ( !ok )
													return YASAE_ERROR_IN_ARGUMENTS;
												jitterlogaction=CreateJitterLogAction(time, scheduledactivationtime);
												//logaction->SetStartTime(paramtime);
												//logaction->SetEndTime(time);
												break;
			case YASAL_THREAD_EXIT:				SetEndRange(&lastthreadrange, 0, time);
												break;
			case YASAL_THREAD_START:			SetEndRange(&lastthreadrange, logaction, time);
												break;
			case YASAL_THREAD_CONT:				SetEndRange(&lastthreadrange, logaction, time);
												break;
			case YASAL_THREAD_SUSPEND:			if (lastthreadcomputedptr)
												{
													lastthreadcomputedptr->AddThreadComputedTmp(time-lastthreadcomputedtime);
													utilization+=time-lastthreadcomputedtime;
													lastthreadcomputedtime=0;
													lastthreadcomputedptr=0;
												}
												SetEndRange(&lastthreadrange, 0, time);
												break;
			case YASAL_THREAD_TOLERANCE_START:	SetEndRange(&lastthreadrange, logaction, time);
												break;
			case YASAL_THREAD_TOLERANCE_CONT:	SetEndRange(&lastthreadrange, logaction, time);
												break;
			case YASAL_THREAD_TOLERANCE_SUSPEND:if (lastthreadcomputedptr)
												{
													lastthreadcomputedptr->AddThreadComputedTmp(time-lastthreadcomputedtime);
													utilization+=time-lastthreadcomputedtime;
													lastthreadcomputedtime=0;
													lastthreadcomputedptr=0;
												}
												SetEndRange(&lastthreadrange, 0, time);
												break;
			case YASAL_THREAD_SKIPPED:			break;
			case YASAL_THREAD_TYPE:				break;
			case YASAL_THREAD_EXCLUDED:			excludedstateentered=str.mid( YASAL_THREAD_EXCLUDED_POS, str.length()-YASAL_THREAD_EXCLUDED_POS ).find(YASALS_NOT_EXCLUDED) != -1;
												break;
			//! The real deadline is given in the parameter
			case YASAL_DEADLINE_MATCH:			matcheddeadlines++;
												currentdeadline=time=ReadYasaTimeParam(str, ok);
												if ( !ok )
													return YASAE_ERROR_IN_ARGUMENTS;
												logaction->SetStartTime(time);
												break;
			case YASAL_DEADLINE_MISS:			misseddeadlines++;
												currentdeadline=ReadYasaTimeParam(str, ok);
												if ( !ok )
													return YASAE_ERROR_IN_ARGUMENTS;
												break;
			case YASAL_DEADLINE_MISS_DETECTED:	detecteddeadlinemisses++;
												break;
			case YASAL_DEADLINE_TOLERANCE_MISS:	deadlinetolerancemisses++;
												break;
			case YASAL_TIME_TO_COMPUTE:			break;
			case YASAL_MUTEX_CREATED:			break;
			case YASAL_MUTEX_DESTROYED:			break;
			case YASAL_MUTEX_LOCKED:			break;
			case YASAL_MUTEX_TRY_N_BLOCKED:		break;
			case YASAL_MUTEX_UNLOCKED:			break;
			case YASAL_MUTEX_RELOCKED:			break;
			case YASAL_MUTEX_NOT_LOCKED:		break;
			case YASAL_MUTEX_PRIORITY_CHANGED:	time=ReadYasaTimeParam(str, ok);
												if ( !ok )
													return YASAE_ERROR_IN_ARGUMENTS;
												changedpriority=(int)time;
												GetStatistics().IncNumberOfPriorityChanges();
												GetStatistics().SetMaxPriorityCeiling(changedpriority);
												break;
			case YASAL_MUTEX_SYSTEM_CEILING:	break;
			case YASAL_MESSAGE:					break;
			case YASAL_EMERGENCY_THREAD:		break;
			case YASAL_SCHEDULER_CHANGED:		break;
			case YASAL_SCHEDULER_CALLED:		if (lastthreadcomputedptr)
												{
													lastthreadcomputedptr->AddThreadComputedTmp(time-lastthreadcomputedtime);
													if ( ! lastthreadcomputedptr->IsIdleTask() && ! lastthreadcomputedptr->IsSystemTask() )
														utilization+=time-lastthreadcomputedtime;
													if ( lastthreadcomputedptr->GetId() == YASA_IDLE_ID)	// add idle times
													{
														lastthreadcomputedptr->GetStatistics().AddComputationTime( lastthreadcomputedptr->GetThreadComputedTmp() );
														lastthreadcomputedptr->SetThreadComputedTmp(0);
														//lastthreadcomputedptr->GetStatistics().IncPreemptions();
													}
													else
													{
														lastthreadcomputedptr->GetStatistics().IncPreemptions();
													}
													lastthreadcomputedtime=0;
													lastthreadcomputedptr=0;
												}
												lasttimeofscheduling=time;
												paramtime=ReadYasaTimeParam(str, ok);
												if ( !ok )
													return YASAE_ERROR_IN_ARGUMENTS;
												jitterlogaction=CreateJitterLogAction(time, paramtime);
												SetEndRange(&lastthreadrange, logaction, time);
												break;
			case YASAL_SCHEDULER_FINISHED:		if ( lastidletime!= -1)		// add cpu scheduling time
												{
													idletime+=time-lastidletime;
													lastidletime=-1;
												}
												if (lastthreadcomputedptr)	// add one scheduling cycle
												{
													lastthreadcomputedptr->AddThreadComputedTmp(time-lastthreadcomputedtime);
													//utilization+=time-lastthreadcomputedtime;	// do not add scheduling times
													lastthreadcomputedptr->GetStatistics().AddComputationTime( lastthreadcomputedptr->GetThreadComputedTmp() );
													lastthreadcomputedptr->SetThreadComputedTmp(0);
													lastthreadcomputedtime=0;
													lastthreadcomputedptr=0;
												}
												if ( lasttimeofscheduling != -1)
												{
													timeofscheduling+=time-lasttimeofscheduling;
													lasttimeofscheduling=-1;
												}
												SetEndRange(&lastthreadrange, 0, time);
												// calculate scheduling jitter
												task=taskset.FindTask( YASA_SCHEDULER_ID );
												if ( task )
												{
													//if ( task->GetThreadActivatedTmp() >= 0 )
													task->GetStatistics().AddReactionTime( time - nextscheduling ); //task->GetThreadActivatedTmp() );
													//else
													//	task->GetStatistics().AddReactionTime( 0 );
													task->SetThreadActivatedTmp(0);
												}
												nextscheduling=ReadYasaTimeParam(str, ok);
												if ( ! nextscheduling )
													nextscheduling=YASA_TIME_INFINITY;
												if ( !ok )
													return YASAE_ERROR_IN_ARGUMENTS;
												break;
			case YASAL_SCHEDULER_SWITCH:		taskswitches++;
												break;
			case YASAL_SCHEDULER_SYNCHRONIZED_START:
												if ( time==0 )				// to prevent figures at t=0
												{
													logactions.RemoveLogAction(logaction);
													logaction=0;
												}
												break;
			case YASAL_IDLE_THREAD:				lastidletime=time;
												SetEndRange(&lastthreadrange, logaction, time);
												break;
			case YASAL_JITTER:					paramtime=ReadYasaTimeParam(str, ok);
												logaction->SetEndTime(paramtime);
												break;
			default:							logaction=0;
												break;
#if YASAL_MAX!=35
#error There are inconsistences within the logging types!
#endif
		}

		if (logaction)
		{
			//----------------------------- Task -----------------------------//
			// find task name
			switch (threadid)
			{
			case YASA_IDLE_ID:		tmp=MSGP_LF_TASK_IDLE;
									break;
			case YASA_YASASUBSYSTEM:if (action==YASAL_THREAD_CREATED)
									{
										if ( action==YASAL_THREAD_CREATED && str.length()>YASAL_THREADNAME_POS )
										{
											threadid=YTIME::ReadYasaTimeFromString( str.mid( YASAL_THREADID_POS, str.length()-YASAL_THREADID_POS ), &ok );
											if ( !ok )
												return YASAE_ERROR_IN_ARGUMENTS;
											tmp=ExtractName(str.mid( YASAL_THREADNAME_POS, str.length()-YASAL_THREADNAME_POS ));
										}
										break;
									}
			case YASA_SCHEDULER_ID:	tmp=MSGP_LF_TASK_SYSTEM;
									threadid=YASA_SCHEDULER_ID;
									break;
			default:				tmp=QString::null;
									break;
			}
			task=taskset.AllocTaskItemOnce( tmp, threadid );
			if (task)
			{
				logaction->SetThread(task);
				if ( jitterlogaction)								// finish jitter actions
					jitterlogaction->SetThread(task);
				if ( usedtasks.findRef(task) == -1 )
				{
					usedtasks.append(task);
				}
				if ( currenttask && currenttask != task )
				{
					if ( action==YASAL_THREAD_START				|| action==YASAL_THREAD_CONT ||
						 action==YASAL_THREAD_TOLERANCE_START	|| action==YASAL_THREAD_TOLERANCE_CONT )
						currenttask->GetStatistics().IncCPULosses();
				}

				switch ( action )
				{
					case YASAL_THREAD_ACTIVATE:			task->SetThreadPeriod( scheduledactivationtime );
														task->SetScheduledActivationTime(scheduledactivationtime);
														task->GetStatistics().AddJitterTime( time-scheduledactivationtime );
														task->GetStatistics().IncActivations();
														task->SetThreadActivatedTmp(time);
														break;
					case YASAL_THREAD_START:			currenttask=task;
														task->GetStatistics().IncStarts();
														task->SetThreadComputedTmp(0);
														lastthreadcomputedptr=task;
														lastthreadcomputedtime=time;
														break;
					case YASAL_THREAD_CONT:				currenttask=task;
														lastthreadcomputedptr=task;
														lastthreadcomputedtime=time;
														//task->GetStatistics().IncPreemptions();
														break;
					case YASAL_THREAD_SUSPEND:			currenttask=0;
														task->GetStatistics().IncSuspendedThreads();
														task->GetStatistics().AddComputationTime( task->GetThreadComputedTmp() );
														task->GetStatistics().AddReactionTime( time - task->GetThreadActivatedTmp() );
														task->SetThreadComputedTmp(0);
														task->SetThreadActivatedTmp(0);
														break;
					case YASAL_THREAD_TOLERANCE_START:	currenttask=task;
														task->GetStatistics().IncDeadlineToleranceStarts();
														task->SetThreadComputedTmp(0);
														lastthreadcomputedptr=task;
														lastthreadcomputedtime=time;
														break;
					case YASAL_THREAD_TOLERANCE_CONT:	currenttask=task;
														lastthreadcomputedptr=task;
														lastthreadcomputedtime=time;
														//task->GetStatistics().IncPreemptions();
														break;
					case YASAL_THREAD_TOLERANCE_SUSPEND:currenttask=0;
														task->GetStatistics().IncSuspendedThreads();
														task->GetStatistics().AddComputationTime( task->GetThreadComputedTmp() );
														task->GetStatistics().AddReactionTime( time - task->GetThreadActivatedTmp() );
														task->SetThreadComputedTmp(0);
														task->SetThreadActivatedTmp(0);
														break;
					case YASAL_THREAD_SKIPPED:			task->GetStatistics().IncSkippedThreads();
														break;
					//case YASAL_THREAD_TYPE:			break;
					case YASAL_THREAD_EXCLUDED:			if ( excludedstateentered )
															task->GetStatistics().IncExclusions();
														break;
					case YASAL_DEADLINE_MATCH:			task->GetStatistics().IncMatchedDeadlines();
														task->SetThreadDeadline( currentdeadline );
														break;
					case YASAL_DEADLINE_MISS:			task->GetStatistics().IncMissedDeadlines();
														task->SetThreadDeadline( currentdeadline );
														break;
					case YASAL_DEADLINE_MISS_DETECTED:	task->GetStatistics().IncDetetectedDeadlineMisses();
														break;
					case YASAL_DEADLINE_TOLERANCE_MISS:	task->GetStatistics().IncDeadlineToleranceMisses();
														break;
					/*case YASAL_TIME_TO_COMPUTE:		break;
					case YASAL_MUTEX_CREATED:			break;
					case YASAL_MUTEX_DESTROYED:			break;
					case YASAL_MUTEX_LOCKED:			break;
					case YASAL_MUTEX_TRY_N_BLOCKED:		break;
					case YASAL_MUTEX_UNLOCKED:			break;
					case YASAL_MUTEX_RELOCKED:			break;
					case YASAL_MUTEX_NOT_LOCKED:		break;*/
					case YASAL_MUTEX_PRIORITY_CHANGED:	if ( changedpriority!=-1 )
														{
															task->GetStatistics().SetMaxPriorityCeiling( changedpriority );
															task->GetStatistics().IncNumberOfPriorityChanges();
														}
														break;
					//case YASAL_MUTEX_SYSTEM_CEILING:	break;
					//case YASAL_MESSAGE:					break;
					case YASAL_EMERGENCY_THREAD:		task->GetStatistics().IncEmergencyThreads();
														break;
					//case YASAL_SCHEDULER_CHANGED:		break;
					case YASAL_SCHEDULER_CALLED:		task->GetStatistics().IncActivations();
														task->GetStatistics().IncStarts();
														lastthreadcomputedptr=task;
														lastthreadcomputedtime=time;
														if ( nextscheduling!= YASA_TIME_INFINITY && nextscheduling<=time)
															task->GetStatistics().AddJitterTime( time-nextscheduling );
														else
															task->GetStatistics().AddJitterTime( 0 );
														if ( nextscheduling > time || nextscheduling==YASA_TIME_INFINITY )
															nextscheduling=time;
														//if ( task->GetThreadActivatedTmp() <= time )	// tick->infinity -> nextscheduling=0 in logfile
														//	task->SetThreadActivatedTmp( time );
														break;
					case YASAL_SCHEDULER_FINISHED:		break;
					//case YASAL_SCHEDULER_SWITCH:		break;
					case YASAL_SCHEDULER_SYNCHRONIZED_START:task->GetStatistics().SetSynchronizationOffset(time);
														break;
					case YASAL_IDLE_THREAD:				task->GetStatistics().IncActivations();
														task->GetStatistics().IncStarts();
														lastthreadcomputedptr=task;
														lastthreadcomputedtime=time;
														break;
					//case YASAL_JITTER:					break;

				}
			}
			else
				return YASAE_OUT_OF_MEMORY;

			//----------------------------- Mutex -----------------------------//
			if ( action >= YASAL_MUTEX_CREATED && action<=YASAL_MUTEX_NOT_LOCKED )	// check mutex logging types ?
			{
				mutexid=YTIME::ReadYasaTimeFromString( str.mid( YASAL_MUTEXID_POS, str.length()-YASAL_MUTEXID_POS ), &ok );

				if ( action==YASAL_MUTEX_LOCKED )
				{
					lastmutexrangelist.append(logaction);
				}

				// set end time of previous logging entry
				if ( action==YASAL_MUTEX_UNLOCKED )
				{
					for ( index=lastmutexrangelist.count() ; index!=0 ; index-- )
					{
						if ( lastmutexrangelist.at(index-1)->GetAction() == YASAL_MUTEX_LOCKED )
						{	// find mutex start logaction (if order wrong !)
							mutexrangeaction=0;
							mutex=0;
							for ( index2=lastmutexrangelist.count() ; index2!=0 ; index2-- )
							{
								mutexrangeaction=lastmutexrangelist.at(index2-1);
								mutex=mutexrangeaction->GetMutex();
								if (mutex && mutex->GetId()==mutexid )
									break;
								mutexrangeaction=0;
								mutex=0;
							}
							if ( ! mutexrangeaction || !mutex)			// start logaction could not be found
							{
								((YApplication*)qApp)->GetOutput()->Warning( MSGP_LF_MUTEX_NOT_LOCKED
									.arg(mutexid).arg( (int)time) );
								break;									// leave loop "set end time"
							}

							mutexrangeaction->SetEndTime(time);
							if ( mutexid!= lastmutexrangelist.at(index-1)->GetMutex()->GetId() &&
								 lastmutexrangelist.count()>1 &&
								 threadid!=mutexrangeaction->GetThread()->GetId() )
							{
								((YApplication*)qApp)->GetOutput()->Warning( MSGP_LF_MUTEX_ORDER_WRONG
								.arg( mutex->GetName() )
								.arg( lastmutexrangelist.last()->GetMutex()->GetName() )
								.arg( (int)time ) );
							}
							lastmutexrangelist.at(index2-1);	// make current
							lastmutexrangelist.remove();
							break;
						}
					}
				}
				tmp=QString::null;
				if ( action== YASAL_MUTEX_CREATED && str.length() > YASAL_MUTEXNAME_POS )
					tmp=ExtractName(str.mid( YASAL_MUTEXNAME_POS, str.length()-YASAL_MUTEXNAME_POS ));
				mutex=mutexset.AllocMutexItemOnce( tmp, mutexid);
				if (mutex)
				{
					logaction->SetMutex(mutex);
					switch ( action)
					{
						case YASAL_MUTEX_CREATED:			break;
						case YASAL_MUTEX_DESTROYED:			break;
						case YASAL_MUTEX_LOCKED:			mutex->GetStatistics().IncMutexLocked();
															GetStatistics().IncMutexLocked();
															mutex->SetMutexLockingTimeTmp(time);
															break;
						case YASAL_MUTEX_TRY_N_BLOCKED:		mutex->GetStatistics().IncMutexBlocked();
															GetStatistics().IncMutexBlocked();
															if ( changedpriority!=-1 )
															{
																mutex->GetStatistics().IncNumberOfPriorityChanges();
																mutex->GetStatistics().SetMaxPriorityCeiling( changedpriority );
															}

															break;
						case YASAL_MUTEX_UNLOCKED:			if ( changedpriority!=-1 )
																mutex->GetStatistics().IncNumberOfPriorityChanges();
															mutex->GetStatistics().IncMutexUnlocked();
															GetStatistics().IncMutexUnlocked();
															mutex->GetStatistics().AddLockingTime( time - mutex->GetMutexLockingTimeTmp() );
															mutex->SetMutexLockingTimeTmp(0);
															break;
						case YASAL_MUTEX_RELOCKED:			mutex->GetStatistics().IncMutexRelocked();
															GetStatistics().IncMutexRelocked();
															break;
						case YASAL_MUTEX_NOT_LOCKED:		break;
					}
				}
				else
					return YASAE_OUT_OF_MEMORY;
			}


			//------------------- Excluded thread ------------------
			if ( action==YASAL_THREAD_EXCLUDED )
			{
				// is this thread entering or leaving the excluded state ?
				//if ( str.mid( YASAL_THREAD_EXCLUDED_POS, str.length()-YASAL_THREAD_EXCLUDED_POS ).find(YASALS_NOT_EXCLUDED) != -1 )
				if ( excludedstateentered )
				{
					// find corresponding thread in list
					for ( index=excludedthreadlist.count() ; index!=0 ; index-- )
					{
						if ( excludedthreadlist.at(index-1)->GetThread()->GetId() == threadid )
						{
							excludedthreadlist.current()->SetEndTime(time);
							excludedthreadlist.remove();
						}
					}
				}
				else
				{	// thread is entering the excluded state, check if it is already in list
					for ( index=excludedthreadlist.count() ; index!=0 ; index-- )
					{
						if ( excludedthreadlist.at(index-1)->GetThread()->GetId() == threadid )
							break;
					}
					if (!index)
					{
						excludedthreadlist.append(logaction);
					}
				}
			}

			switch (action)
			{
				case YASAL_MUTEX_PRIORITY_CHANGED:	if (task && changedpriority!=-1 )
													{
														task->GetStatistics().SetMaxPriorityCeiling( changedpriority );
													}
													//changedpriority!=-1;
													break;
			}
		}
	};


	if ( lastidletime!= -1)
		idletime += statistics.GetExecutionEnd() - lastidletime;
	if ( lasttimeofscheduling != -1)
		timeofscheduling += statistics.GetExecutionEnd() - lasttimeofscheduling;
	if (lastthreadcomputedptr)
	{
		if ( ! lastthreadcomputedptr->IsIdleTask() && ! lastthreadcomputedptr->IsSystemTask() )
			utilization+=time-lastthreadcomputedtime;
		lastthreadcomputedptr->AddThreadComputedTmp(time-lastthreadcomputedtime);
		lastthreadcomputedtime=0;
		lastthreadcomputedptr=0;
	}
	// Add all unfinished computation times
	for (index=0; index<taskset.GetCount() ; index++ )
	{
		task=(YLogFileTask*)taskset.At(index);
		if ( task && task->GetThreadComputedTmp() )
		{
			task->GetStatistics().AddComputationTime( task->GetThreadComputedTmp() );
		}
	}

	// idle task statistic properties
	/*task=taskset.FindTask(YASA_IDLE_ID);
	if ( task && task->GetThreadComputedTmp() )
	{
		task->GetStatistics().AddComputationTime( task->GetThreadComputedTmp() );
	}
	// system task statistic properties
	task=taskset.FindTask(YASA_SCHEDULER_ID);
	if ( task && task->GetThreadComputedTmp() )
	{
		task->GetStatistics().AddComputationTime( task->GetThreadComputedTmp() );
	}*/
	// calc user task number
	index=usedtasks.count();
	task=taskset.FindTask(YASA_IDLE_ID);
	if ( task )
	{
		index--;
		task->GetStatistics().InvalidSynchronizationOffset();
		task->GetStatistics().InvalidPeriod();
		task->GetStatistics().InvalidDeadline();
		task->GetStatistics().InvalidJitter();
		task->GetStatistics().InvalidMatchedDeadlines();
		task->GetStatistics().InvalidMissedDeadlines();
		task->GetStatistics().InvalidDetectedDeadlineMissed();
		task->GetStatistics().InvalidDeadlineToleranceStarts();
		task->GetStatistics().InvalidDeadlineToleranceMisses();
		task->GetStatistics().InvalidReactionTime();
		task->GetStatistics().InvalidExclusions();
		task->GetStatistics().InvalidCPULosts();
		task->GetStatistics().InvalidSkipped();
		task->GetStatistics().InvalidSuspended();
		task->GetStatistics().InvalidEmergencyThreads();
		task->GetStatistics().InvalidLockedMutexes();
		task->GetStatistics().InvalidUnlockedMutexes();
		task->GetStatistics().InvalidMutexesRelocked();
		task->GetStatistics().InvalidBlockedByMutexes();
		task->GetStatistics().InvalidMaxPriorityCeiling();
		task->GetStatistics().InvalidNumberOfPriorityChanges();
		// map idle name if scheduler known
		task->SetName( pm->GetExecutivePlugIns().GetIdleName(
							(( YEnvironmentLogFileSet*)GetSet())->GetStatistics().GetExecutiveName(),
								task->GetName() ) );
	}
	task=taskset.FindTask(YASA_SCHEDULER_ID);
	if ( task )
	{
		index--;
		task->GetStatistics().InvalidSynchronizationOffset();
		task->GetStatistics().InvalidPreemptions();
		task->GetStatistics().InvalidDeadline();
		task->GetStatistics().InvalidEmergencyThreads();
		task->GetStatistics().InvalidMatchedDeadlines();
		task->GetStatistics().InvalidMissedDeadlines();
		task->GetStatistics().InvalidDetectedDeadlineMissed();
		task->GetStatistics().InvalidDeadlineToleranceStarts();
		task->GetStatistics().InvalidDeadlineToleranceMisses();
		task->GetStatistics().InvalidExclusions();
		task->GetStatistics().InvalidCPULosts();
		task->GetStatistics().InvalidSkipped();
		task->GetStatistics().InvalidSuspended();
		task->GetStatistics().InvalidEmergencyThreads();
		task->GetStatistics().InvalidLockedMutexes();
		task->GetStatistics().InvalidUnlockedMutexes();
		task->GetStatistics().InvalidMutexesRelocked();
		task->GetStatistics().InvalidBlockedByMutexes();
		task->GetStatistics().InvalidMaxPriorityCeiling();
		task->GetStatistics().InvalidNumberOfPriorityChanges();
		task->GetStatistics().SetPeriod( statistics.GetSchedulerTick() );
	}
	statistics.SetNumberOfTasks(index);

	// Finalize statistics
	statistics.SetTaskSwitches(taskswitches);
	statistics.SetMissedDeadlines(misseddeadlines);
	statistics.SetMatchedDeadlines(matcheddeadlines);
	statistics.SetDetectedDeadlineMisses(detecteddeadlinemisses);
	statistics.SetDeadlineToleranceMisses(deadlinetolerancemisses);
	statistics.SetTimeOfScheduling( timeofscheduling );
	statistics.SetIdleTime( idletime );
	statistics.SetUtilization( utilization );

	SetEndRange(&lastthreadrange, 0, statistics.GetExecutionEnd() );
	// set end time of previous logging entry
	for ( index=excludedthreadlist.count() ; index!=0 ; index-- )
	{
		excludedthreadlist.at(index-1)->SetEndTime( statistics.GetExecutionEnd() );
		excludedthreadlist.remove();
	}

	// set end time of previous logging entry
	for ( index=lastmutexrangelist.count() ; index!=0 ; index-- )
	{
		lastmutexrangelist.at(index-1)->SetEndTime( statistics.GetExecutionEnd() );
		lastmutexrangelist.remove();
	}

	logactions.sort();
	return 0;
}





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Reads a line from the logfile
int YLogFile::ReadLine(QString &str)
{
	int	retvalue;

	str=QString::null;
	retvalue=readLine(str, 255);
	if ( retvalue != -1 )
	{
		linenumber++;
		if ( str.length() )
		{
			while ( str.length() && ( (str.constref( str.length()-1 ) == '\n'	 ) ||
									  (str.constref( str.length()-1 ) == (char)13)	) )
				str.remove( str.length()-1, 1);
		}
		retvalue=0;
	}
	return retvalue;
}
