//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA output
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/output.cpp,v 1.3 2003/01/23 09:18:36 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file handles the execution of Yasa and displays the output in the appropriate window
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "output.h"
#include "translations.h"
#include <qpopupmenu.h>
#include <qpixmap.h>
#include <qbrush.h>
#include "application.h"
#include "filedialog.h"
#include "settings.h"


#define POPUP_OUTPUT_CLEAR		1
#define POPUP_OUTPUT_FULLSCREEN	2

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YOutputView::InitInstance(YOutput *o)
{
	output=o;
	connect( output, SIGNAL( NewImportant(const QString &) ), this, SLOT( PrintImportant(const QString &) ) );
	connect( output, SIGNAL( NewCommand(const QString &) ), this, SLOT( PrintCommand(const QString &) ) );
	connect( output, SIGNAL( NewComment(const QString &) ), this, SLOT( PrintComment(const QString &) ) );
	connect( output, SIGNAL( NewMessage(const QString &) ), this, SLOT( PrintMessage(const QString &) ) );
	connect( output, SIGNAL( NewWarning(const QString &, int) ), this, SLOT( PrintWarning(const QString &, int) ) );
	connect( output, SIGNAL( NewError(const QString &) ), this, SLOT( PrintError(const QString &) ) );

	YApplicationFigures::SetTextViewBackdrop(*this, YApplicationFigures::BACKGROUND_OUTPUTVIEW );

	setTextFormat(Qt::AutoText);
	//setTextFormat(Qt::PlainText);
	//setTextFormat(Qt::RichText);
	PrintComment(MSGP_FIRST_HINT);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool YOutputView::eventFilter(QObject *o, QEvent *e)
{
	// The event filter is needed to receive events occured by pressing the right button of the mouse
	if (e->type() == QEvent::MouseButtonRelease && ((QMouseEvent*)e)->button() == Qt::RightButton )	//&& o == this )
	{
		QPopupMenu	*menu=createPopupMenu( ((QMouseEvent*)e)->globalPos() );
		menu->exec();
		return true;
	}
#if QT_VERSION<300
	return QTextView::eventFilter(o, e);
#else
	return QTextEdit::eventFilter(o, e);
#endif
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Create a popup menu in a QT3 way
QPopupMenu * YOutputView::createPopupMenu( const QPoint & pos )
{
	QPopupMenu	*menu=new QPopupMenu(0);
	connect( menu, SIGNAL( activated(int) ), this, SLOT( PopupMenuActivated(int) ) );
	menu->insertItem( MSGP_OUTPUTVIEW_CLEAR			, POPUP_OUTPUT_CLEAR );
	menu->insertItem( MSGP_OUTPUTVIEW_FULLSCREEN	, POPUP_OUTPUT_FULLSCREEN );
	menu->move( pos );
	return menu;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YOutputView::PopupMenuActivated(int index)
{
	switch (index)
	{
	case POPUP_OUTPUT_CLEAR:		ClearView();
									break;
	case POPUP_OUTPUT_FULLSCREEN:	YMDIWindow::OpenNewWindow( new YOutputWindowFrameArgs( this ), false, true );
									break;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YOutputView::_print(const QString &str)
{
	int	a;
#if QT_VERSION>=300
	append( QString("<br>") + str);
#else
//	append( QString("<br>") + str);
	append( str+"\n");
#endif
	buffer+="<br>";
	buffer+=str;
	a=contentsHeight ();
	scrollBy(0, a );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const QString & YOutputView::GetText()
{
	return buffer;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YOutputWindowFrameArgs::YOutputWindowFrameArgs(YOutputView *o) :
YFrameArgs( YOutputWindowFrame::CreateOutputWindowFrame, MSGP_OUTPUT_TITLE ),
oritextview(o)
{
//	qWarning( o->text() );
}
