//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA XML configuration parser
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/parser.cpp,v 1.3 2003/01/23 09:18:36 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qfile.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <stdio.h>
#include "list.h"
#include "parser.h"
#include "config.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::Parse(const QString &filename, YObject &yobject)
{
	QString		key,keyerror;
	int			retvalue=YASAE_COULD_NOT_OPEN_FILE;

	file.setName(filename);
	if ( file.open(IO_ReadOnly) )
	{
		// file opened successfully
		configstream = new QTextStream( &file );			// use a text stream
		if ( configstream )
		{
			linenumber=1;
			columnnumber=1;
			//sectionkey=
			retvalue=ReadHeader();
			if ( retvalue )
				ParseError(retvalue);
			if ( !retvalue )
			{
				cancelparsing=false;
				retvalue=ParseSection(yobject);
			}
		}
        file.close();
    }
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//\return retvalue=0	Ignore this parsing error and continue
//\return retvalue is unchanged, parsing will be stopped
void YXMLInputParser::ParseError(int &retvalue)
{
	QFileInfo	fileinfo(file);
	if ( retvalue )
	{
		QString	errormsg(MSGP_CONFIG_ERROR);
		errormsg=errormsg.arg( fileinfo.filePath() );
		errormsg=errormsg.arg(linenumber);
		errormsg=errormsg.arg(columnnumber);
		errormsg=errormsg.arg(keyerror);
		if ( QMessageBox::warning( 0, MSGP_ERROR_TITLE, errormsg, MSGP_ERROR_IGNORE,
			 MSGP_ERROR_CANCEL, 0, 0, 1 ) == 0 )
		{
			retvalue=0;
			return;
		}
	}
	cancelparsing=true;
	return;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::ParseSection(YObject &yobject)
{
	int retvalue=0;
	QChar		b;

	while ( ! retvalue )
	{
		retvalue=ReadStartKey(key);
		keyerror=key;
		if ( ! retvalue )
		{
			b=key[0];
			if ( CheckEndingDelimiter(b) )
			{
				key.remove(0,1);												// remove '-'
				if ( sectionkey && key==sectionkey )						// is this section finished ?
				{
					return 0;
				}
				return ReadFinal();
			}
			else
			{
				retvalue=yobject.SetConfig(*this);
/*				if (retvalue && !cancelparsing)
				{
					ParseError(retvalue);
				}
*/			}
		}
		if (retvalue && !cancelparsing)
		{
			ParseError(retvalue);
		}
	};
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::ReadHeader()
{
	QString	tmp;
	int		retvalue;

	retvalue=ReadStartKey(key);
	if ( !retvalue )
	{
		retvalue=YASAE_ERROR_IN_ARGUMENTS;
		tmp=YPC_DOCTYPE;
		tmp+=" ";
		tmp+=doctype;	//YPC_YASA_WORKSPACE;
		tmp=tmp.lower();
		if ( tmp==key )
		{
			retvalue=0;
		}
	}
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::ReadFinal()
{
	QString	tmp;
	int		retvalue;

	retvalue=YASAE_ERROR_IN_ARGUMENTS;
	if ( key==doctype ) //YPC_YASA_WORKSPACE ) )
	{
		retvalue=0;
	}
	else
		ParseError(retvalue);
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Start a new section with own properties
// Output: Error or NULL
int YXMLInputParser::NewSectionSet(const CONFIG_STRING newsectionkey, YSet &set)
{
	int					retvalue;
	const CONFIG_STRING	oldsectionkey=sectionkey;	//set->GetIdentifier();
	YObject				*config;

	key="";
	retvalue=YASAE_OUT_OF_MEMORY;
	config = set.CreateNewItem(key);
	if ( config )
	{
		retvalue=0;
		sectionkey=newsectionkey;
		retvalue=ParseSection( *config );
		sectionkey=oldsectionkey;
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Start a new section with own properties
// Output: Error or NULL
int YXMLInputParser::NewSection(const CONFIG_STRING newsectionkey, YObject &yobject)
{
	int					retvalue;
	const CONFIG_STRING	oldsectionkey=sectionkey;	//set->GetIdentifier();

	sectionkey=newsectionkey;
	retvalue=ParseSection( yobject );
	sectionkey=oldsectionkey;
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output: True, if new section begins
//int YXMLInputParser::CheckSectionStart(QString &key, const char *sectionkey, YSet *sectionconfig)
int YXMLInputParser::CheckSectionStart(const CONFIG_STRING sectionkey)
{
	return QString::compare(sectionkey,key )==0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Read char from input. If this the current char contains multiple chars line Ü= then read it
// INput: key=known key "<deadline>" to determine end
// Output: Error or NULL
int YXMLInputParser::ReadInterpretedChar(QChar &b)
{
	if ( CheckEnd() )
		return YASAE_UNEXPECTED_EOF;
	do
	{
		*configstream>>b;
		if ( b==CONFIG_TAB )
			columnnumber+=4-((columnnumber+4)%4);
		else
			columnnumber++;
	} while ( ! CheckEnd() && ( CheckNewLine(b) ) );
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::IgnoreSpaces(QChar &b)
{
	int		retvalue;
	do
	{
		retvalue=ReadInterpretedChar(b);
	} while ( !retvalue && ( b==CONFIG_SPACE || b==CONFIG_TAB || CheckNewLine(b) ) );
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output:true=new tag begins, target was not changed
bool YXMLInputParser::CheckEndingDelimiter(QChar &b)
{
	if ( b==CONFIG_KEY_END )
		return true;
	return false;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool YXMLInputParser::CheckNewLine(QChar &b)
{
	if ( b==CONFIG_NEWLINE || b==CONFIG_RETURN )
	{
		linenumber++;
		columnnumber=1;
		return true;
	}
	return false;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool YXMLInputParser::CheckEnd()
{
	if ( configstream->atEnd() )
	{
		return true;
	}
	return false;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output:true=new tag begins, target was not changed
bool YXMLInputParser::CheckOpeningTag(QChar &b)
{
	if ( b==CONFIG_KEY_OPEN )
		return true;
	return false;
}
bool YXMLInputParser::CheckClosingTag(QChar &b)
{	if ( b==CONFIG_KEY_CLOSE )
		return true;
	return false;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output:true=new tag begins or old ends, target was not changed
bool YXMLInputParser::CheckTag(QChar &b)
{
	if ( CheckOpeningTag(b) || CheckClosingTag(b) )
		return true;
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Read string from now until new command begins. No spaces, tabs will be removed.
// INput: key=known key "<deadline>" to determine end
// Output:c=Tag which breaks this reading "<" or ">"
int YXMLInputParser::ReadString(QString &target, QChar &c)
{
	int		retvalue;

	while ( ! ( retvalue=ReadInterpretedChar(c ) ) )
	{
		if ( CheckTag(c) )
			break;
		target+=c;
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// reads a command from stream
int YXMLInputParser::ReadStartKey(QString &key)
{
	QString	k;
	QChar	qchar;
	int		retvalue;

	key="";
	retvalue=IgnoreSpaces(qchar);
	if ( ! retvalue )
	{
		if ( CheckOpeningTag(qchar) )
		{
			retvalue=ReadString( k, qchar);				// read content string
			if ( ! retvalue )
			{
				retvalue=YASAE_PARSING_ERROR;
				if ( CheckClosingTag(qchar) )
				{
					retvalue=0;
					key=k.lower();
				}
			}
		}
		else
		{
			// to get an error message
			if ( ! CheckEnd() )
			{
				QChar	b;
				int		number;
				retvalue=YASAE_PARSING_ERROR;
				number=columnnumber;
				ReadString(key, b);
				columnnumber=number;
				key.insert(0,qchar);
			}
		}
	}
	else																// no key found
		retvalue=0;
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// reads a command from stream
int YXMLInputParser::ReadEndKey(QString &key)
{
	QString	k;
	QChar	qchar;
	int		retvalue;

	retvalue=IgnoreSpaces(qchar);
	if ( ! retvalue )
	{
		retvalue=YASAE_PARSING_ERROR;
		if ( CheckEndingDelimiter(qchar) )
		{
			retvalue=ReadString(k, qchar);				// read content string
			if ( ! retvalue )
			{
				retvalue=YASAE_PARSING_ERROR;
				if ( CheckClosingTag(qchar) )
				{
					retvalue=0;
					key=k.lower();
				}
			}
		}
	}
	else
	{																	// no key found
		retvalue=0;
		key="";
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Reads a content string and check the ending tag
int YXMLInputParser::ReadContentString(QString &target)
{
	QChar	qchar;
	QString	content;
	QString	endkey;
	int		retvalue;

	retvalue=ReadString(content, qchar);				// read content string
	if ( ! retvalue )
	{
		retvalue=YASAE_PARSING_ERROR;
		if ( CheckOpeningTag(qchar) )					// "<" from next command
		{
			retvalue=ReadEndKey(endkey );				// read "/" from ending command
			if ( ! retvalue )
			{
				if ( key==endkey )						// read end key == target end key ?
				{
					target=content;
					retvalue=0;
				}
			}
		}
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::ReadContentObject(YDerivedObject &target)
{
	QString			content;
	int				retvalue;

	retvalue=ReadContentString(content);
	if ( ! retvalue )
		retvalue=target.ReadFromString(content);
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::ReadContentInt(int &target)
{
	QString			content;
	int				retvalue;
	int				b=0;
	bool			ok;

	retvalue=ReadContentString(content);
	if ( ! retvalue )
	{
		retvalue=YASAE_ERROR_IN_ARGUMENTS;
		b=content.toInt(&ok);
		if ( ok )
			retvalue=0;
	}
	target=b;
	return retvalue;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLInputParser::ReadContentState(bool &target)
{
	QString	content;
	int		retvalue;

	retvalue=ReadContentString(content);
	if ( ! retvalue )
	{
		content.lower();
		if ( content=="yes" || content=="1" )
		{
			target=true;
			return 0;
		}
		if ( content=="no" || content=="0" )
		{
			target=false;
			return 0;
		}
		retvalue=YASAE_ERROR_IN_ARGUMENTS;
	}
	return retvalue;
}










//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YXMLOutputParser::SPrintfInt64(QString &str, unsigned INT64 time)
{
	char	buffer[30];

	//_snprintf(buffer, 22, "%lld", time);
	sprintf(buffer, "%lld", time);
	str=buffer;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteTabs()
{
	int		count=hierarchy;
	while(count)
	{
		(*configstream)<<CONFIG_TAB;
		count--;
	};
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteNewLine()
{
	(*configstream)<<CONFIG_NEWLINE;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteStartKey(const CONFIG_STRING key)
{
	WriteTabs();
	(*configstream)<<CONFIG_KEY_OPEN;
	(*configstream)<<key;
	(*configstream)<<CONFIG_KEY_CLOSE;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteSetStartKey(const CONFIG_STRING key)
{
	WriteStartKey(key);
	WriteNewLine();
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteEndKey(const CONFIG_STRING key)
{
	(*configstream)<<CONFIG_KEY_OPEN;
	(*configstream)<<CONFIG_KEY_END;
	(*configstream)<<key;
	(*configstream)<<CONFIG_KEY_CLOSE;
	return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteSetEndKey(const CONFIG_STRING key)
{
	WriteTabs();
	WriteEndKey(key);
	WriteNewLine();
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteKey(const CONFIG_STRING key, const QString &str)
{
	WriteStartKey(key);
	(*configstream)<<str;
	WriteEndKey(key);
	WriteNewLine();
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Empty strings will not be written to output
//\return Error or NULL
int YXMLOutputParser::WriteContentString(const CONFIG_STRING key, const QString &s)
{
	if ( s.length() )
		return WriteKey(key, s);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! values of zero will not be written to output
//\return Error or NULL
int YXMLOutputParser::WriteContentInt(const CONFIG_STRING key, int i)
{
	QString		s;
	if ( i )
	{
		s.setNum(i);
		return WriteKey(key,s);
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteContentState(const CONFIG_STRING key, bool state)
{
	QString		s;
	if (state)
		s=YPC_STATE_YES;
	else
		s=YPC_STATE_NO;
	return WriteKey(key,s);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//int YXMLOutputParser::WriteContentYasaTime(const CONFIG_STRING key, YASA_TIME i)
int YXMLOutputParser::WriteContentObject(const CONFIG_STRING key, const YDerivedObject &object)
{
	if ( object.IsValid() )
		return WriteKey(key, object.GetContentString() );
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteHeader()
{
	(*configstream)<<CONFIG_KEY_OPEN;
	(*configstream)<<YPC_DOCTYPE;
	(*configstream)<<" ";
	(*configstream)<<YPC_YASA_WORKSPACE;
	(*configstream)<<CONFIG_KEY_CLOSE;
	return WriteNewLine();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::WriteFinal()
{
	(*configstream)<<CONFIG_KEY_OPEN;
	(*configstream)<<CONFIG_KEY_END;
	(*configstream)<<YPC_YASA_WORKSPACE;
	(*configstream)<<CONFIG_KEY_CLOSE;
	return WriteNewLine();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::Parse(YObject &yobject, const QString &filename)
{
	QFile	file;
	int		retvalue=YASAE_COULD_NOT_OPEN_FILE;

	file.setName(filename);
	if ( file.open(IO_WriteOnly) )
	{
		// file opened successfully
		configstream = new QTextStream( &file );
		if ( configstream )
		{
			retvalue=WriteHeader();
			if ( !retvalue )
			{
				retvalue=yobject.GetConfig(*this);
				if ( !retvalue )
					WriteFinal();
			}
		}
        file.close();
	}
	return retvalue;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::NewSection(const CONFIG_STRING key, YObject &yconfig )
{
	int				retvalue;

	// parse set now
	retvalue=WriteSetStartKey(key);
	if ( !retvalue )
	{
		hierarchy++;
		retvalue=yconfig.GetConfig(*this);
		hierarchy--;
		if ( ! retvalue )
		{
			retvalue=WriteSetEndKey(key);
		}
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YXMLOutputParser::NewSectionSet(const CONFIG_STRING newsectionkey, YSet &yset)
{
	YObject			*object;
	int				retvalue=0;
	unsigned int	index=0;

	while ( !retvalue && ( index<yset.GetCount() ) )
	{
		object=yset.At(index);
		if ( object )
		{
			retvalue=NewSection( newsectionkey, *object );
		}
		index++;
	};
	return retvalue;
}
