//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA XML configuration parser
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/parser.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_PARSER_INCLUDE
#define	YASAGUI_PARSER_INCLUDE

#include <yasagui/yasagui.h>
#include <qobject.h>
#include <qtextstream.h>
#include <qfile.h>

#include "translations.h"
#include "config.h"
#include "datatypes.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSet;
class YObject;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YInputParser
{
public:
					YInputParser()
					{}
	virtual			~YInputParser()
					{}
	virtual int		CheckSectionStart(const CONFIG_STRING sectionkey)=0;
	virtual int		SectionEnd()											{ return 0; }
	virtual int		NewSection(const CONFIG_STRING newsectionkey, YObject &entry)=0;
	virtual int		NewSectionSet(const CONFIG_STRING newsectionkey, YSet &set)=0;
	virtual int		ReadContentString(QString &target)=0;
	virtual int		ReadContentInt(int &target)=0;
	virtual int		ReadContentObject(YDerivedObject &target)=0;
	virtual int		ReadContentState(bool &target)=0;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YXMLInputParser : public YInputParser
{
	QFile			file;
	QString			key;
	QString			keyerror;
	QString			doctype;
	CONFIG_STRING	sectionkey;
	bool			cancelparsing;
	QTextStream		*configstream;
	int				linenumber;
	int				columnnumber;
	int				SetConfig();
	int				GetConfig();
	int				ReadInterpretedChar(QChar &b);
	bool			CheckEndingDelimiter(QChar &b);
	bool			CheckNewLine(QChar &b);
	bool			CheckOpeningTag(QChar &b);
	bool			CheckClosingTag(QChar &b);
	bool			CheckTag(QChar &b);
	int				IgnoreSpaces(QChar &b);
	int				ReadStartKey(QString &key);
	int				ReadEndKey(QString &key);
	int				ReadString(QString &target, QChar &c);
	bool			CheckEnd();
	int				ReadHeader();
	int				ReadFinal();
	int				ParseSection(YObject &yconfig);
	void			ParseError(int &retvalue);						//! Error dialog

public:
					YXMLInputParser(const QString &dt) :
					doctype(dt.lower()),
					sectionkey(0),
					cancelparsing(true),
					configstream(0),
					linenumber(0),
					columnnumber(0)
					{}
	virtual			~YXMLInputParser()
					{}
	virtual int		CheckSectionStart(const CONFIG_STRING sectionkey);
	virtual int		NewSection(const CONFIG_STRING sectionkey, YObject &entry);
	virtual int		NewSectionSet(const CONFIG_STRING sectionkey, YSet &set);
	virtual int		ReadContentString(QString &target);
	virtual int		ReadContentInt(int &target);
	virtual int		ReadContentObject(YDerivedObject &target);
	virtual int		ReadContentState(bool &target);
	int				Parse(const QString &filename, YObject &yobject);
};







//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YOutputParser
{
public:					YOutputParser()
						{}
	virtual int			WriteContentString(const CONFIG_STRING key, const QString &s) =0;
	virtual int			WriteContentInt(const CONFIG_STRING key, int value) =0;
	virtual int			WriteContentObject(const CONFIG_STRING key, const YDerivedObject &object) =0;
	virtual int			WriteContentState(const CONFIG_STRING key, bool state) =0;

	virtual int			NewSection(const CONFIG_STRING newsectionsetkey, YObject &yobject ) =0;
	virtual int			NewSectionSet(const CONFIG_STRING newsectionkey, YSet &yset ) =0;
	virtual int			SectionEnd()													{ return 0; }
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YXMLOutputParser : public YOutputParser
{
	int					hierarchy;
	QTextStream			*configstream;
	int					WriteKey(const CONFIG_STRING key, const QString &str);
	int					WriteStartKey(const CONFIG_STRING key);
	int					WriteEndKey(const CONFIG_STRING key);
	inline int			WriteTabs();
	inline int			WriteNewLine();
	inline int			WriteSetStartKey(const CONFIG_STRING key);
	inline int			WriteSetEndKey(const CONFIG_STRING key);
	virtual int			WriteHeader();
	virtual int			WriteFinal();
	void				SPrintfInt64(QString &str, unsigned INT64 time);
public:
						YXMLOutputParser() :
						hierarchy(0),
						configstream(0)
						{}
	virtual int			WriteContentString(const CONFIG_STRING key, const QString &s);
	virtual int			WriteContentInt(const CONFIG_STRING key, int value);
	virtual int			WriteContentObject(const CONFIG_STRING key, const YDerivedObject &object);
	virtual int			WriteContentState(const CONFIG_STRING key, bool state);

	virtual int			NewSection(const CONFIG_STRING newsectionsetkey, YObject &yobject );
	virtual int			NewSectionSet(const CONFIG_STRING newsectionkey, YSet &yset );
	virtual int			Parse(YObject &yobject, const QString &filename);
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_PARSER_INCLUDE
