//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA plugin's
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/plugins.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_PLUGINS_INCLUDE
#define	YASAGUI_PLUGINS_INCLUDE

#include <yasagui/yasagui.h>
#include <qobject.h>
#include <qlist.h>
#include "workspace.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// YObject::name is set to shortname too by default, but can be overidden during GetConfig
class YPlugIn : public YObject
{	Q_OBJECT
	QString					path;						// path to directory e.g. "/schedulers/llf"
	QString					shortname;					// e.g. EDF, "Earliest Deadline First"
	bool					support_calcpreemptiontime;
	bool					support_timertick;
	YASA_MAXCPU				maxcpu;
	YASA_TICK				min_timertick;
public:						YPlugIn(const YObjectArgs &args) :
							YObject(args),
							support_calcpreemptiontime(false),
							support_timertick(false),
							min_timertick((YASA_TIME)YG_MIN_TIMER_TICK)
							{}
	virtual int				InitInstance(const QString &p, const QString &dt);
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
	bool					GetSupportCalcPreemptionTime() const			{	return support_calcpreemptiontime;	}
	bool					GetSupportTimerTick() const						{	return support_timertick;			}
	YASA_TIME				GetMinTimerTick() const							{	return min_timertick.GetContent();	}
	const YASA_MAXCPU&		GetMaxCPU() const								{	return maxcpu;						}
	const QString &			GetPath() const									{	return path;						}
	//! Returns "ellf" instead "Enhanced least laxity first"
	const QString &			GetShortName() const							{	return shortname;					}
	void					SetShortName(const QString &sn)					{	shortname=sn;						}
	const QString &			GetFullName() const								{	return GetName();					}
	void					SetFullName(const QString &fn)					{	SetName(fn);						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerPlugIn : public YPlugIn
{	YASA_SCHEDULERTYPE		type;
public:						YSchedulerPlugIn(const YObjectArgs &args) :
							YPlugIn(args)
							{}
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YExecutivePlugIn : public YPlugIn
{	//! if true this executive can work with foreign source code (e.g. rtlinux)
	bool					sourcecodesupport;
	//! if true=this executive can work with different number of CPU's (useful for simulators)
	bool					unlimitedcpusupport;
	//! if true=this executive can run threads on different cpu (once cpu0 next cpu1 like Linux)
	bool					changeablecpusupport;
	//! if true=this executive supports resume times
	bool					resumetimesupport;
	//! true=this executive supports resource actions
	bool					resourceactionsupport;
	//! true=this executive supports emergency threads
	bool					emergencythreadsupport;
	//! true=this executive supports the cancellation of threads if the deadline is missed
	bool					cancelthreadsupport;
	//! true=this executive supports a flexible scheduling time (e.g. simulators)
	bool					schedulingtimesupport;
	//! true=this executive supports a synchronized start offset (e.g. simulators, rtlinux)
	bool					synchronizationoffset;
	YString					gcccompileroptions;
	YString					idlename;
public:						YExecutivePlugIn(const YObjectArgs &args) :
							YPlugIn(args),
							sourcecodesupport(false),
							unlimitedcpusupport(false),
							changeablecpusupport(false),
							resumetimesupport(false),
							resourceactionsupport(false),
							emergencythreadsupport(false),
							cancelthreadsupport(false),
							schedulingtimesupport(false),
							synchronizationoffset(false),
							idlename(MSGP_LF_TASK_IDLE)
							{}
	bool					GetUnlimitedCPUSupport() const					{	return unlimitedcpusupport;			}
	bool					GetChangeableCPUSupport() const					{	return changeablecpusupport;		}
	bool					GetSourceCodeSupport() const					{	return sourcecodesupport;			}
	bool					GetResumeTimeSupport() const					{	return resumetimesupport;			}
	bool					GetResourceActionsSupport() const				{	return resourceactionsupport;		}
	bool					GetEmergencyThreadSupport() const				{	return emergencythreadsupport;		}
	bool					GetCancelThreadSupport() const					{	return cancelthreadsupport;			}
	bool					GetSchedulingTimeSupport() const				{	return schedulingtimesupport;		}
	bool					GetSynchronizationOffsetSupport() const			{	return synchronizationoffset;		}
	const YString &			GetGCCCompilerOptions() const					{	return gcccompileroptions;			}
	const YString &			GetIdleName() const								{	return idlename;					}
	QStringList				GetFeatureList() const;
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPlugInList : public YSet
{	QString					subdirectory;				//! name of subdirectory, e.g. "schedulers" or "executives"
	QString					doctype;
public:						YPlugInList(const QString &name, const QString &subdir, const QString &key, const QString &dt ) :
							YSet(key, 0, YObjectArgs(name) ),
							subdirectory(subdir),
							doctype(dt)
							{}
	int						InitInstance();
	//! Search for full name in YSet by locking for the given shortname. If there could no YPlugIn found
	//! the original shortname is returned.
	const QString &			FindLongName(const QString &shortname)
	{
		for (unsigned int i=0; i<GetCount(); i++)
		{
			if ( ((YPlugIn*)At(i))->GetShortName()==shortname )
				return ((YPlugIn*)At(i))->GetName();
		}
		return shortname;
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerPlugIns : public YPlugInList
{
public:						YSchedulerPlugIns(const QString &subdir) :
							YPlugInList(YPC_SCHEDULER, subdir, MSGP_PLUGIN_SCHEDULERS, YPC_YASA_SCHEDULER)
							{
							}
	static YSchedulerPlugIns& GetSchedulerPlugIns();
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)		{ return new YSchedulerPlugIn(args);	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YExecutivePlugIns : public YPlugInList
{
public:						YExecutivePlugIns(const QString &subdir) :
							YPlugInList(YPC_EXECUTIVENAME, subdir, MSGP_PLUGIN_EXECUTIVES, YPC_YASA_EXECUTIVE)
							{
							}
	static YExecutivePlugIns& GetExecutivePlugIns();
	const QString &			GetIdleName( const QString &executivename, const QString &idle)
	{
		for (unsigned int i=0; i<GetCount(); i++)
		{
			if (	((YExecutivePlugIn*)At(i))->GetShortName()==executivename ||
					((YExecutivePlugIn*)At(i))->GetFullName() ==executivename	)
			{
				return ((YExecutivePlugIn*)At(i))->GetIdleName().GetContent();
			}
		}
		return idle;
	}
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)		{ return new YExecutivePlugIn(args);	}
};



















//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPlugInManager
{
	YSchedulerPlugIns		schedulerplugins;
	YExecutivePlugIns		executiveplugins;
public:
							YPlugInManager() :
							schedulerplugins( QString(SCHEDULERS_DIR) ),
							executiveplugins( QString(EXECUTIVES_DIR) )
							{
							}
	int						InitInstance();
	YSchedulerPlugIns&		GetSchedulerPlugIns()					{ return schedulerplugins;	}
	YExecutivePlugIns&		GetExecutivePlugIns()					{ return executiveplugins;	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif															// ifndef YASAGUI_PLUGINS_INCLUDE
