//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Result diagram windows implementation
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/04/18
// $Header: /sources/yasa/yasagui/resultwindows.cpp,v 1.3 2003/01/23 09:18:37 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "resultwindows.h"
#include "timinghandler.h"
#include "timings.h"


#define MSGP_ENVIRONMENT_STATISTIC_TITLE	qApp->translate("Statisticwindow", " - Statistic"			, "add-on of environment window title"	)
#define MSGP_PROJECT_STATISTIC_TITLE		qApp->translate("Statisticwindow", " - Statistics"			, "add-on of environment window title"	)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YResultFrame::~YResultFrame()
{
	if (resulthandler)
		delete resulthandler;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTimingFrameObjectArgs::YTimingFrameObjectArgs(CreateYFrameCall call, YTimingResults *timingresult) :
YFrameObjectArgs(call, timingresult)
{
	if ( timingresult && timingresult->GetParent() && timingresult->GetParent()->GetParent() )
	{
		switch ( timingresult->GetParent()->GetResultArgs().GetResultType() )
		{
		case YResultArgs::PROJECT_RESULTS:		SetTitle( timingresult->GetParent()->GetParent()->GetName() + MSGP_PROJECT_TIMING_TITLE );
												break;
		case YResultArgs::ENVIRONMENT_RESULTS:	SetTitle( timingresult->GetParent()->GetParent()->GetName() + MSGP_ENVIRONMENT_TIMING_TITLE );
												break;
		default:								break;
		}
	}
}
YStatisticFrameObjectArgs::YStatisticFrameObjectArgs(CreateYFrameCall call, YStatisticResults *statisticresult) :
YFrameObjectArgs(call, statisticresult)
{
	if ( statisticresult && statisticresult->GetParent() && statisticresult->GetParent()->GetParent() )
	{
		switch ( statisticresult->GetParent()->GetResultArgs().GetResultType() )
		{
		case YResultArgs::PROJECT_RESULTS:		SetTitle( statisticresult->GetParent()->GetParent()->GetName() + MSGP_PROJECT_STATISTIC_TITLE );
												break;
		case YResultArgs::ENVIRONMENT_RESULTS:	SetTitle( statisticresult->GetParent()->GetParent()->GetName() + MSGP_ENVIRONMENT_STATISTIC_TITLE );
												break;
		default:								break;
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YResultFrame::InitInstance()
{
	int	retvalue=YASAE_OUT_OF_MEMORY;
	resulthandler=CreateResultHandler(&splitter);
	if ( resulthandler)
	{
		retvalue=YObjectFrame::InitInstance();
		if (!retvalue)
		{
			retvalue=InsertResults(resulthandler);
		}
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! One of the logfiles was updated
void YResultFrame::UpdateResults()
{
	if ( resulthandler )
	{
		resulthandler->ClearResults();
		if ( InsertResults( resulthandler ) || resulthandler->PostUpdate() )
			CloseFrame();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironmentResultFrame::PostCreateTimingFrame()
{
	YEnvironment		*environment	= GetEnvironment();

	// make connections
	// close this environment timing diagram if the log file will be removed
	connect( environment, SIGNAL( EnvironmentCleanedUp() ), this, SIGNAL( CloseFrame() ) );
	connect( &environment->GetEnvironmentLogFileSet(), SIGNAL( LogFileUpdated() ), this, SLOT( UpdateResults() ) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YProjectResultFrame::PostCreateTimingFrame()
{
	YProject	*project	= GetProject();
	// make connections
	// update this project timing diagram if the log file will be removed
	for (unsigned int i=0; i<project->GetEnvironmentSet().GetCount(); i++)
	{
		connect( &((YEnvironment*)project->GetEnvironmentSet().At(i))->GetEnvironmentLogFileSet(),
					SIGNAL( LogFileUpdated() ), this, SLOT( UpdateResults() ) );
	}
	connect( &project->GetEnvironmentSet(), SIGNAL( NewItem(YObject *) ), this, SLOT( NewEnvironment(YObject *) ) );
	connect( &project->GetEnvironmentSet(), SIGNAL( ItemRemoved(YObject *) ), this, SLOT( RemoveEnvironment(YObject *) ) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new environment was added
void YProjectResultFrame::NewEnvironment(YObject *object)
{
	connect( &((YEnvironment*)object)->GetEnvironmentLogFileSet(), SIGNAL( LogFileUpdated() ), this, SLOT( UpdateTimings() ) );
	UpdateResults();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! An environment was removed
void YProjectResultFrame::RemoveEnvironment(YObject *object)
{
	disconnect( &((YEnvironment*)object)->GetEnvironmentLogFileSet(), SIGNAL( LogFileUpdated() ), this, SLOT( UpdateResults() ) );
	connect( object, SIGNAL( destroyed() ), this, SLOT( UpdateResults() ) );
}

























