//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA settings management
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/settings.h,v 1.5 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_SETTINGS_INCLUDE
#define	YASAGUI_SETTINGS_INCLUDE

#include <qobject.h>
#include <qlistview.h>
#include <qstringlist.h>
#include <qtextview.h>
#include <qpixmap.h>
#include <yasa/yasa_types.h>
#include <yasagui/yasagui_customize.h>
#include <qapplication.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YApplication;
class YPixmap;
class QMainWindow;
class QAction;
class YPainter;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define COLOR_DEFAULT		QColor(250,250,250)	//QApplication::desktop()->backgroundColor()
#define COLOR_TIMINGWINDOW	QColor(240,222,208)
//#define COLOR_TIMINGWINDOW	QColor(255,255,255)
#define COLOR_SYSTEM_TASK	QColor(0,120,150)
#define COLOR_TASK			QColor(0,150,0)
#define COLOR_TASK_IDLE		QColor(150,150,150)
#define COLOR_EXCLUDED_TASK	QColor(240,222,208)
#define COLOR_MUTEX			QColor(150,150,0)
#define COLOR_JITTER		QColor(200,100,100)


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPixmapArgs
{	bool				warn_if_failed;
public:					YPixmapArgs(bool w=true) : warn_if_failed(w) {}
	bool				GetWarningMode() const							{	return warn_if_failed;	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPixmap
{	QString				filename;
	QColor				color;
	QBrush				brush;
	QPixmap				*pixmap;
	bool				tried_n_failed;
	bool				warn_if_failed;								// warn if loading failed (e.g. print an output warning)
	bool				IsPixmapValid() const					{	return pixmap;		}
	int					LoadPixmap();
public:					YPixmap(const QString &f, const QColor &c, const YPixmapArgs &args=YPixmapArgs() ) :
						filename(f),
						color(c),
						brush(c),
						pixmap(0),
						tried_n_failed(false),
						warn_if_failed( args.GetWarningMode() )
						{ }
						YPixmap(const QBrush &b, QBrush::BrushStyle s, const YPixmapArgs &args=YPixmapArgs() ) :
						color( COLOR_DEFAULT ),
						brush( b ),
						pixmap(0),
						tried_n_failed(false),
						warn_if_failed( args.GetWarningMode() )
						{
							brush.setStyle(s);
						}
	//! Attention: Before calling the destructor be sure that no client is using the pixmap !
						~YPixmap()
						{
							if (pixmap)
								delete pixmap;
						}
	const QPixmap *		GetPixmap()
						{	if (this && ! LoadPixmap() )
								return pixmap;
							return 0;
						}
	const QBrush *		GetBrush()
						{	LoadPixmap();
							return &brush;
						}
	bool				GetTriedAndFailed() const					{	return tried_n_failed;		}
	void				WarnIfFailed(bool w)						{	warn_if_failed=w;			}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPixmapSettings : public QObject
{	Q_OBJECT
protected:
	QList<YPixmap>		pixmaps;
	//! Insert a figure to the internal list. The figure will not be loaded! It will be loaded if it is used
	//! the first time. filedir can be a relative path from $YASA
	void				InsertFigure(const QString &filedir, const QColor &color=COLOR_DEFAULT );
	void				InsertBrush(const QBrush &brush, QBrush::BrushStyle style);
public:					YPixmapSettings()
						{
							pixmaps.setAutoDelete(TRUE);
						}
	virtual int			InitInstance()							{	return 0;				}
	virtual YPixmapArgs	GetPixmapArgs()							{	return YPixmapArgs();	}
	YPixmap *			GetPixmap(unsigned int index)			{
																	if (  index <= pixmaps.count() )
																	{
																		return pixmaps.at(index);
																	}
																	return 0;
																}
	const QBrush*		GetBrush(unsigned int index)			{
																	if ( index <= pixmaps.count() )
																	{
																		return pixmaps.at(index)->GetBrush();
																	}
																	return 0;
																}
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class provides
class YApplicationFigures : public YPixmapSettings
{	Q_OBJECT
public:					YApplicationFigures() {} //YApplication *app) : YPixmapSettings(app) {}
	virtual int			InitInstance()
						{
							InsertFigure( FIGURE_BACKGROUND_OUTPUT );							// "BACKGROUND_OUTPUT" - set output background image
							InsertFigure( FIGURE_BACKGROUND_HELP );								// "BACKGROUND_HELP" - set output background of help window
							InsertFigure( FIGURE_BACKGROUND_SOURCES );							// "BACKGROUND_SOURCES" - set output background of sources view
							InsertFigure( FIGURE_BACKGROUND_LOGFILE );							// "BACKGROUND_LOGFILE" - set output background of logfile view
							InsertFigure( FIGURE_BACKGROUND_WORKSPACE_TREE );					// "BACKGROUND_WORKSPACE_TREE" - set output background image
							InsertFigure( FIGURE_BACKGROUND_TIMINGWINDOWS, COLOR_TIMINGWINDOW );// "BACKGROUND_WINDOWS_TREE" - set output background image
							InsertFigure( FIGURE_BACKGROUND_WINDOWS );							// "BACKGROUND_WINDOWS_TREE" - set output background image
							InsertFigure( FIGURE_ICON_APPLICATION );							// "ICON_APPLICATION"
							InsertFigure( FIGURE_ICON_MDIWINDOW );								// "ICON_WINDOW"
							InsertFigure( FIGURE_WORKSPACE_NEW );								// "WORKSPACE_NEW"
							InsertFigure( FIGURE_WORKSPACE_OPEN );								// "WORKSPACE_OPEN"
							InsertFigure( FIGURE_WORKSPACE_SAVE );								// "WORKSPACE_SAVE"
							InsertFigure( FIGURE_EXECUTE );										// "EXECUTE"
							InsertFigure( FIGURE_ZOOM_IN );										// "ZOOM_IN"
							InsertFigure( FIGURE_ZOOM_OUT );									// "ZOOM_OUT"
							InsertFigure( FIGURE_POINTER_HAND );								// "POINTER_HAND"
							InsertFigure( FIGURE_POINTER_HAND_PUSHED );							// "POINTER_HAND_PUHSED"
							GetPixmap(BACKGROUND_TIMINGWINDOWS)->WarnIfFailed(false);
							return YPixmapSettings::InitInstance();
						}
	enum PIXMAP_TYPE	{	BACKGROUND_OUTPUTVIEW=0,
							BACKGROUND_HELPVIEW,
							BACKGROUND_SOURCESVIEW,
							BACKGROUND_LOGFILEVIEW,
							BACKGROUND_WORKSPACE_TREE,
							BACKGROUND_TIMINGWINDOWS,
							BACKGROUND_WINDOWS,
							ICON_APPLICATION,
							ICON_MDIWINDOW,
							WORKSPACE_NEW,
							WORKSPACE_OPEN,
							WORKSPACE_SAVE,
							EXECUTE,
							ZOOM_IN,
							ZOOM_OUT,
							POINTER_HAND,
							POINTER_HAND_PUSHED,
							MAX_FIGURES	};
	static void			SetTextViewBackdrop(QTextView &w, YApplicationFigures::PIXMAP_TYPE type);
	static void			SetListViewBackdrop(QListView &lv, YApplicationFigures::PIXMAP_TYPE type);
	static void			SetIcon(QMainWindow &mv, YApplicationFigures::PIXMAP_TYPE type);
	static void			SetIconSet(QAction &a, YApplicationFigures::PIXMAP_TYPE type);
	static int			CreateCursor(QCursor **c, YApplicationFigures::PIXMAP_TYPE releasedtype);
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class provides
class YLogFileFigures : public YPixmapSettings
{	Q_OBJECT
public:					YLogFileFigures() {} //YApplication *app) : YPixmapSettings(app) {}
	virtual int			InitInstance()
						{	// must be the same sequence like the types defined in LOGGING_ACTIONS
							InsertFigure( QString::null );					// Unknown
							InsertFigure( FIGURE_THREAD_CREATED );			// 1
							InsertFigure( FIGURE_THREAD_ACTIVATE );
							InsertFigure( FIGURE_THREAD_EXIT );
							InsertFigure( FIGURE_THREAD_START );
							InsertFigure( FIGURE_THREAD_CONT );
							InsertFigure( FIGURE_THREAD_SUSPEND );
							InsertFigure( FIGURE_THREAD_TOLERANCE_START );	// 7
							InsertFigure( FIGURE_THREAD_TOLERANCE_CONT );	// 8
							InsertFigure( FIGURE_THREAD_TOLERANCE_SUSPEND );// 9
							InsertFigure( FIGURE_THREAD_SKIPPED );			// 10
							InsertFigure( FIGURE_THREAD_TYPE );				// 11
							InsertFigure( FIGURE_THREAD_EXCLUDED );			// 12
							InsertFigure( FIGURE_DEADLINE_MATCH );
							InsertFigure( FIGURE_DEADLINE_MISS );
							InsertFigure( FIGURE_DEADLINE_MISS_DETECTED );	// 15
							InsertFigure( FIGURE_DEADLINE_TOLERANCE_MISS );
							InsertFigure( FIGURE_TIME_TO_COMPUTE );
							InsertFigure( FIGURE_MUTEX_CREATED );
							InsertFigure( FIGURE_MUTEX_DESTROYED );
							InsertFigure( FIGURE_MUTEX_LOCKED );			// 20
							InsertFigure( FIGURE_MUTEX_TRY_N_BLOCKED );
							InsertFigure( FIGURE_MUTEX_UNLOCKED );
							InsertFigure( FIGURE_MUTEX_RELOCKED );
							InsertFigure( FIGURE_MUTEX_NOT_LOCKED );
							InsertFigure( FIGURE_PRIORITY_CHANGED );		// 25
							InsertFigure( FIGURE_SYSTEM_CEILING_CHANGED );
							InsertFigure( FIGURE_MESSAGE );
							InsertFigure( FIGURE_EMERGENCY_THREAD );
							InsertFigure( FIGURE_SCHEDULER_CHANGED );
							InsertFigure( FIGURE_SCHEDULER_CALLED );		// 30
							InsertFigure( FIGURE_SCHEDULER_FINISHED );
							InsertFigure( FIGURE_SCHEDULER_SWITCH );
							InsertFigure( FIGURE_SCHEDULER_SYNCHRONIZED_START );
							InsertFigure( FIGURE_IDLE_THREAD );
							InsertFigure( FIGURE_JITTER );
#if YASAL_MAX!=35
#error There are inconsistences handling logging types!
#endif
							// insert task, mutex, task excluded brushes
							InsertBrush( COLOR_SYSTEM_TASK	, QBrush::SolidPattern );		// system task
							InsertBrush( COLOR_TASK			, QBrush::SolidPattern );		// task
							InsertBrush( COLOR_TASK_IDLE	, QBrush::SolidPattern );		// idle task
							InsertBrush( COLOR_EXCLUDED_TASK, QBrush::Dense6Pattern );		// excluded
							InsertBrush( COLOR_MUTEX		, QBrush::SolidPattern );		// mutex
							InsertBrush( COLOR_JITTER		, QBrush::Dense4Pattern );		// jitter
							return YPixmapSettings::InitInstance();
						}
	enum PIXMAP_TYPE	{	SYSTEM_TASK_BRUSH=YASAL_MAX+1,
							TASK_BRUSH,
							TASK_IDLE_BRUSH,
							TASK_EXCLUDED_BRUSH,
							MUTEX_BRUSH,
							JITTER_BRUSH,
							MAX_LOGGINGFIGURES
							};

	//void				DrawPixmap(YPainter &o, int x, int y, unsigned int type);
	const QPixmap*		GetLogFilePixmap(unsigned int type)
						{
							const QPixmap *pixmap=0;
							if (  type <= MAX_LOGGINGFIGURES )
							{
								pixmaps.at(type);
								if ( ! pixmaps.current()->GetTriedAndFailed() )
									pixmap=pixmaps.current()->GetPixmap();
							}
							return pixmap;
						}
	const QBrush *		GetLogFileBrush(unsigned int type)
						{
							if (  type <= MAX_LOGGINGFIGURES )
							{
								return pixmaps.at(type)->GetBrush();
							}
							return 0;
						}
	virtual YPixmapArgs	GetPixmapArgs()							{	return YPixmapArgs(false);		}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSettings
{
	YApplicationFigures	applicationfigures;
	YLogFileFigures		logfilefigures;
public:					YSettings() {}
	int					InitInstance()
						{
							int retvalue;
							retvalue=applicationfigures.InitInstance();
							if (retvalue)
								return retvalue;
							retvalue=logfilefigures.InitInstance();
							return retvalue;
						}
	YApplicationFigures&GetApplicationFigures()							{	return applicationfigures;		}
	YLogFileFigures &	GetLogFileFigures()								{	return logfilefigures;		}
};









//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_SETTINGS_INCLUDE
