//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA sources support (managing source and header files)
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/sources.cpp,v 1.4 2003/01/30 09:29:55 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "list.h"
#include "actionhandler.h"
#include "filedialog.h"
#include "parser.h"
#include "project.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YFilePath::YFilePath(const YObjectArgs &args) :
YObject(args)
{}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YFilePath::GetAbsPath()
{
	if ( GetSet() && GetSet()->GetParent() )
	{
		QString	abspath;
		YProject *project;
		project=(YProject*)(GetSet()->GetParent());
		abspath=project->CopyProjectDir(true);
		YFileOperations::AddSubDirectory(abspath, path);
		return abspath;
	}
	return QString(path);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFilePath::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_PATH) )
		return parser.ReadContentString(path);
	return YObject::SetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFilePath::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	retvalue=parser.WriteContentString(YPC_PATH, path);
		if ( retvalue )	return retvalue;
	return YObject::GetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Insert a new item into the string list. Empty item names can not be created
// Output: Error or NULL
YObject* YFilePathSet::AllocNewItem(const YObjectArgs &args)
{
	int			k=GetCount();

	if ( ! args.GetName().length() )
		return 0;
	if ( checkdouble )
	{
		for ( k=0; k<(int)GetCount() ; k++)
		{
			if ( args.GetName() == At(k)->GetName() )
				return 0;
		}
	}
	return new YFilePath(args);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YSourceSet::YSourceSet() :
YFilePathSet(YPC_HEADER, MSGP_DEFAULT_SOURCENAME, YActionHandler::GetActions(YActionHandler::SOURCESET_ACTIONS) )
{
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YHeaderSet::YHeaderSet() :
YFilePathSet(YPC_HEADER, MSGP_DEFAULT_HEADERNAME, YActionHandler::GetActions(YActionHandler::HEADERSET_ACTIONS) )
{
}
