//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Statistic handler code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/04/18
// $Header: /sources/yasa/yasagui/statistichandler.cpp,v 1.3 2003/01/23 09:18:38 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "statistichandler.h"
#include "statistics.h"
#include "application.h"
#include "output.h"



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YStatisticsInfo::~YStatisticsInfo()
{
	if ( statistics )
		delete statistics;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new statistic table is inserting. InitInstance will called here !
YStatisticHandler::YStatisticHandler(QWidget *p) : parent(p)
{
	list.setAutoDelete(true);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new statistic table is inserting. InitInstance will called here !
int YStatisticTableHandler::InsertStatistics(YStatistics *statistics)
{
	YStatisticsInfo	*si;
	int				retvalue=YASAE_OUT_OF_MEMORY;

	if ( statistics )
	{
		retvalue=statistics->InitInstance(this, scrollframe);
		if ( ! retvalue )
		{
			si = new YStatisticsInfo();
			if ( si )
			{
				si->SetStatistics(statistics);
				/*connect(	this					, SIGNAL( NewStatisticsInserted(YStatisticsInfo *) ),
							statistics->GetTable()	, SLOT( NewStatisticsInserted(YStatisticsInfo *) ) );*/
				vboxlayout->addWidget( statistics->GetMainWidget() );
				list.append(si);
				emit NewStatisticsInserted(si);
				retvalue=0;
			}
		}
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Post create statistic handler. Here we are resizing the vertical headers
int YStatisticTableHandler::PostCreate()
{
	int				headerwidth=0, columnwidth=0;
	unsigned int	i;
	YStatisticTable	*table;
	YStatistics		*statistics;

	// determine max headerwidth/columnwidth
	for (i=0; i<list.count(); i++)
	{
		statistics=list.at(i)->GetStatistics();
		if ( statistics )
		{
			table=statistics->GetTable();
			if ( table )
			{
				if ( table->GetMaxVerticalHeaderWidth() > headerwidth )
					headerwidth=table->GetMaxVerticalHeaderWidth();
				if ( table->GetMaxColumnWidth() > columnwidth )
					columnwidth=table->GetMaxColumnWidth();
			}
		}
	}
	// set max headerwidth/columnwidth in each QTable
	for (i=0; i<list.count(); i++)
	{
		statistics=list.at(i)->GetStatistics();
		if ( statistics )
		{
			table=statistics->GetTable();
			if ( table )
			{
				table->SetVerticalHeaderWidth(headerwidth);
				table->SetColumnWidth(columnwidth);

				if ( statistics->GetLabel() && statistics->GetMainWidget() )
				{
					statistics->GetLabel()->setText( QString("<h3>%1</h3>").arg( statistics->GetTableTitle() ) );
					statistics->GetMainWidget()->setMaximumHeight( statistics->GetLabel()->height() + table->height() );
				}
			}
		}
	}

	if ( list.count() )
	{
		if ( list.at(0)->GetStatistics() && list.at(0)->GetStatistics()->GetTable() && list.at(0)->GetStatistics()->GetTable()->numCols() )
			return 0;
	}

	YApplication::GetOutput()->Warning(MSGP_SR_WINDOW_CLOSED);
	return YASAE_ERROR_IN_ARGUMENTS;
}
