//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA statistic window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/03/18
// $Header: /sources/yasa/yasagui/statistichandler.h,v 1.4 2003/01/24 15:47:43 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_STATISTIC_HANDLER_INCLUDE
#define	YASAGUI_STATISTIC_HANDLER_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qscrollview.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <qframe.h>
#include "timingsettings.h"
#include "settings.h"
#include "resulthandler.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStatistics;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class is used to store information about the statistic infos
class YStatisticsInfo
{	//! ptr to statistics class
	YStatistics			*statistics;
public:					YStatisticsInfo() : statistics(0)			{									}
						~YStatisticsInfo();
	YStatistics *		GetStatistics()								{	return statistics;				}
	void				SetStatistics(YStatistics *t)				{	statistics=t;					}
};





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The YStatisticHandler provides an interface to handle statistic results
class YStatisticHandler : public YResultHandler
{	Q_OBJECT
	QWidget				*parent;
protected:
	QList<YStatisticsInfo>	list;
public:					YStatisticHandler(QWidget *p);
	virtual int			InsertStatistics(YStatistics *t) =0;
	QList<YStatisticsInfo>&	GetStatisticsList()						{	return list;					}
	virtual void		ClearResults()								{	list.clear();					}
signals:
	void				NewStatisticsInserted(YStatisticsInfo *si);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStatisticTableHandler : public YStatisticHandler
{	QScrollView			scrollview;
	QFrame				*scrollframe;
	QVBoxLayout			*vboxlayout;
public:					YStatisticTableHandler(QWidget *p) : YStatisticHandler(p), scrollview(p), scrollframe(0), vboxlayout(0)
						{
							InitScrollFrame();
						}
	void				DestroyScrollFrame()
						{
							if (scrollframe)
							{
								scrollview.removeChild(scrollframe);
								scrollframe->close();
							}
							scrollframe=0;
						}
	int					InitScrollFrame()
						{
							DestroyScrollFrame();
							scrollframe	= new QFrame( &scrollview );
							vboxlayout	= new QVBoxLayout(scrollframe, MDI_SCROLLVIEW_BORDER );
							scrollview.addChild(scrollframe);
							scrollview.setResizePolicy(QScrollView::AutoOneFit);
							return 0;
						}
	//! A new statistic table is inserting. InitInstance will called here !
	virtual int			InsertStatistics(YStatistics *t);
	virtual int			PostCreate();
	//! This function can be used to recreate destroyed widgets (e.g. during ClearResults() )
	virtual int			PostUpdate()
						{
							scrollframe->show();
							return 0;
						}
	virtual void		ClearResults()								{	DestroyScrollFrame();
																		YStatisticHandler::ClearResults();
																		InitScrollFrame();			}
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_TIMING_HANDLER_INCLUDE
