//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project, task support
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/task.cpp,v 1.3 2003/01/23 09:18:38 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "task.h"
#include "actionhandler.h"
#include "parser.h"
#include "project.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTaskSet::YTaskSet(YTask *parent) :
YSet(YPC_TASK, MSGP_DEFAULT_TASKNAME, YObjectArgs(MSGP_CAPTION_TASKS, YActionHandler::GetActions(YActionHandler::TASKSET_ACTIONS))),
parenttask(parent)
{
}

YObject* YTaskSet::AllocNewItem(const YObjectArgs &args)
{
	return new YTask(args);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTask::YTask(const YObjectArgs &args) :
YObject(args),
parenttaskset(0),
cpu( & GetProject()->GetMaxCPU(), YG_DEFAULT_THREAD_CPU ),
subtaskset(0)
{
	subtaskset.SetParent(this);
	resourceactionset.SetParent(this);
	resumetimeset.SetParent(this);
	connect( &subtaskset	  , SIGNAL(UpdateItem(YObject *)), this, SLOT( SubTaskChanged(YObject *) ) );
	//connect( &subtaskset	  , SIGNAL(NewItem(YObject *)), this, SLOT( SubTaskChanged(YObject *) ) );
	//connect( &subtaskset	  , SIGNAL(ItemRemoved(YObject *)), this, SLOT( SubTaskChanged(YObject *) ) );
	connect( &resumetimeset	  , SIGNAL(UpdateItem(YObject *)), this, SLOT( ResumeTimeChanged(YObject *) ) );
	//connect( &resumetimeset	  , SIGNAL(NewItem(YObject *)), this, SLOT( ResumeTimeChanged(YObject *) ) );
	//connect( &resumetimeset	  , SIGNAL(ItemRemoved(YObject *)), this, SLOT( ResumeTimeChanged(YObject *) ) );
	connect( &resourceactionset, SIGNAL(UpdateItem(YObject *)), this, SLOT( ResourceActionChanged(YObject *) ) );
	//connect( &resourceactionset, SIGNAL(NewItem(YObject *)), this, SLOT( ResourceActionChanged(YObject *) ) );
	//connect( &resourceactionset, SIGNAL(ItemRemoved(YObject *)), this, SLOT( ResourceActionChanged(YObject *) ) );

	connect( &period			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &offset			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &requiredtime		, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &requiredtime		, SIGNAL( UpdateObject() ), this, SLOT( CalculateLaxity() ) );
	connect( &laxity			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &deadline			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &deadline			, SIGNAL( UpdateObject() ), this, SLOT( CalculateLaxity() ) );
	connect( &deadlinetolerance	, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &cpu				, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &tasktype			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &servertype		, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &priority			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &deadlinebehaviour	, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &function_init		, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &function_execute	, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &function_cleanup	, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTask::SubTaskChanged(YObject *object)
{
	ObjectUpdated();
	Q_UNUSED(object);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTask::ResumeTimeChanged(YObject *object)
{
	ObjectUpdated();
	Q_UNUSED(object);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTask::ResourceActionChanged(YObject *object)
{
	ObjectUpdated();
	Q_UNUSED(object);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read task specific configuration data from instance and set to stream
int YTask::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_DEADLINE)	)
		return parser.ReadContentObject(deadline);
	if ( parser.CheckSectionStart(YPC_REQUIRED_TIME) )
		return parser.ReadContentObject(requiredtime );
	if ( parser.CheckSectionStart(YPC_DEADLINE_TOLERANCE) )
		return parser.ReadContentObject(deadlinetolerance );
	if ( parser.CheckSectionStart(YPC_OFFSET) )
		return parser.ReadContentObject(offset);
	if ( parser.CheckSectionStart(YPC_PERIOD) )
		return parser.ReadContentObject(period );
	if ( parser.CheckSectionStart(YPC_TASK_TYPE) )
		return parser.ReadContentObject(tasktype);
	if ( parser.CheckSectionStart(YPC_SERVER_TYPE) )
		return parser.ReadContentObject(servertype);
	if ( parser.CheckSectionStart(YPC_CPU) )
		return parser.ReadContentObject(cpu);
	if ( parser.CheckSectionStart(YPC_PRIORITY) )
		return parser.ReadContentObject(priority);
	if ( parser.CheckSectionStart(YPC_DEADLINE_BEHAVIOUR) )
		return parser.ReadContentObject(deadlinebehaviour );
	if ( parser.CheckSectionStart(YPC_INIT)	)
		return parser.ReadContentObject(function_init);
	if ( parser.CheckSectionStart(YPC_EXECUTE) )
		return parser.ReadContentObject(function_execute);
	if ( parser.CheckSectionStart(YPC_CLEANUP) )
		return parser.ReadContentObject(function_cleanup);
	if ( parser.CheckSectionStart(YPC_RESUME_TIME) )
		return parser.NewSectionSet(YPC_RESUME_TIME, resumetimeset);
	if ( parser.CheckSectionStart(YPC_RESOURCE_TIME) )
		return parser.NewSectionSet(YPC_RESOURCE_TIME, resourceactionset);

	if ( parser.CheckSectionStart(YPC_TASK ) )
	{
		qWarning("YTask::SetConfig Subtasks not implemented!\n");
		return parser.NewSectionSet(YPC_TASK, subtaskset );
	}

	return YObject::SetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read task specific configuration data from stream and set to instance
int YTask::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	retvalue=parser.WriteContentObject(YPC_DEADLINE, deadline);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_REQUIRED_TIME, requiredtime);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_DEADLINE_TOLERANCE, deadlinetolerance);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_OFFSET, offset);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_PERIOD, period);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_TASK_TYPE, tasktype);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_SERVER_TYPE, servertype);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_CPU, cpu);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_PRIORITY, priority );
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_DEADLINE_BEHAVIOUR, deadlinebehaviour );
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_INIT, function_init );
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_EXECUTE, function_execute );
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_CLEANUP, function_cleanup );
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_RESUME_TIME, resumetimeset );
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_RESOURCE_TIME, resourceactionset );
		if ( retvalue )	return retvalue;

	return YObject::GetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTask* YTask::CreateSubTask(const YObjectArgs &i )
{
	return (YTask*) subtaskset.CreateNewItem(i.GetName() );
}






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResumeTime::SetTime(YASA_TIME t)
{
	if ( time.operator != (t) )
	{
		time=t;
//		SetName( time.GetVisibleString() );
//		((YResumeTimeSet*)GetSet())->sort();	//Resort(this);
		ObjectUpdated();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read task specific configuration data from instance and set to stream
int YResumeTime::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_TIME)	)
		return parser.ReadContentObject(time);
/*	{
		YASA_TIME	time;
		int retvalue=parser.ReadContentObject(time);
		if (!retvalue)
			SetTime(time);
		return retvalue;
	}
*/	//return YObject::SetConfig(parser);
	return parser.SectionEnd();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read task specific configuration data from stream and set to instance
int YResumeTime::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	retvalue=parser.WriteContentObject(YPC_TIME, time);
		if ( retvalue )	return retvalue;
	//return YObject::GetConfig(parser);
	return parser.SectionEnd();
}














//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read task specific configuration data from instance and set to stream
int YResourceAction::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_TIME)	)
		return parser.ReadContentObject(time);
/*	{
		YASA_TIME	newtime;
		int retvalue=parser.ReadContentObject(newtime);
		if (!retvalue)
			SetTime(newtime);
		return retvalue;
	}
*/	if ( parser.CheckSectionStart(YPC_ACTION) )
		return parser.ReadContentObject(action);
	if ( parser.CheckSectionStart(YPC_MUTEX) )
		return parser.ReadContentString(mutexname);
	return parser.SectionEnd();
//	return YObject::SetConfig(parser);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read task specific configuration data from stream and set to instance
int YResourceAction::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	retvalue=parser.WriteContentObject(YPC_TIME, time);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_ACTION, action);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentString(YPC_MUTEX, mutexname);
		if ( retvalue )	return retvalue;
	return parser.SectionEnd();
//	return YObject::GetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YResourceAction::GetMutex()
{
	if ( mutex )
		return mutex;
	YMutexSet	*mutexset=YResourceActionSet::GetMutexSet( GetSet() );
	if ( mutexset )
	{
		for (unsigned i=0; i<mutexset->GetCount(); i++)
		{
			if ( mutexset->At(i)->GetName() == mutexname )
			{
				mutex=mutexset->At(i);
				mutexname=mutex->GetName();
				connect( mutex, SIGNAL(UpdateObject(YObject *)), this, SLOT( MutexUpdated(YObject *) ) );
				connect( mutex, SIGNAL(destroyed()), this, SLOT( MutexDestroyed() ) );
				return mutex;
			}
		}
	}
	return 0;			// Object not found -> mutex undefined
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResourceAction::SetMutex(YObject *m)
{
	if ( mutex!=m )
	{
		if ( mutex )
		{
			disconnect( mutex, SIGNAL(UpdateObject(YObject *)), this, SLOT( MutexUpdated(YObject *) ) );
			disconnect( mutex, SIGNAL(destroyed()), this, SLOT( MutexDestroyed() ) );
		}
		if ( m )
		{
			mutexname=m->GetName();
			mutex=m;
			connect( mutex, SIGNAL(UpdateObject(YObject *)), this, SLOT( MutexUpdated(YObject *) ) );
			connect( mutex, SIGNAL(destroyed()), this, SLOT( MutexDestroyed() ) );
			ObjectUpdated();
		}
		else
		{
			MutexDestroyed();
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResourceAction::MutexUpdated(YObject *m)
{
	if ( m )
	{
		mutexname=m->GetName();
	}
	ObjectUpdated();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResourceAction::MutexDestroyed()
{
	mutexname=QString::null;
	mutex=0;
	ObjectUpdated();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YMutexSet* YResourceActionSet::GetMutexSet(YSet *set)
{
	YTask *task		= (YTask*) set->GetParent();
	if ( task )
	{
		YProject *project	= (YProject*) task->GetSet()->GetParent();
		if ( project )
		{
			YMutexSet &mutexset	= project->GetMutexSet();
			if ( &mutexset )
			{
				return &mutexset;
			}
		}
	}
	return 0;
}