//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA task window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/taskwindow.h,v 1.4 2003/01/24 15:47:43 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about task windows
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_TASKWINDOW_INCLUDE
#define	YASAGUI_TASKWINDOW_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qtextbrowser.h>
#include "mainwindow.h"
#include "widgets.h"
#include "task.h"
#include "project.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the task configuration
class YTaskFrame : public YObjectFrame
{	Q_OBJECT
	QScrollView			scrollview;
public:					YTaskFrame(QWidget *parent, YFrameArgs &args) :
						YObjectFrame(args),
						scrollview(parent)
						{}
	virtual int			InitInstance();
	virtual QWidget*	GetMainWidget()								{ return &scrollview;	}
	static YFrame*		CreateTaskFrame( QWidget *parent, YFrameArgs &args)
						{
							return new YTaskFrame(parent, args);
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPriorityWidget : public YIntegerWidget
{	Q_OBJECT
public:					YPriorityWidget(YTask *task, QWidget *parent) : YIntegerWidget(task, parent)
						{	setMinValue(YG_MIN_PRIORITY);
							setMaxValue(YG_MAX_PRIORITY);
							UpdateWidget();
							updateDisplay();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( value() != ((YTask*)object)->GetPriority() )
																	setValue( ((YTask*)object)->GetPriority() );	}
	virtual void		UpdateValue(int value)					{ ((YTask*)object)->SetPriority( value );			}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v==YG_MIN_PRIORITY) return QString(MSGP_TASK_PRIORITY_LOWEST);
																	if ( v==YG_MAX_PRIORITY) return QString(MSGP_TASK_PRIORITY_HIGHEST);
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_TASK_PRIORITY_LOWEST )	return YG_MIN_PRIORITY;
																	if ( text()==MSGP_TASK_PRIORITY_HIGHEST )	return YG_MAX_PRIORITY;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! On which CPU this task should run
class YCPUTaskWidget : public YIntegerWidget
{	Q_OBJECT
public:
	YCPUTaskWidget(YTask *t, QWidget *parent) : YIntegerWidget(t, parent)
						{	setMinValue( t->GetCPU().GetMinValue() );
							UpdateWidget();
							updateDisplay();
						}
public slots:
	virtual void		UpdateWidget()							{ setMaxValue( ((YProject*)object->GetSet()->GetParent())->GetMaxCPU().GetContent() );
																  if ( value()!=((YTask*)object)->GetCPU().GetContent() )
																	setValue( ((YTask*)object)->GetCPU().GetContent() );	}
	virtual void		UpdateValue(int value)					{ ((YTask*)object)->SetCPU( value );				}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v==YG_MIN_CPU)	return QString( MSGP_TASK_CPU_ANY );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_TASK_CPU_ANY )	return YG_MIN_CPU;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPeriodWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YPeriodWidget(YTask *task, QWidget *parent) : YYasaTimeWidget(task, parent)
						{	UpdateWidget();
							updateDisplay();	}
public slots:
	virtual void		UpdateWidget()							{ if ( GetYasaTime()!=((YTask*)object)->GetPeriod() ) //.GetContent() )
																	SetYasaTime( ((YTask*)object)->GetPeriod());	}
	virtual void		UpdateValue(YASA_TIME value)			{ ((YTask*)object)->SetPeriod( value );			}
	virtual QString		mapValueToText(int v)					{	if ( v==YTIME::YASA_TIME_MIN)	return QString( MSGP_YASA_TIME_INFINITY );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_YASA_TIME_INFINITY )	return YTIME::YASA_TIME_MIN;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YOffsetWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YOffsetWidget(YTask *task, QWidget *parent) : YYasaTimeWidget(task, parent)
						{	UpdateWidget();
							updateDisplay();	}
public slots:
	virtual void		UpdateWidget()							{ if ( GetYasaTime()!=((YTask*)object)->GetOffset() )
																	SetYasaTime( ((YTask*)object)->GetOffset());	}
	virtual void		UpdateValue(YASA_TIME value)			{ ((YTask*)object)->SetOffset( value );			}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v==YTIME::YASA_TIME_MIN)	return QString( MSGP_TASK_OFFSET_NOOFFSET );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_TASK_OFFSET_NOOFFSET )	return YTIME::YASA_TIME_MIN;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YRequiredTimeWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YRequiredTimeWidget(YTask *task, QWidget *parent) : YYasaTimeWidget(task, parent)
						{	//setMinValue(YASA_REQUIREDTIME::YASA_MIN_REQUIRED_TIME);
							UpdateWidget();
							updateDisplay();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( GetYasaTime()!=((YTask*)object)->GetRequiredTime() )
																	SetYasaTime( ((YTask*)object)->GetRequiredTime());	}
	virtual void		UpdateValue(YASA_TIME value)					{ ((YTask*)object)->SetRequiredTime( value );			}
	virtual QString		mapValueToText(int v)					{	if ( v==0)	return QString( MSGP_YASA_TIME_INFINITY );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_YASA_TIME_INFINITY )	return 0;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YDeadlineWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YDeadlineWidget(YTask *task, QWidget *parent) : YYasaTimeWidget(task, parent)
						{	UpdateWidget();
 							updateDisplay();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( GetYasaTime()!=((YTask*)object)->GetDeadline() )
																	SetYasaTime( ((YTask*)object)->GetDeadline());	}
	virtual void		UpdateValue(YASA_TIME value)			{ ((YTask*)object)->SetDeadline( value );		}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v<=0)	return QString( MSGP_YASA_TIME_INFINITY );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_YASA_TIME_INFINITY )	return -1;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YDeadlineToleranceWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YDeadlineToleranceWidget(YTask *task, QWidget *parent) : YYasaTimeWidget(task, parent)
						{	UpdateWidget();
							updateDisplay();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( GetYasaTime()!=((YTask*)object)->GetDeadlineTolerance() )
																	SetYasaTime( ((YTask*)object)->GetDeadlineTolerance());	}
	virtual void		UpdateValue(YASA_TIME value)			{ ((YTask*)object)->SetDeadlineTolerance( value );		}
	virtual QString		mapValueToText(int v)					{	if ( v<=0)	return QString(MSGP_TASK_DEADLINE_TOLERANCE_NO);
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_TASK_DEADLINE_TOLERANCE_NO)	return 0;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTaskTypeWidget : public YComboWidget
{	Q_OBJECT
public:					YTaskTypeWidget(YTask *task, QWidget *parent) : YComboWidget(task, parent)
						{	insertItem(MSGP_TASKTYPE_NONPERIODIC);
							insertItem(MSGP_TASKTYPE_PERIODIC);
							insertItem(MSGP_TASKTYPE_APERIODIC);
#if YASA_THREAD_MAXTYPE!=2
#error Insert/Remove new item to combobox here !!!
#endif
							UpdateWidget();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( currentItem()!=((YTask*)object)->GetTaskType() )
																	setCurrentItem( ((YTask*)object)->GetTaskType());	}
	virtual void		UpdateValue(int value)					{ ((YTask*)object)->SetTaskType( value );		}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YDeadlineBehaviourWidget : YComboWidget
{	Q_OBJECT
public:					YDeadlineBehaviourWidget(YTask *task, QWidget *parent) : YComboWidget(task, parent)
						{	insertItem(MSGP_DLBEHAVIOUR_IGNORE);
							insertItem(MSGP_DLBEHAVIOUR_CANCEL_TASK);
							insertItem(MSGP_DLBEHAVIOUR_SKIP);
							insertItem(MSGP_DLBEHAVIOUR_EMERGENCY_TASK);
#if YASA_DL_MAX!=3
#error Insert/Remove new item to combobox here !!!
#endif
							UpdateWidget();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( currentItem()!=((YTask*)object)->GetDeadlineBehaviour() )
																	setCurrentItem( ((YTask*)object)->GetDeadlineBehaviour());	}
	virtual void		UpdateValue(int value)					{ ((YTask*)object)->SetDeadlineBehaviour( value );		}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YInitFunctionWidget : public YStringWidget
{	Q_OBJECT
public:					YInitFunctionWidget(YTask *task, QWidget *parent) : YStringWidget(task, parent)	{	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()							{ if ( ((YTask*)object)->GetInitFunction() != text() )
																	setText( ((YTask*)object)->GetInitFunction().GetContent() );		}
	virtual void		UpdateValue( const QString &t)			{ ((YTask*)object)->SetInitFunction( text() );
																	Q_CONSTUNUSED(t);									}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YExecuteFunctionWidget : public YStringWidget
{	Q_OBJECT
public:					YExecuteFunctionWidget(YTask *task, QWidget *parent) : YStringWidget(task, parent)	{	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()							{ if ( ((YTask*)object)->GetExecuteFunction() != text() )
																	setText( ((YTask*)object)->GetExecuteFunction().GetContent() );		}
	virtual void		UpdateValue( const QString &t)			{ ((YTask*)object)->SetExecuteFunction( text() );
																	Q_CONSTUNUSED(t);										}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YCleanupFunctionWidget : public YStringWidget
{	Q_OBJECT
public:					YCleanupFunctionWidget(YTask *task, QWidget *parent) : YStringWidget(task, parent)	{	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()							{ if ( ((YTask*)object)->GetCleanupFunction() != text())
																	setText( ((YTask*)object)->GetCleanupFunction().GetContent() );		}
	virtual void		UpdateValue( const QString &t)			{ ((YTask*)object)->SetCleanupFunction( text() );
																	Q_CONSTUNUSED(t);										}
};





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YListViewGroup : public QGroupBox, public YWidget
{	Q_OBJECT
protected:
	YListView			*listview;
	QHBox				*hbox;
	QPushButton			*add;
	QPushButton			*del;
	QPushButton			*deleteall;
public:					YListViewGroup(YSet *set, QWidget *parent, const QString &title) :
						QGroupBox(1,Horizontal,title, parent), YWidget(set),
						listview(0),
						hbox(0),
						add(0),
						del(0),
						deleteall(0)
						{
							setAlignment(AlignHCenter);
						}
	virtual int			InitInstance()							{ return 0;			}
	virtual QWidget*	GetWidget()								{ return this;		}
	void				InsertListView(YListView *l);
	int					CreateWidgets(YSet *set, QWidget *parent);
public slots:
	// Slots for buttons
	YObject*			AddEntry();
	void				RemoveAll();
	void				DeleteEntry();
	// Slots for listview
	virtual void		UpdateWidgets(QListViewItem *currentitem);
	virtual void		ListViewEmpty()							{ UpdateWidgets(0);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResumeTimeListViewItem : public YListViewItem
{	Q_OBJECT
public:					YResumeTimeListViewItem(YObject *o, QListView *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{ }
						YResumeTimeListViewItem(YObject *o, QListViewItem *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{ }
protected:
	QString				key(int column, bool ascending) const
						{
							if ( column==0)
							{
								return YString::Justify( ((YResumeTime*)object)->GetTime().GetVisibleContentString() );
							}
							return text(column);
							Q_UNUSED(ascending);
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResumeTimeListView : public YListView
{	Q_OBJECT
public:					YResumeTimeListView(YResumeTimeSet *s, QWidget *parent, const QString &title) :
						YListView(s,parent,title)
						{}
	virtual	YListViewItem* CreateNewItem(QListViewItem *parent, YObject *object, QListViewItem *after=0)
						{
							if ( parent )
								return new YResumeTimeListViewItem(object,parent,after);
							else
								return new YResumeTimeListViewItem(object,this,after);
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResumeTimeWidget : public YListViewGroup
{	Q_OBJECT
	QSpinBox			*edittime;
public:					YResumeTimeWidget(YResumeTimeSet *set, QWidget *parent) :
						YListViewGroup(set, parent, MSGP_TASK_RESUME_TIMES),
						edittime(0)
						{ }
	virtual int			InitInstance();
public slots:
	virtual void		UpdateTime(int t);							//! time in "edit spinbox" was changed
	virtual YObject*	AddEntry();									//! Button "Add" pressed
	virtual void		UpdateWidgets(QListViewItem *item);			//! CurrentItem in Listview changed
};









//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResourceActionListViewItem : public YListViewItem
{	Q_OBJECT
public:
	public:				YResourceActionListViewItem(YObject *o, QListView *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{
							UpdateObject(o);					// fill all columns
						}
						YResourceActionListViewItem(YObject *o, QListViewItem *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{
							UpdateObject(o);					// fill all columns
						}
	YObject*			GetMutexObject()
						{
							return ((YResourceAction*)object)->GetMutex();
						}
public slots:
	virtual void		UpdateObject(YObject *);
protected:
	//! This function is used to sort the columns
	QString				key(int column, bool ascending) const
						{
							switch ( column )
							{
							// column time
							case 0:	{	return YString::Justify( ((YResourceAction*)object)->GetTime().GetVisibleContentString() );
									}
							// action time
							default:return text(column);
							}
							Q_UNUSED(ascending);
						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResourceActionListView : public YListView
{	Q_OBJECT
public:					YResourceActionListView(YResourceActionSet *s, QWidget *parent, const QString &title) :
						YListView(s,parent,title)
						{}
	virtual	YListViewItem* CreateNewItem(QListViewItem *parent, YObject *object, QListViewItem *after=0)
						{
							if ( parent )
								return new YResourceActionListViewItem(object,parent,after);
							else
								return new YResourceActionListViewItem(object,this,after);
						}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResourceActionWidget : public YListViewGroup
{	Q_OBJECT
	QSpinBox			*edittime;
	QComboBox			*actioncombo;
	QComboBox			*mutexcombo;
	YStringList			*mutexstringlist;
public:					YResourceActionWidget(YResourceActionSet *set, QWidget *parent) :
						YListViewGroup(set, parent, MSGP_TASK_RESOURCE_TIMES),
						edittime(0),
						actioncombo(0),
						mutexcombo(0),
						mutexstringlist(0)
						{}
	virtual int			InitInstance();
public slots:
	// time in edit line was changed
	virtual void		TimeChanged(int t)
						{
							YListViewItem	*item=listview->GetCurrentItem();
							if ( item )
								((YResourceAction*)item->GetObject())->SetTime(t);
						}
	// mutex id was changed
	virtual void		MutexChanged(int t)
						{
							YListViewItem	*item=listview->GetCurrentItem();
							if ( item )
								((YResourceAction*)item->GetObject())->SetMutex( mutexstringlist->FindObject(t) );
						}
	// action type was changed
	virtual void		ActionChanged(int t)
						{
							YListViewItem	*item=listview->GetCurrentItem();
							if ( item )
								((YResourceAction*)item->GetObject())->SetAction(t);
						}
	// Button "Add" pressed
	virtual YObject*	AddEntry();
	// CurrentItem changed or removed
	virtual void		UpdateWidgets(QListViewItem *item)
						{
							edittime->setEnabled( item );
							actioncombo->setEnabled( item );
							mutexcombo->setEnabled( item );
							if ( item )
							{
								edittime->setValue( ((YResourceAction*)((YResourceActionListViewItem*)item)->GetObject())->GetTime() );
								actioncombo->setCurrentItem( ((YResourceAction*)((YResourceActionListViewItem*)item)->GetObject())->GetAction() );
								mutexcombo->setCurrentItem( mutexstringlist->FindIndex(
									((YResourceActionListViewItem*)item)->GetMutexObject())
									);
							}
							//else
							//	edittime->setValue(0);
							YListViewGroup::UpdateWidgets(item);
						}
	// Stringlist(Mutex members) was changed
	void				MutexStringListChanged()
						{
							mutexcombo->clear();
							mutexcombo->insertStringList(*mutexstringlist);
							UpdateWidgets( listview->GetCurrentItem() );
						}
};





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_TASKWINDOW_INCLUDE
