//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA timing parser
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/timingparser.h,v 1.4 2003/01/24 15:47:43 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about timing parser which are used to parse the logfile list
// to extract the needed information
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_TIMING_PARSER_INCLUDE
#define	YASAGUI_TIMING_PARSER_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "timingsettings.h"
#include "timingpainter.h"
#include "logfile.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTimings;
class YTimingHandler;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTimingParser : public YLoggingParser
{	YPainter			*painter;
	YTimings			*timings;
public:					YTimingParser() :
						painter(0),
						timings(0)
						{}
	void				SetPainter(YPainter *p)							{	painter=p;							}
	YPainter*			GetPainter() const								{	return painter;						}
	void				SetTimings(YTimings *t)							{	timings=t;							}
	//! During the parsing process the Function "YTimingParser::EvaluateLoggingEntry()"
	//! is called to filter and draw the contents using YPainter::DrawLoggingAction()
	void				EvaluateLoggingEntry(YLogAction *action);
	//! Override this function to filter the needed logging actions
	virtual bool		TimingFilter(const YLogAction *) const; //			{	return true;						}
	//! Should there be printed a duration time between start and end of a range ?
	//! Example: "T1: [  200  ]" instead "T1: [      ]"
//	virtual bool		PrintDurationAllowed(const YLogAction *) const	{	return false;						}
	//! This function is called every time a new timing diagram is drawn to enable/disable
	//! some features, e.g. drawtaskrange=true
	virtual void		PrepareLogfilePainting();
	//! Returns the row number of this entry
	virtual int			GetTimingRow(YLogAction *action) const;
	//! Prepare the painting of one logfile widget
};






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class filters only task important log actions. It acts as a base class for task parsers
class YTaskTimingParser : public YTimingParser
{public:
//	virtual bool		PrintDurationAllowed(const YLogAction *) const	{	return false;						}
	virtual void		PrepareLogfilePainting();
};
//! This parser is used in project timing diagrams using more than one environments
class YTaskProjectTimingParser : public YTaskTimingParser
{};
//! This class is a parser used in environment timing diagrams
class YTaskEnvironmentTimingParser : public YTaskTimingParser
{public: virtual int	GetTimingRow(YLogAction *action) const			{	return action->GetThread()->GetNumber();	}
};






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class filters only mutex important log actions. It acts as a base class for mutex parsers
class YMutexTimingParser : public YTimingParser
{public:
	virtual bool		TimingFilter(const YLogAction *action) const	{	if (action->GetAction() >=YASAL_MUTEX_CREATED &&
																				action->GetAction() <=YASAL_MUTEX_PRIORITY_CHANGED )
																				return YTimingParser::TimingFilter(action);
																			return false;						}
//	virtual bool		PrintDurationAllowed(const YLogAction *) const	{	return false;						}
	virtual void		PrepareLogfilePainting();
};
//! This parser is used in project timing diagrams using more than one environments
class YMutexProjectTimingParser : public YMutexTimingParser
{};
//! This class is a parser used in environment timing diagrams
class YMutexEnvironmentTimingParser : public YMutexTimingParser
{public:virtual int		GetTimingRow(YLogAction *action) const			{	if ( action->GetMutex() )
																				return action->GetMutex()->GetNumber();
																			return 0;							}
};







//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class filters only cpu important log actions. It acts as a base class for cpu parsers
class YCPUTimingParser : public YTimingParser
{public:virtual bool	TimingFilter(const YLogAction *action) const	{	if ( action->GetAction() == YASAL_THREAD_EXCLUDED ||
																				 action->GetAction() == YASAL_THREAD_EXIT )
																				return false;
																			return YTimingParser::TimingFilter(action);	}
//	virtual bool		PrintDurationAllowed(const YLogAction *) 		{	return false;						}
	virtual void		PrepareLogfilePainting();
};
//! This parser is used in project timing diagrams using more than one environments
class YCPUProjectTimingParser : public YCPUTimingParser
{};
//! This class is a parser used in environment timing diagrams
class YCPUEnvironmentTimingParser : public YCPUTimingParser
{public:virtual int		GetTimingRow(YLogAction *) const				{	return GetCPULogFile()->GetCPU();	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_TIMING_PARSER_INCLUDE
