//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA widgets support code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/widgets.cpp,v 1.3 2003/01/23 09:18:40 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "widgets.h"
#include "list.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The contents of the object are changed (Update title now)
void YListViewItem::UpdateObject(YObject *o)
{
	Q_UNUSED(o);
	if ( object->GetName() != text(0) )
	{
		setText( 0, object->GetName() );
		((YListView*)listView())->UpdateList(object);
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YListView::YListView(YSet *s, QWidget *parent, const QString &title) :
QListView(parent, title),
set(s)
{
	if ( ! title.isEmpty() )
		addColumn(title);
	setShowSortIndicator(true);
	setSorting(0,true);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Initializes the listview and adds all items from the YList to the listview
int YListView::InitInstance()
{
	if ( set )
	{
		connect( (YObject*)set, SIGNAL(UpdateObject(YObject *)), this, SLOT(UpdateList(YObject *) ) );
		connect( (YObject*)set, SIGNAL(NewItem(YObject *)), this, SLOT(NewChildObject(YObject *) ) );
		connect( (YObject*)set, SIGNAL(ItemRemoved(YObject *)), this, SLOT(ChildObjectRemoved(YObject *) ) );
	}
	return CreateSetItems();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YListView::CreateSetItems()
{
	QListViewItem	*item=0;
	clear();
	if ( set )
	{
		for (unsigned int i=0; i<set->GetCount(); i++)
		{
			item=CreateNewItem(0, set->At(i), item);
		}
	}
	return 0;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YListViewItem* YListView::CreateNewItem(QListViewItem *parent, YObject *object, QListViewItem *after)
{
	if ( parent )
		return new YListViewItem(object,parent,after);
	else
		return new YListViewItem(object, this, after);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new object was inserted. A new new child item must be created(SLOT)
void YListView::NewChildObject(YObject *object)
{
	YListViewItem	*item=CreateNewItem(0,object);
	if ( item )
	{
		setSelected(item,true);
	}

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Set the new item to the selected in the listview
void YListView::SetSelectedItem(YListViewItem *item)
{
	if ( item )
		setSelected(item, true);
	else
	{
		QListViewItem	*nextcurrent=0;
		if ( !nextcurrent && item->itemBelow() )
			nextcurrent=item->itemBelow();
		if ( !nextcurrent && item->itemAbove() )
			nextcurrent=item->itemAbove();
		if ( !nextcurrent && item->QListViewItem::parent() )
			nextcurrent=item->QListViewItem::parent();
		setSelected(nextcurrent, true);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The associated item must be removed from list due to destroying the specified object
void YListView::ChildObjectRemoved(YObject *object)
{
	YListViewItem *item=FindItemFromObject(object);

	if ( item )
	{
		QListViewItem	*nextcurrent=0;
		if ( !nextcurrent && item->itemBelow() )
			nextcurrent=item->itemBelow();
		if ( !nextcurrent && item->itemAbove() )
			nextcurrent=item->itemAbove();
		if ( !nextcurrent && item->QListViewItem::parent() )
			nextcurrent=item->QListViewItem::parent();
		if ( nextcurrent )
			setSelected(nextcurrent, true);
		else
			emit ListViewEmpty();
		delete item;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YListViewItem* YListView::_FindItemRecursive(YListViewItem *item, YObject *o)
{
	if ( item && item->nextSibling() )
	{
		YListViewItem*	item2;
		item2=_FindItemRecursive( (YListViewItem*)item->nextSibling(), o);
		if ( item2 )
			return item2;
	}

	while (item)
	{
		if ( item->GetObject() == o )
			return item;
		item=(YListViewItem*)item->itemBelow();
	};
	return 0;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YStringList::FindObject(unsigned int index)
{
	if ( index < objectlist.count() )
		return objectlist.at(index);
	return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YStringList::FindObject(const QString &string)
{
	for (unsigned int i=0; i<objectlist.count(); i++)
	{
		if ( objectlist.at(i) && objectlist.at(i)->GetName() == string )
			return objectlist.at(i);
	}
	return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned int YStringList::FindIndex(YObject *object)
{
	for (unsigned int i=0; i<objectlist.count(); i++)
	{
		if ( objectlist.at(i) == object )
			return i;
	}
	return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YStringList::ClearList()
{
	while ( objectlist.count() )
	{
		ItemRemoved( objectlist.at(0) );
	};
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YStringList::UpdateList(YObject *changedobject)
{
	Q_UNUSED(changedobject);	//!!!
	ClearList();
	for (unsigned int i=0; i<set->GetCount(); i++)
	{
		NewItem( set->At(i) );
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YStringList::UpdateItem(YObject *changedobject)
{
	int	i;
	i=FindIndex(changedobject);
	if ( i )
	{
		if ( changedobject->GetName() != QStringList::operator [](i) )
		{
			QStringList::operator [](i)=changedobject->GetName();
			emit StringListChanged();
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Adds an item to the stringlist at position p. Only if o==0 name is used as item string (!). These items can not
//! be removed during runtime
void YStringList::AddItem(YObject *newobject, const QString &name, bool appenditem)
{
	if (newobject)
	{
		if (appenditem)
			append( newobject->GetName() );
		else
			prepend( newobject->GetName() );
	}
	else
	{
		if (appenditem)
			append( name );
		else
			prepend( name );
	}
	if (appenditem)
		objectlist.append( newobject);
	else
		objectlist.prepend( newobject);
	if ( makeconnections && newobject )
	{
		connect( newobject, SIGNAL(UpdateObject(YObject *)), this, SLOT( UpdateItem(YObject *) ) );
	}
	emit StringListChanged();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YStringList::ItemRemoved(YObject *object)
{
	int	i;
	i=FindIndex(object);
	if ( i )
	{
		if ( makeconnections )
		{
			disconnect( object, SIGNAL(UpdateObject(YObject *)), this, SLOT( UpdateItem(YObject *) ) );
		}
		objectlist.removeRef( object );
		remove( object->GetName() );
		emit StringListChanged();
	}
}
