//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA widgets declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/widgets.h,v 1.4 2003/01/24 15:47:44 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all base widgets classes of YASAGUI
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_WIDGETS_INCLUDE
#define	YASAGUI_WIDGETS_INCLUDE

#include <qspinbox.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qmultilineedit.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qdatetime.h>
#if QT_VERSION>=300
#include <qdatetimeedit.h>
#endif
#include "datatypes.h"



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YListViewItem : public QObject, public QListViewItem
{
	Q_OBJECT
protected:
	YObject				*object;
public:
						YListViewItem(YObject *o, QListView *parent, QListViewItem *after=0) :
						QListViewItem(parent,after), object(o)
						{
							o->UpdateArgConnect(this, SLOT( UpdateObject(YObject *) ) );
							UpdateObject(o);
						}
						YListViewItem(YObject *o, QListViewItem *parent, QListViewItem *after=0) :
						QListViewItem(parent, after, o->GetName() ), object(o)
						{
							o->UpdateArgConnect(this, SLOT( UpdateObject(YObject *) ) );
							//connect( o, SIGNAL(UpdateObject(YObject *)), this, SLOT( UpdateObject(YObject *) ) );
							UpdateObject(o);
						}
	virtual void		Activate()								// toggle mode open/closed "+"
						{
							/*if ( isOpen() )
								setOpen(FALSE);
							else
								setOpen(TRUE);*/
						}
	virtual int			InitInstance()
						{
							EnableInlineRename();
							return 0;
						}
	YObject*			GetObject()	const				{ return object; }
	void				DisableInlineRename()
						{
#if QT_VERSION>=300
							setRenameEnabled(0, FALSE);
#endif
						}
	void				EnableInlineRename()
						{
#if QT_VERSION>=300
							setRenameEnabled(0, TRUE);
#endif
						}
public slots:
	virtual void		UpdateObject(YObject *);					//! The contents of the object are changed (Update title now)
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YListView : public QListView
{	Q_OBJECT
private:
	YListViewItem*		_FindItemRecursive(YListViewItem *item, YObject *o);
protected:
	YSet				*set;
public:					YListView(YSet *s, QWidget *parent, const QString &title);
	YListViewItem*		FindItemFromObject(YObject *object)				{ return _FindItemRecursive( (YListViewItem*)firstChild(), object); }
	virtual	YListViewItem* CreateNewItem(QListViewItem *parent, YObject *object, QListViewItem *after=0);
	YSet*				GetSet()									{ return set;	}
	YListViewItem*		GetCurrentItem() const						{ return (YListViewItem*)currentItem();	}
	void				SetSelectedItem(YListViewItem *item=0);
	virtual int			InitInstance();
public slots:
	virtual void		NewChildObject(YObject *object);			//! A new child object must be inserted
	virtual void		ChildObjectRemoved(YObject *object);		//! The associated item must be removed from list
	virtual int			CreateSetItems();							//! Create listview item again/new
	virtual void		UpdateList(YObject *object)					//! list changed (maybe name of set)
						{
							sort();
							Q_UNUSED(object);
						}
signals:
	void				ListViewEmpty();							//! The listview is now empty
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YWidget
{
protected:
	YObject				*object;
public:					YWidget(YObject *o) : object(o)
						{}
	virtual void		UpdateWidget() {}
	virtual QWidget*	GetWidget() =0;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YCheckBoxWidget : public QCheckBox, public YWidget
{	Q_OBJECT
public:					YCheckBoxWidget(YObject *object, QWidget *parent) : QCheckBox(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject(YObject *)), this, SLOT(UpdateWidget(YObject *) ) );
							connect( this, SIGNAL(toggled(bool)), this, SLOT(UpdateValue(bool) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;		}
public slots:
	virtual void		UpdateWidget(YObject *o)				{ Q_UNUSED(o);		}
	virtual void		UpdateValue(bool state)					{ Q_UNUSED(state);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YIntegerWidget : public QSpinBox, public YWidget
{	Q_OBJECT
public:					YIntegerWidget(YObject *object, QWidget *parent) : QSpinBox(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(valueChanged(int)), this, SLOT(UpdateValue(int) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;		}
public slots:
	virtual void		UpdateWidget()							{ }
	virtual void		UpdateValue(int value)					{ Q_UNUSED(value);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YComboWidget : public QComboBox, public YWidget
{	Q_OBJECT
public:					YComboWidget(YObject *object, QWidget *parent) : QComboBox(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(activated(int)), this, SLOT(UpdateValue(int) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;		}
public slots:
	virtual void		UpdateWidget()							{					}
	virtual void		UpdateValue(int value)					{ Q_UNUSED(value);	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTextComboWidget : public QComboBox, public YWidget
{	Q_OBJECT
public:					YTextComboWidget(YObject *object, QWidget *parent) : QComboBox(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(activated(const QString &)), this, SLOT(UpdateValue(const QString &) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;		}
public slots:
	virtual void		UpdateWidget()							{ }
	virtual void		UpdateValue(const QString &value)		{ Q_CONSTUNUSED(value);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStringWidget : public QLineEdit, public YWidget
{	Q_OBJECT
public:					YStringWidget(YObject *object, QWidget *parent) : QLineEdit(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(textChanged(const QString &)), this, SLOT(UpdateValue( const QString &) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;		}
public slots:
	virtual void		UpdateWidget()							{ }
	virtual void		UpdateValue( const QString &t)			{ (const void)(t);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTextWidget : public QMultiLineEdit, public YWidget
{	Q_OBJECT
public:					YTextWidget(YObject *object, QWidget *parent) : QMultiLineEdit(parent), YWidget(object)
						{	setWordWrap(WidgetWidth);
							connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(textChanged()), this, SLOT(UpdateValue() ) );
						}
	virtual QWidget*	GetWidget()								{ return this;	}
public slots:
	virtual void		UpdateWidget()							{ }
	virtual void		UpdateValue()							{				}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YYasaTimeWidget : public QSpinBox, public YWidget
{	Q_OBJECT
public:					YYasaTimeWidget(YObject *object, QWidget *parent) : QSpinBox(parent), YWidget(object)
						{	setMinValue(0);
							setMaxValue(YTIME::YASA_TIME_INT_MAX);
							if ( object )
								connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(valueChanged(int)), this, SLOT(UpdateIntValue(int) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;				}
	void				SetYasaTime(YASA_TIME t)				{ setValue(t);				}	//! used from derived classes
	YASA_TIME 			GetYasaTime() const						{ return value();			}	//! used from derived classes
	void				SetReadOnly(bool b)						{ editor()->setReadOnly(b);
#if QT_VERSION<300
																  upButton()->setEnabled(!b);
																  downButton()->setEnabled(!b);
#endif
																}
public slots:
	virtual void		UpdateWidget()							{							}
	virtual void		UpdateIntValue(int value)				{ UpdateValue(value);		}	//! convert value to YASA_TIME
	virtual void		UpdateValue(YASA_TIME value)			{ Q_UNUSED(value);			}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This is the QT3 version bug there is a bug in "moc". It does not resolve the right class
/*
#if QT_VERSION>=300
class YDateTimeWidget : public QDateTimeEdit, public YWidget
{	Q_OBJECT
public:					YDateTimeWidget(YObject *object, QWidget *parent) : QDateEdit(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							connect( this, SIGNAL(valueChanged(const QDateTime &)), this, SLOT(UpdateDate(const QDate &) ) );
						}
	virtual QWidget*	GetWidget()								{ return this;		}
	void				SetDateTime(const QDateTime &t)
						{
							setDate( QDate( t.date() ) );
							setTime( QTime(t.time() ) );
						}
public slots:
	virtual void		UpdateWidget()							{ }
	virtual void		UpdateDate(const QDateTime &date)		{ Q_UNUSED(date);	}
};
#else
*/
class YDateTimeWidget : public QLineEdit, public YWidget
{	Q_OBJECT
	QDateTime			datetime;
public:					YDateTimeWidget(YObject *object, QWidget *parent) : QLineEdit(parent), YWidget(object)
						{	connect( object, SIGNAL(UpdateObject()), this, SLOT(UpdateWidget() ) );
							setReadOnly(true);
						}
	virtual QWidget*	GetWidget()								{ return this;		}
	void				SetDateTime(const QDateTime &t)
						{
							datetime=t;
							setText(MSGP_NOT_AVAILABLE_BEFORE_QT3);
							//setDate( QDate( t.date() ) );
							//setTime( QTime(t.time() ) );
						}
	QDateTime			GetDateTime()
						{
							return datetime;
							//return addDays(0);
						}
public slots:
	virtual void		UpdateWidget()							{						}
	virtual void		UpdateDate(const QDateTime &date)		{ Q_CONSTUNUSED(date);	}
};
//#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YNameWidget : public YStringWidget
{	Q_OBJECT
public:					YNameWidget(YObject *object, QWidget *parent) : YStringWidget(object, parent)
						{	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()							{ if ( text()!=object->GetName())
																  setText(object->GetName() );	}
	virtual void		UpdateValue( const QString &t)			{ object->SetName( text() );
																	Q_CONSTUNUSED(t);			}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YDescriptionWidget : public YTextWidget
{	Q_OBJECT
public:					YDescriptionWidget(YObject *object, QWidget *parent) : YTextWidget(object,parent)
						{	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()							{ if ( text()!=object->GetDescription())
																	setText(object->GetDescription() );	}
	virtual void		UpdateValue()							{ object->SetDescription( text() );		}
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStringList : public QObject, public QStringList
{	Q_OBJECT
	YSet			*set;
	bool			makeconnections;	//! connect signals to every YObject
	QList<YObject>	objectlist;			//! maybe original YList can change the order so we store the objects ptr too
public:				YStringList(YSet *s, bool mc=false) :
					set(s),
					makeconnections(mc)
					{
						connect( (YObject*)set, SIGNAL(NewItem(YObject *)), this, SLOT( NewItem(YObject *) ) );
						connect( (YObject*)set, SIGNAL(ItemRemoved(YObject *)), this, SLOT( ItemRemoved(YObject *) ) );
						UpdateList(0);
					}
	void			ClearList();
	unsigned int	FindIndex(YObject *object);
	YObject*		FindObject(unsigned int index);
	YObject*		FindObject(const QString &s);
	void			AddItem(YObject *newobject, const QString &name=QString::null, bool appenditem=true);
public slots:
	virtual void	UpdateList(YObject *);
	void			UpdateItem(YObject *);
	void			NewItem(YObject *o)							{ AddItem(o);	}
	void			ItemRemoved(YObject *object);
signals:
	void			StringListChanged();
};





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_WIDGETS_INCLUDE
